@@ -5,7 +5,8 @@
 #include <agar/core/begin.h>
 
 typedef struct ag_cpuinfo {
-	const char *_Nonnull arch;	/* Architecture name */
+	const char *_Nonnull arch;	/* CPU architecture name */
+	const char *_Nonnull syst;	/* System name (or "") */
 	char vendorID[13];		/* CPU Vendor ID string */
 	char     _pad1[3];
 
@@ -37,9 +38,11 @@ typedef struct ag_cpuinfo {
 #define AG_EXT_VMX		0x00800000 /* Virtual Machine extensions */
 #define AG_EXT_SSSE3		0x01000000 /* SSSE3 Extensions */
 #define AG_EXT_SSE41		0x02000000 /* SSE4.1 extensions */
-#define AG_EXT_SSE42		0x04000000 /* SSE4.1 extensions */
+#define AG_EXT_SSE42		0x04000000 /* SSE4.2 extensions */
 
-	Uint32 _pad2;
+	/* TODO: AVX */
+
+	Uint32 icon;                         /* Graphical Icon (Unicode) */
 } AG_CPUInfo;
 
 /* Begin generated block */
