@@ -30,7 +30,7 @@
 
 # define AG_PTR(v) ((v <= event->argc && event->argv[v].type == AG_VARIABLE_POINTER && !(event->argv[v].info.pFlags & AG_VARIABLE_P_READONLY)) ? event->argv[v].data.p : AG_PtrMismatch())
 
-# define AG_CONST_PTR(v) ((v <= event->argc && event->argv[v].type == AG_VARIABLE_POINTER && (event->argv[v].info.pFlags & AG_VARIABLE_P_READONLY)) ? (const void *)event->argv[v].data.p : (const void *)AG_PtrMismatch())
+# define AG_cPTR(v) ((v <= event->argc && event->argv[v].type == AG_VARIABLE_POINTER && (event->argv[v].info.pFlags & AG_VARIABLE_P_READONLY)) ? (const void *)event->argv[v].data.p : (const void *)AG_PtrMismatch())
 
 # define AG_STRING(v) ((v < event->argc && event->argv[v].type==AG_VARIABLE_STRING) ? event->argv[v].data.s : AG_StringMismatch())
 # define AG_INT(v)    ((v < event->argc && event->argv[v].type==AG_VARIABLE_INT)    ? event->argv[v].data.i : AG_IntMismatch())
@@ -41,7 +41,7 @@
 # define AG_DOUBLE(v) ((v < event->argc && event->argv[v].type==AG_VARIABLE_DOUBLE) ? event->argv[v].data.dbl : AG_DoubleMismatch())
 #else /* !AG_TYPE_SAFETY */
 # define AG_PTR(v)       (event->argv[v].data.p)
-# define AG_CONST_PTR(v) (event->argv[v].data.p)
+# define AG_cPTR(v)      (event->argv[v].data.p)
 # define AG_STRING(v)    (event->argv[v].data.s)
 # define AG_INT(v)       (event->argv[v].data.i)
 # define AG_UINT(v)      (event->argv[v].data.u)
@@ -55,17 +55,23 @@
 #else
 # define AG_CHAR(v)      ((AG_Char)AG_UINT(v))
 #endif
+
 #define AG_SELF()        AG_PTR(0)
 
-#define AG_PTR_NAMED(k)       AG_GetNamedPtr(event,(k))
-#define AG_CONST_PTR_NAMED(k) AG_GetNamedConstPtr(event,(k))
-#define AG_STRING_NAMED(k)    AG_GetNamedString(event,(k))
-#define AG_INT_NAMED(k)       AG_GetNamedInt(event,(k))
-#define AG_UINT_NAMED(k)      AG_GetNamedUint(event,(k))
-#define AG_LONG_NAMED(k)      AG_GetNamedLong(event,(k))
-#define AG_ULONG_NAMED(k)     AG_GetNamedUlong(event,(k))
-#define AG_FLOAT_NAMED(k)     AG_GetNamedFlt(event,(k))
-#define AG_DOUBLE_NAMED(k)    AG_GetNamedDbl(event,(k))
+#define AG_PTR_NAMED(k)    AG_GetNamedPtr(event,(k))
+#define AG_cPTR_NAMED(k)   AG_GetNamedConstPtr(event,(k))
+#define AG_STRING_NAMED(k) AG_GetNamedString(event,(k))
+#define AG_INT_NAMED(k)    AG_GetNamedInt(event,(k))
+#define AG_UINT_NAMED(k)   AG_GetNamedUint(event,(k))
+#define AG_LONG_NAMED(k)   AG_GetNamedLong(event,(k))
+#define AG_ULONG_NAMED(k)  AG_GetNamedUlong(event,(k))
+#define AG_FLOAT_NAMED(k)  AG_GetNamedFlt(event,(k))
+#define AG_DOUBLE_NAMED(k) AG_GetNamedDbl(event,(k))
+
+#ifdef AG_LEGACY
+#define AG_CONST_PTR(v)       AG_cPTR(v)
+#define AG_CONST_PTR_NAMED(k) AG_cPTR_NAMED(k)
+#endif
 
 struct ag_timer;
 struct ag_event_sink;
@@ -207,6 +213,11 @@ typedef void (*AG_EventFn)(AG_Event *_Nonnull);
 #define AG_EVENT_PUSH_ARG(ap,ev) { AG_Variable *V = NULL; switch (*c) { case 'p': AG_EVENT_INS_ARG(ev, ap, AG_VARIABLE_POINTER, p, void *); V->info.pFlags = 0; break; case 'C': if (c[1] == 'p') { AG_EVENT_INS_ARG(ev, ap, AG_VARIABLE_POINTER, p, void *); V->info.pFlags = AG_VARIABLE_P_READONLY; c++; } break; case 'i': AG_EVENT_INS_ARG(ev, ap, AG_VARIABLE_INT, i, int); break; case 'u': AG_EVENT_INS_ARG(ev, ap, AG_VARIABLE_UINT, u, Uint); break; AG_EVENT_PUSH_ARG_CASE_FLT(ev) case 's': AG_EVENT_INS_ARG(ev, ap, AG_VARIABLE_STRING, s, char *); break; case 'l': switch (c[1]) { AG_EVENT_PUSH_ARG_CASE_LONG(ev) default: AG_EVENT_PARSER_ERROR; } c++; break; case ' ': case ',': case '%': c++; continue; default: AG_EVENT_PARSER_ERROR; } c++; if (V != NULL) { AG_EVENT_GET_NAMED_ARG(); } }
 #define AG_EVENT_GET_ARGS(ev, fmtp) if (fmtp != NULL) { const char *c = (const char *)fmtp; va_list ap; va_start(ap, fmtp); while (*c != '\0') { AG_EVENT_PUSH_ARG(ap, (ev)); } va_end(ap); }
 
+/*
+ * Dump AG_Event arguments to the console.
+ */
+#define AG_EVENT_DUMP(ev) { char buf[128]; int i; AG_Debug(NULL, "Event \"" AGSI_YEL "%s" AGSI_RST "\" {\n", (ev)->name); for (i = 0; i < (ev)->argc; i++) { AG_Variable *V = &(ev)->argv[i]; AG_PrintVariable(buf, sizeof(buf), V); if (V->name[0] != '\0') { AG_Debug(NULL, "\t#%02d. " AGSI_CYAN "%s" AGSI_RST " \"%s\" = " AGSI_BOLD "%s" AGSI_RST "\n", i, agVariableTypes[V->type].name, V->name, buf); } else { AG_Debug(NULL, "\t#%02d. " AGSI_CYAN "%s" AGSI_RST " = " AGSI_BOLD "%s" AGSI_RST "\n", i, agVariableTypes[V->type].name, buf); } if (i == (ev)->argc0-1) AG_Debug(NULL, "\t---------------------------------\n"); } AG_Debug(NULL, "}\n"); }
+
 /* Begin generated block */
 __BEGIN_DECLS
 extern DECLSPEC void AG_EventInit(AG_Event *_Nonnull);
@@ -220,9 +231,6 @@ extern DECLSPEC AG_Event *_Nonnull AG_SetEvent(void *_Nonnull, const char *_Null
 extern DECLSPEC AG_Event *_Nonnull AG_AddEvent(void *_Nonnull, const char *_Nullable, _Nullable AG_EventFn, const char *_Nullable, ...);
 extern DECLSPEC void AG_UnsetEvent(void *_Nonnull, const char *_Nonnull);
 extern DECLSPEC void AG_UnsetEventByPtr(void *_Nonnull, AG_Event *_Nonnull);
-/*
- * TODO AG_AppendEvent(), AG_ReplaceEvent().
- */
 extern DECLSPEC void AG_PostEventByPtr(void *_Nonnull, AG_Event *_Nonnull, const char *_Nullable, ...);
 extern DECLSPEC void AG_PostEvent(void *_Nonnull, const char *_Nonnull, const char *_Nullable, ...);
 extern DECLSPEC void AG_EventGetArgs(AG_Event *_Nonnull, const char *_Nullable, va_list);
