@@ -3,44 +3,43 @@
 #ifndef _AGAR_CORE_OBJECT_H_
 #define _AGAR_CORE_OBJECT_H_
 
-#ifndef AG_OBJECT_NAME_MAX
-# if AG_MODEL == AG_SMALL
-#  define AG_OBJECT_NAME_MAX 16
-# else
-#  define AG_OBJECT_NAME_MAX 36
-# endif
+#ifndef AG_OBJECT_NAME_MAX                     /* Max length of object name */
+#define AG_OBJECT_NAME_MAX 36
 #endif
-#ifndef AG_OBJECT_TYPE_MAX
+#ifndef AG_OBJECT_MAX_VARIABLES            /* Max object instance variables */
+#define AG_OBJECT_MAX_VARIABLES 0xffff
+#endif
+#ifndef AG_OBJECT_TYPE_MAX               /* Max length of object class name */
 # if AG_MODEL == AG_SMALL
 #  define AG_OBJECT_TYPE_MAX 24
 # else
 #  define AG_OBJECT_TYPE_MAX 48
 # endif
 #endif
-#ifndef AG_OBJECT_HIER_MAX
+#ifndef AG_OBJECT_HIER_MAX      /* Max length of full inheritance hierarchy */
 # if AG_MODEL == AG_SMALL
-#  define AG_OBJECT_HIER_MAX 48
+#  define AG_OBJECT_HIER_MAX 4
 # elif AG_MODEL == AG_MEDIUM
 #  define AG_OBJECT_HIER_MAX 64
 # elif AG_MODEL == AG_LARGE
 #  define AG_OBJECT_HIER_MAX 96
 # endif
 #endif
-#ifndef AG_OBJECT_PATH_MAX
+#ifndef AG_OBJECT_PATH_MAX   /* Max length of a complete path name in a VFS */
 # if AG_MODEL == AG_SMALL
 #  define AG_OBJECT_PATH_MAX 64
 # else
 #  define AG_OBJECT_PATH_MAX 128
 # endif
 #endif
-#ifndef AG_OBJECT_LIBS_MAX
+#ifndef AG_OBJECT_LIBS_MAX                /* Max length of DSO module lists */
 # if AG_MODEL == AG_SMALL
 #  define AG_OBJECT_LIBS_MAX 16
 # else
 #  define AG_OBJECT_LIBS_MAX 32
 # endif
 #endif
-#ifndef AG_OBJECT_CLASSTBLSIZE
+#ifndef AG_OBJECT_CLASSTBLSIZE     /* Number of buckets for the class table */
 # if AG_MODEL == AG_SMALL
 #  define AG_OBJECT_CLASSTBLSIZE 8
 # elif AG_MODEL == AG_MEDIUM
@@ -50,14 +49,6 @@
 # endif
 #endif
 
-#ifndef AG_OBJECT_MAX_VARIABLES
-# define AG_OBJECT_MAX_VARIABLES 0xffff
-#endif
-#ifndef AG_OBJECT_TYPE_TAG
-# define AG_OBJECT_TYPE_TAG "AgarObj"
-#endif
-#define AG_OBJECT_TYPE_TAG_LEN 8
-
 #include <agar/core/begin.h>
 
 struct ag_object;
@@ -68,10 +59,11 @@ struct ag_dbt;
 #include <agar/core/variable.h>
 #include <agar/core/event.h>
 #include <agar/core/agtime.h>
+#include <agar/core/classes.h>
 
-/* Object class specification (generated) */
+/* Normalized object class specification. */
 typedef struct ag_object_class_spec {
-	char hier[AG_OBJECT_HIER_MAX];	/* Inheritance hierarchy (normalized) */
+	char hier[AG_OBJECT_HIER_MAX];	/* Inheritance hierarchy */
 	char spec[AG_OBJECT_HIER_MAX];	/* Full class string + optional @libs */
 	char name[AG_OBJECT_NAME_MAX];	/* Name of the last class in hierarchy */
 #ifdef AG_ENABLE_DSO
@@ -79,14 +71,12 @@ typedef struct ag_object_class_spec {
 #endif
 } AG_ObjectClassSpec;
 
-/* Registered name space (for NAMESPACES) */
 typedef struct ag_namespace {
 	const char *_Nonnull name;	/* Name string */
 	const char *_Nonnull pfx;	/* Prefix string */
 	const char *_Nonnull url;	/* URL of package */
 } AG_Namespace;
 
-/* Convenient typedefs for base AG_Object methods */
 typedef void (*AG_ObjectInitFn)(void *_Nonnull);
 typedef void (*AG_ObjectResetFn)(void *_Nonnull);
 typedef void (*AG_ObjectDestroyFn)(void *_Nonnull);
@@ -95,74 +85,68 @@ typedef int (*AG_ObjectLoadFn)(void *_Nonnull, AG_DataSource *_Nonnull,
                                const AG_Version *_Nonnull);
 typedef int (*AG_ObjectSaveFn)(void *_Nonnull, AG_DataSource *_Nonnull);
 #else
-typedef int (*AG_ObjectLoadFn)(void *_Nonnull, void *_Nonnull,
-                               const AG_Version *_Nonnull);
+typedef int (*AG_ObjectLoadFn)(void *_Nonnull, void *_Nonnull, const AG_Version *_Nonnull);
 typedef int (*AG_ObjectSaveFn)(void *_Nonnull, void *_Nonnull);
 #endif
 typedef void *_Nullable (*AG_ObjectEditFn) (void *_Nonnull);
 
-/* Object class description (private, generated section) */
-typedef struct ag_object_class_pvt {
+/*
+ * Agar object class description.
+ */
+typedef struct ag_object_class {              /* --- Required fields --- */
+	char hier[AG_OBJECT_HIER_MAX];        /* Full inheritance hierarchy */
+	AG_Size size;                         /* Size of instance structure */
+	AG_Version ver;                       /* Version, Class ID & Unicode */
+
+	_Nullable AG_ObjectInitFn init;       /* Initialization */
+	_Nullable AG_ObjectResetFn reset;     /* Pre-Load & Pre-Finalization */
+	_Nullable AG_ObjectDestroyFn destroy; /* Finalization */
+	_Nullable AG_ObjectLoadFn load;       /* Deserialization */
+	_Nullable AG_ObjectSaveFn save;       /* Serialization */
+	_Nullable AG_ObjectEditFn edit;       /* User-defined routine */
+
+	                                            /* --- Generated fields --- */
+	char name[AG_OBJECT_TYPE_MAX];		    /* Name of this class only */
+	struct ag_object_class *_Nullable super;    /* Direct superclass */
 	char libs[AG_OBJECT_LIBS_MAX];              /* List of required modules */
 	AG_TAILQ_HEAD_(ag_object_class) sub;        /* Direct subclasses */
 	AG_TAILQ_ENTRY(ag_object_class) subclasses; /* Subclass entry */
-} AG_ObjectClassPvt;
-
-/* Object class description */
-typedef struct ag_object_class {              /* --- Required fields --- */
-	char hier[AG_OBJECT_HIER_MAX];        /* Inheritance hierarchy */
-	AG_Size size;                         /* Instance structure size (bytes) */
-	AG_Version ver;                       /* Serialized version (or 0.0) */
-	_Nullable AG_ObjectInitFn init;       /* Initialization */
-	_Nullable AG_ObjectResetFn reset;     /* Pre-serialization code */
-	_Nullable AG_ObjectDestroyFn destroy; /* Finalization */
-	_Nullable AG_ObjectLoadFn load;       /* Deserialization (reading) */
-	_Nullable AG_ObjectSaveFn save;       /* Serialization (writing) */
-	_Nullable AG_ObjectEditFn edit;       /* User-defined editor callback */
-
-	                                         /* --- Generated fields --- */
-	char name[AG_OBJECT_TYPE_MAX];		 /* Short name of this class */
-	struct ag_object_class *_Nullable super; /* Direct superclass */
-	AG_ObjectClassPvt pvt;			 /* Private data */
 } AG_ObjectClass;
 
 AG_TAILQ_HEAD(ag_objectq, ag_object);
 
-/* Object instance */
+/*
+ * Agar object instance structure.
+ */
 typedef struct ag_object {
-#ifdef AG_TYPE_SAFETY
-	char tag[AG_OBJECT_TYPE_TAG_LEN]; /* For runtime type-safety check */
+#if AG_MODEL != AG_SMALL
+	Uint32 tag;                       /* Validity signature */
+	AG_Class cid;                     /* Numerical class ID */
+	char name[AG_OBJECT_NAME_MAX];    /* Name string (unique in parent) */
+	Uint flags;                       /* Option flags */
+#else
+	Uint8 tag8;                       /* Validity signature */
+	Uint16 cid;                       /* Numerical class ID */
+	Uint8 name;                       /* Numerical ID (unique in parent) */
+	Uint8 flags;                      /* Option flags */
 #endif
-	char name[AG_OBJECT_NAME_MAX];    /* Object ID (unique in parent) */
-	Uint flags;
-#define AG_OBJECT_FLOATING_VARS	 0x00001  /* Clear variables before load */
-#define AG_OBJECT_NON_PERSISTENT 0x00002  /* Never include in saves */
-#define AG_OBJECT_INDESTRUCTIBLE 0x00004  /* Not destructible (advisory) */
-#define AG_OBJECT_RESIDENT	 0x00008  /* Data part is resident */
-#define AG_OBJECT_STATIC	 0x00020  /* Don't free() after detach */
-#define AG_OBJECT_READONLY	 0x00040  /* Disallow edition (advisory) */
-#define AG_OBJECT_WAS_RESIDENT	 0x00080  /* Used internally by ObjectLoad() */
-#define AG_OBJECT_REOPEN_ONLOAD	 0x00200  /* Recreate editor UI on ObjectLoad() */
-#define AG_OBJECT_REMAIN_DATA	 0x00400  /* Keep user data resident */
-#define AG_OBJECT_DEBUG		 0x00800  /* Enable debugging */
-#define AG_OBJECT_NAME_ONATTACH	 0x01000  /* Generate name on attach */
-#define AG_OBJECT_CHLD_AUTOSAVE	 0x02000  /* Include child obj data in archive */
-#define AG_OBJECT_DEBUG_DATA	 0x04000  /* Datafiles contain debug info */
-#define AG_OBJECT_INATTACH	 0x08000  /* In AG_ObjectAttach() */
-#define AG_OBJECT_INDETACH	 0x10000  /* In AG_ObjectDetach() */
-#define AG_OBJECT_BOUND_EVENTS	 0x20000  /* Generate "bound" events whenever
-					     AG_Bind*() is invoked */
-#define AG_OBJECT_SAVED_FLAGS (AG_OBJECT_FLOATING_VARS| AG_OBJECT_INDESTRUCTIBLE| AG_OBJECT_READONLY| AG_OBJECT_REOPEN_ONLOAD| AG_OBJECT_REMAIN_DATA| AG_OBJECT_DEBUG| AG_OBJECT_BOUND_EVENTS)
-
-	AG_ObjectClass *_Nonnull cls;     /* Class description */
+#define AG_OBJECT_STATIC         0x01     /* Don't free() after detach */
+#define AG_OBJECT_INDESTRUCTIBLE 0x02     /* Not deletable (advisory) */
+#define AG_OBJECT_READONLY       0x04     /* Not modifiable (advisory) */
+#define AG_OBJECT_DEBUG          0x08     /* Enable debugging */
+#define AG_OBJECT_DEBUG_DATA     0x10     /* Datafiles contain debug info */
+#define AG_OBJECT_NAME_ONATTACH  0x20     /* Generate name on attach */
+#define AG_OBJECT_BOUND_EVENTS   0x40     /* Raise "bound" events in AG_Bind*() */
+#define AG_OBJECT_SAVED_FLAGS (AG_OBJECT_INDESTRUCTIBLE | AG_OBJECT_READONLY | AG_OBJECT_DEBUG | AG_OBJECT_BOUND_EVENTS)
 
+	AG_ObjectClass *_Nonnull cls;     /* Class description structure */
 	AG_TAILQ_HEAD_(ag_event) events;  /* Event handlers */
 #ifdef AG_TIMERS
-	AG_TAILQ_HEAD_(ag_timer) timers;  /* Running timers */
+	AG_TAILQ_HEAD_(ag_timer) timers;  /* Registered timers */
 #endif
-	AG_TAILQ_HEAD_(ag_variable) vars; /* Named variables / bindings */
-	struct ag_objectq children;       /* Child objects */
-	AG_TAILQ_ENTRY(ag_object) cobjs;  /* Entry in parent */
+	AG_TAILQ_HEAD_(ag_variable) vars; /* Properties / Variables */
+	struct ag_objectq children;       /* List of child objects */
+	AG_TAILQ_ENTRY(ag_object) cobjs;  /* Entry in parent's children list */
 	void *_Nullable parent;           /* Parent in VFS (NULL = is root) */
 	void *_Nonnull root;              /* VFS root (possibly self) */
 #ifdef AG_TIMERS
@@ -179,65 +163,92 @@ typedef struct ag_object_header {
 	Uint flags;                       /* Object flags */
 } AG_ObjectHeader;
 
-#define AGOBJECT(ob) ((struct ag_object *)(ob))
-#define AGCLASS(cls) ((struct ag_object_class *)(cls))
-#define AGOBJECT_CLASS(obj) ((struct ag_object_class *)(AGOBJECT(obj)->cls))
+#define  AGOBJECT(ob)        ((struct ag_object *)(ob))
+#define AGCOBJECT(ob)        ((const struct ag_object *)(ob))
 
-/* Argument Accessors */
+#define  AGOBJECTCLASS(cls)  ((struct ag_object_class *)(cls))
+#define AGCOBJECTCLASS(cls)  ((const struct ag_object_class *)(cls))
+
+/* Return a pointer to an object's class description structure. */
+#define  AGOBJECT_CLASS(obj) ((struct ag_object_class *)(AGOBJECT(obj)->cls))
+#define AGCOBJECT_CLASS(obj) ((struct ag_object_class *)(AGCOBJECT(obj)->cls))
+#define  AGOBJECT_CONST_CLASS(obj) ((const struct ag_object_class *)(AGOBJECT(obj)->cls))
+#define AGCOBJECT_CONST_CLASS(obj) ((const struct ag_object_class *)(AGCOBJECT(obj)->cls))
+
+/*
+ * Minimal object validity test. Return 1 if the given object has a valid
+ * signature word or 0 if it doesn't match. The signature is a pseudo-random
+ * number generated once on initialization such that it is a different value
+ * than that used by previous instances of the application.
+ */
+#define AG_OBJECT_VALID(obj)   (AGOBJECT(obj)->tag == agObjectSignature)
+#define AG_NON_OBJECT_VALID(p) ((p)->tag == agNonObjectSignature)
+
+/*
+ * Event Argument Accessors
+ */
 #ifdef AG_TYPE_SAFETY
-# define AG_OBJECT_VALID(p) (strncmp(AGOBJECT(p)->tag, AG_OBJECT_TYPE_TAG, AG_OBJECT_TYPE_TAG_LEN) == 0)
 # define AG_OBJECT(v,hier) ((v <= event->argc && event->argv[v].type == AG_VARIABLE_POINTER && !(event->argv[v].info.pFlags & AG_VARIABLE_P_READONLY) && AG_OBJECT_VALID(event->argv[v].data.p) && AG_OfClass(event->argv[v].data.p,(hier))) ? event->argv[v].data.p : AG_ObjectMismatch())
 # define AG_OBJECT_PTR(v) ((v <= event->argc && event->argv[v].type == AG_VARIABLE_POINTER && !(event->argv[v].info.pFlags & AG_VARIABLE_P_READONLY) && AG_OBJECT_VALID(event->argv[v].data.p)) ? event->argv[v].data.p : AG_ObjectMismatch())
-# define AG_CONST_OBJECT(v,hier) ((v <= event->argc && event->argv[v].type == AG_VARIABLE_POINTER && (event->argv[v].info.pFlags & AG_VARIABLE_P_READONLY) && AG_OBJECT_VALID(event->argv[v].data.p) && AG_OfClass(event->argv[v].data.p,(hier))) ? (const void *)event->argv[v].data.p : (const void *)AG_ObjectMismatch())
-# define AG_CONST_OBJECT_PTR(v) ((v <= event->argc && event->argv[v].type == AG_VARIABLE_POINTER && (event->argv[v].info.pFlags & AG_VARIABLE_P_READONLY) && AG_OBJECT_VALID(event->argv[v].data.p)) ? (const void *)event->argv[v].data.p : (const void *)AG_ObjectMismatch())
-# define AG_OBJECT_ISA(obj,class) { if (!AG_OBJECT_VALID(obj)) { AG_FatalErrorF("%p is not a valid AG_Object", (obj)); } if (!AG_OfClass((obj),(class))) { AG_FatalErrorF("%s is not a %s", AGOBJECT(obj)->name, class); } }
+# define AG_cOBJECT(v,hier) ((v <= event->argc && event->argv[v].type == AG_VARIABLE_POINTER && (event->argv[v].info.pFlags & AG_VARIABLE_P_READONLY) && AG_OBJECT_VALID(event->argv[v].data.p) && AG_OfClass(event->argv[v].data.p,(hier))) ? (const void *)event->argv[v].data.p : (const void *)AG_ObjectMismatch())
+# define AG_cOBJECT_PTR(v) ((v <= event->argc && event->argv[v].type == AG_VARIABLE_POINTER && (event->argv[v].info.pFlags & AG_VARIABLE_P_READONLY) && AG_OBJECT_VALID(event->argv[v].data.p)) ? (const void *)event->argv[v].data.p : (const void *)AG_ObjectMismatch())
+
+/* Assertion based on object validity and class-membership test. */
+# define AG_OBJECT_ISA(obj,class) { if (!AG_OBJECT_VALID(obj)) AG_FatalError("Invalid AG_Object"); if (!AG_OfClass((obj),(class))) AG_FatalError("Bad AG_Object class"); }
 
 #else /* !AG_TYPE_SAFETY */
 
-# define AG_OBJECT_VALID(p)       (1)
-# define AG_OBJECT(v,hier)        (event->argv[v].data.p)
-# define AG_OBJECT_PTR(v)         (event->argv[v].data.p)
-# define AG_CONST_OBJECT(v,hier)  (event->argv[v].data.p)
-# define AG_CONST_OBJECT_PTR(v)  ((const void *)event->argv[v].data.p)
-# define AG_OBJECT_ISA(obj,class)
+# define  AG_OBJECT(v,hier) (event->argv[v].data.p)
+# define AG_cOBJECT(v,hier) (event->argv[v].data.p)
+# define  AG_OBJECT_PTR(v)  (event->argv[v].data.p)
+# define AG_cOBJECT_PTR(v)  ((const void *)event->argv[v].data.p)
+# define  AG_OBJECT_ISA(obj,class)
 
 #endif /* AG_TYPE_SAFETY */
 
-#define AG_OBJECT_SELF()         AG_OBJECT_PTR(0)
-#define AG_OBJECT_NAMED(n)       AG_PTR_NAMED(n)
-#define AG_CONST_OBJECT_SELF()   AG_CONST_OBJECT_PTR(0)
-#define AG_CONST_OBJECT_NAMED(n) AG_CONST_PTR_NAMED(n)
+#define AG_OBJECT_SELF()    AG_OBJECT_PTR(0)
+#define AG_OBJECT_NAMED(n)  AG_PTR_NAMED(n)
+#define AG_cOBJECT_SELF()   AG_cOBJECT_PTR(0)
+#define AG_cOBJECT_NAMED(n) AG_cPTR_NAMED(n)
 
-/* Iterate over child objects */
-#define AGOBJECT_FOREACH_CHILD(var, ob, t) for((var) = (struct t *)AG_TAILQ_FIRST(&AGOBJECT(ob)->children); (var) != (struct t *)AG_TAILQ_END(&AGOBJECT(ob)->children); (var) = (struct t *)AG_TAILQ_NEXT(AGOBJECT(var), cobjs))
-#define AGOBJECT_FOREACH_CHILD_REVERSE(var, ob, t) for((var) = (struct t *)AG_TAILQ_LAST(&AGOBJECT(ob)->children, ag_objectq); (var) != (struct t *)AG_TAILQ_END(&AGOBJECT(ob)->children); (var) = (struct t *)AG_TAILQ_PREV(AGOBJECT(var), ag_objectq, cobjs))
-#define AGOBJECT_NEXT_CHILD(var,t) ((struct t *)AG_TAILQ_NEXT(AGOBJECT(var),cobjs))
-#define AGOBJECT_LAST_CHILD(var,t) ((struct t *)AG_TAILQ_LAST(&AGOBJECT(var)->children,ag_objectq))
-
-/* Iterate over child objects of a given class. */
-# define AGOBJECT_FOREACH_CLASS(var, ob, t, subclass) AGOBJECT_FOREACH_CHILD(var,ob,t) if (!AG_OBJECT_VALID(var) || !AG_OfClass(var,(subclass))) { continue; } else
-
-#if defined(_AGAR_INTERNAL) || defined(_USE_AGAR_CORE)
 /*
- * Shorthands
+ * Iterators
  */
-# define OBJECT(ob)              AGOBJECT(ob)
-# define OBJECT_CLASS(ob)        AGOBJECT_CLASS(ob)
-# define CLASS(ob)               AGCLASS(ob)
 
-# define OBJECT_RESIDENT(ob)    (AGOBJECT(ob)->flags & AG_OBJECT_RESIDENT)
-# define OBJECT_PERSISTENT(ob) !(AGOBJECT(ob)->flags & AG_OBJECT_NON_PERSISTENT)
-# define OBJECT_DEBUG(ob)       (AGOBJECT(ob)->flags & AG_OBJECT_DEBUG)
+/* Iterate over all child objects. */
+#define AGOBJECT_FOREACH_CHILD(var, ob, t) for ((var) = (struct t *)AG_TAILQ_FIRST(&AGOBJECT(ob)->children); (var) != (struct t *)AG_TAILQ_END( &AGOBJECT(ob)->children); (var) = (struct t *)AG_TAILQ_NEXT( AGOBJECT(var), cobjs))
+#define AGOBJECT_FOREACH_CHILD_REVERSE(var, ob, t) for ((var) = (struct t *)AG_TAILQ_LAST(&AGOBJECT(ob)->children, ag_objectq); (var) != (struct t *)AG_TAILQ_END( &AGOBJECT(ob)->children); (var) = (struct t *)AG_TAILQ_PREV( AGOBJECT(var), ag_objectq, cobjs))
+#define AGOBJECT_NEXT_CHILD(var,t) ((struct t *)AG_TAILQ_NEXT(AGOBJECT(var), cobjs))
+#define AGOBJECT_LAST_CHILD(var,t) ((struct t *)AG_TAILQ_LAST(&AGOBJECT(var)->children, ag_objectq))
+
+/* Iterate over child objects of a given class (string pattern match). */
+# define AGOBJECT_FOREACH_CLASS(var, ob, t, pattern) AGOBJECT_FOREACH_CHILD(var,ob,t) if (!AG_OBJECT_VALID(var) || !AG_OfClass(var,(pattern))) { continue; } else
+
+/* Iterate over child objects of a given class (numerical ID exact match). */
+# define AGOBJECT_FOREACH_CLASSID(var, ob, t, cid) AGOBJECT_FOREACH_CHILD(var,ob,t) if (!AG_OBJECT_VALID(var) || AGOBJECT(var)->cid != cid) { continue; } else
+
+/* Iterate over child objects of a given class (numerical ID range match). */
+# define AGOBJECT_FOREACH_CLASSIDS(var, ob, t, c1,c2) AGOBJECT_FOREACH_CHILD(var,ob,t) if (!AG_OBJECT_VALID(var) || AGOBJECT(var)->cid < c1 || AGOBJECT(var)->cid >= c2) { continue; } else
+
+/* Internal shorthand forms */
+#if defined(_AGAR_INTERNAL) || defined(_USE_AGAR_CORE)
+# define OBJECT(ob)       AGOBJECT(ob)
+# define OBJECT_CLASS(ob) AGOBJECT_CLASS(ob)
+# define OBJECTCLASS(ob)  AGOBJECTCLASS(ob)
 
 # define OBJECT_FOREACH_CHILD(var,ob,t)          AGOBJECT_FOREACH_CHILD((var),(ob),t)
 # define OBJECT_FOREACH_CHILD_REVERSE(var,ob,t)  AGOBJECT_FOREACH_CHILD_REVERSE((var),(ob),t)
-# define OBJECT_FOREACH_CLASS(var,ob,t,subclass) AGOBJECT_FOREACH_CLASS((var),(ob),t,(subclass))
+# define OBJECT_FOREACH_CLASS(var,ob,t,pattern)  AGOBJECT_FOREACH_CLASS((var),(ob),t,(pattern))
+# define OBJECT_FOREACH_CLASSID(var,ob,t,cid)    AGOBJECT_FOREACH_CLASSID((var),(ob),t,(cid))
+# define OBJECT_FOREACH_CLASSIDS(var,ob,t,c1,c2) AGOBJECT_FOREACH_CLASSIDS((var),(ob),t,(c1),(c2))
 # define OBJECT_NEXT_CHILD(var,t)                AGOBJECT_NEXT_CHILD((var),t)
 # define OBJECT_LAST_CHILD(var,t)                AGOBJECT_LAST_CHILD((var),t)
-#endif /* _AGAR_INTERNAL or _USE_AGAR_CORE */
+#endif
 
 /* Begin generated block */
 __BEGIN_DECLS
+extern DECLSPEC Uint32 agObjectSignature; 
+extern DECLSPEC Uint32 agNonObjectSignature; 
 extern DECLSPEC AG_ObjectClass agObjectClass; 
 #ifdef AG_THREADS
 extern DECLSPEC _Nonnull_Mutex AG_Mutex agClassLock; 
@@ -260,12 +271,12 @@ extern DECLSPEC void AG_UnloadClass(AG_ObjectClass *_Nonnull);
 extern DECLSPEC void AG_RegisterModuleDirectory(const char *_Nonnull);
 extern DECLSPEC void AG_UnregisterModuleDirectory(const char *_Nonnull);
 #endif
+extern DECLSPEC void AG_RegisterClass(void *_Nonnull);
+extern DECLSPEC void AG_UnregisterClass(void *_Nonnull);
 #ifdef AG_NAMESPACES
 extern DECLSPEC AG_Namespace *_Nonnull AG_RegisterNamespace(const char *_Nonnull, const char *_Nonnull, const char *_Nonnull);
 extern DECLSPEC void AG_UnregisterNamespace(const char *_Nonnull);
 #endif
-extern DECLSPEC void AG_RegisterClass(void *_Nonnull);
-extern DECLSPEC void AG_UnregisterClass(void *_Nonnull);
 #if AG_MODEL != AG_SMALL
 extern DECLSPEC void *_Nullable AG_CreateClass(const char *_Nonnull, AG_Size, AG_Size, Uint, Uint);
 extern DECLSPEC void AG_DestroyClass(void *_Nonnull);
@@ -284,6 +295,7 @@ extern DECLSPEC void AG_ObjectAttach(void *_Nullable _Restrict, void *_Nonnull _
 extern DECLSPEC void AG_ObjectInit(void *_Nonnull _Restrict, void *_Nullable _Restrict);
 extern DECLSPEC void AG_ObjectInitStatic(void *_Nonnull, void *_Nullable);
 extern DECLSPEC void AG_ObjectDetach(void *_Nonnull);
+extern DECLSPEC void AG_ObjectDetachLockless(void *_Nonnull);
 extern DECLSPEC void AG_ObjectReset(void *_Nonnull);
 #if AG_MODEL != AG_SMALL
 extern DECLSPEC char *_Nonnull AG_ObjectGetClassName(const void *, int);
@@ -296,26 +308,28 @@ extern DECLSPEC void *_Nullable AG_ObjectFind(void *_Nonnull, const char *_Nonnu
 extern DECLSPEC void *_Nullable AG_ObjectFindParent(void *_Nonnull, const char *_Nonnull, const char *_Nonnull) _Warn_Unused_Result;
 extern DECLSPEC void AG_ObjectSetNameS(void *_Nonnull, const char *_Nullable);
 extern DECLSPEC void AG_ObjectSetName(void *_Nonnull, const char *_Nullable, ...) FORMAT_ATTRIBUTE(printf,2,3);
-extern DECLSPEC void AG_SetFn(void *_Nonnull, const char *_Nonnull, _Nullable AG_EventFn, const char *_Nullable, ...);
+extern DECLSPEC AG_Variable *AG_SetFn(void *_Nonnull, const char *_Nonnull, _Nullable AG_EventFn, const char *_Nullable, ...);
 #if AG_MODEL != AG_SMALL
 extern DECLSPEC void AG_ObjectMoveUp(void *_Nonnull);
 extern DECLSPEC void AG_ObjectMoveDown(void *_Nonnull);
 extern DECLSPEC void AG_ObjectMoveToHead(void *_Nonnull);
-#endif
 extern DECLSPEC void AG_ObjectMoveToTail(void *_Nonnull);
+#endif
 extern DECLSPEC void AG_ObjectDestroy(void *_Nonnull);
+#if AG_MODEL != AG_SMALL
+extern DECLSPEC void AG_ObjectFreeChildrenOfTypeLockless(AG_Object *_Nonnull, const char *_Nonnull);
+extern DECLSPEC void AG_ObjectFreeChildrenOfType(void *_Nonnull, const char *_Nonnull);
+#endif
 extern DECLSPEC void AG_ObjectFreeVariables(void *_Nonnull);
 extern DECLSPEC void AG_ObjectFreeChildren(void *_Nonnull);
+extern DECLSPEC void AG_ObjectFreeChildrenLockless(AG_Object *_Nonnull);
 extern DECLSPEC void AG_ObjectFreeEvents(AG_Object *_Nonnull);
 #ifdef AG_SERIALIZATION
 extern DECLSPEC int AG_ObjectCopyFilename(void *_Nonnull, char *_Nonnull, AG_Size);
 extern DECLSPEC int AG_ObjectCopyDirname(void *_Nonnull, char *_Nonnull, AG_Size);
 extern DECLSPEC int AG_ObjectChanged(void *_Nonnull);
 extern DECLSPEC int AG_ObjectChangedAll(void *_Nonnull);
-extern DECLSPEC int AG_ObjectInUse(void *_Nonnull) _Pure_Attribute_If_Unthreaded _Warn_Unused_Result;
 extern DECLSPEC void AG_ObjectUnlinkDatafiles(void *_Nonnull);
-extern DECLSPEC int AG_ObjectPageIn(void *_Nonnull);
-extern DECLSPEC int AG_ObjectPageOut(void *_Nonnull);
 extern DECLSPEC int AG_ObjectSerialize(void *_Nonnull, AG_DataSource *_Nonnull);
 extern DECLSPEC int AG_ObjectUnserialize(void *_Nonnull, AG_DataSource *_Nonnull);
 extern DECLSPEC int AG_ObjectSave(void *_Nonnull);
