@@ -84,12 +84,12 @@
 /*
  * Singly-linked List definitions.
  */
-#define AG_SLIST_HEAD(name, t) struct name { struct t *slh_first; /* first element */ }
-#define AG_SLIST_HEAD_(t) struct { struct t *slh_first; /* first element */ }
+#define AG_SLIST_HEAD(name, t) struct name { struct t *_Nullable slh_first; }
+#define AG_SLIST_HEAD_(t) struct { struct t *_Nullable slh_first; }
  
 #define AG_SLIST_HEAD_INITIALIZER(head) { NULL }
  
-#define AG_SLIST_ENTRY(t) struct { struct t *sle_next; /* next element */ }
+#define AG_SLIST_ENTRY(t) struct { struct t *_Nullable sle_next; }
  
 /*
  * Singly-linked List access methods.
@@ -117,12 +117,12 @@
 /*
  * List definitions.
  */
-#define AG_LIST_HEAD(name, t) struct name { struct t *lh_first; /* first element */ }
-#define AG_LIST_HEAD_(t) struct { struct t *lh_first; /* first element */ }
+#define AG_LIST_HEAD(name, t) struct name { struct t *_Nullable lh_first; }
+#define AG_LIST_HEAD_(t) struct { struct t *_Nullable lh_first; }
 
 #define AG_LIST_HEAD_INITIALIZER(head) { NULL }
 
-#define AG_LIST_ENTRY(t) struct { struct t *le_next; /* next element */ struct t **le_prev; /* address of previous next element */ }
+#define AG_LIST_ENTRY(t) struct { struct t *_Nullable le_next; struct t *_Nullable *_Nullable le_prev; }
 
 /*
  * List access methods
@@ -152,12 +152,12 @@
 /*
  * Simple queue definitions.
  */
-#define AG_SIMPLEQ_HEAD(name, t) struct name { struct t *sqh_first; /* first element */ struct t **sqh_last; /* addr of last next element */ }
-#define AG_SIMPLEQ_HEAD_(t) struct { struct t *sqh_first; /* first element */ struct t **sqh_last; /* addr of last next element */ }
+#define AG_SIMPLEQ_HEAD(name, t) struct name { struct t *_Nullable sqh_first; struct t *_Nullable *_Nullable sqh_last; }
+#define AG_SIMPLEQ_HEAD_(t) struct { struct t *_Nullable sqh_first; struct t *_Nullable *_Nullable sqh_last; }
 
 #define AG_SIMPLEQ_HEAD_INITIALIZER(head) { NULL, &(head).sqh_first }
 
-#define AG_SIMPLEQ_ENTRY(t) struct { struct t *sqe_next; /* next element */ }
+#define AG_SIMPLEQ_ENTRY(t) struct { struct t *_Nullable sqe_next; }
 
 /*
  * Simple queue access methods.
@@ -185,12 +185,12 @@
 /*
  * Tail queue definitions.
  */
-#define AG_TAILQ_HEAD(name, t) struct name { struct t *tqh_first; /* first element */ struct t **tqh_last; /* addr of last next element */ }
-#define AG_TAILQ_HEAD_(t) struct { struct t *tqh_first; /* first element */ struct t **tqh_last; /* addr of last next element */ }
+#define AG_TAILQ_HEAD(name, t) struct name { struct t *_Nullable tqh_first; /* first element */ struct t *_Nullable *_Nullable tqh_last; /* addr of last next element */ }
+#define AG_TAILQ_HEAD_(t) struct { struct t *_Nullable tqh_first; /* first element */ struct t *_Nullable *_Nullable tqh_last; /* addr of last next element */ }
 
 #define AG_TAILQ_HEAD_INITIALIZER(head) { NULL, &(head).tqh_first }
 
-#define AG_TAILQ_ENTRY(t) struct { struct t *tqe_next; /* next element */ struct t **tqe_prev; /* address of previous next element */ }
+#define AG_TAILQ_ENTRY(t) struct { struct t *_Nullable tqe_next; /* next element */ struct t *_Nullable *_Nullable tqe_prev; /* address of previous next element */ }
 #define AG_TAILQ_ENTRY_INITIALIZER { NULL, NULL }
 
 /* 
@@ -199,9 +199,11 @@
 #define	AG_TAILQ_FIRST(head)		((head)->tqh_first)
 #define	AG_TAILQ_END(head)		NULL
 #define	AG_TAILQ_NEXT(elm, field)	((elm)->field.tqe_next)
+
 #define AG_TAILQ_LAST(head, headname) (*(((struct headname *)((head)->tqh_last))->tqh_last))
-/* XXX */
+
 #define AG_TAILQ_PREV(elm, headname, field) (*(((struct headname *)((elm)->field.tqe_prev))->tqh_last))
+
 #define AG_TAILQ_EMPTY(head) (AG_TAILQ_FIRST(head) == AG_TAILQ_END(head))
 
 #define AG_TAILQ_FOREACH(var, head, field) for((var) = AG_TAILQ_FIRST(head); (var) != AG_TAILQ_END(head); (var) = AG_TAILQ_NEXT(var, field))
@@ -228,12 +230,12 @@
 /*
  * Circular queue definitions.
  */
-#define AG_CIRCLEQ_HEAD(name, t) struct name { struct t *cqh_first; /* first element */ struct t *cqh_last; /* last element */ }
-#define AG_CIRCLEQ_HEAD_(t) struct { struct t *cqh_first; /* first element */ struct t *cqh_last; /* last element */ }
+#define AG_CIRCLEQ_HEAD(name, t) struct name { struct t *_Nullable cqh_first; struct t *_Nullable cqh_last; }
+#define AG_CIRCLEQ_HEAD_(t) struct { struct t *_Nullable cqh_first; struct t *_Nullable cqh_last; }
 
 #define AG_CIRCLEQ_HEAD_INITIALIZER(head) { AG_CIRCLEQ_END(&head), AG_CIRCLEQ_END(&head) }
 
-#define AG_CIRCLEQ_ENTRY(t) struct { struct t *cqe_next; /* next element */ struct t *cqe_prev; /* previous element */ }
+#define AG_CIRCLEQ_ENTRY(t) struct { struct t *_Nullable cqe_next; struct t *_Nullable cqe_prev; }
 
 /*
  * Circular queue access methods 
@@ -268,91 +270,91 @@
 
 #if defined(_AGAR_INTERNAL) || defined(_USE_AGAR_QUEUE)
 
-#define SLIST_HEAD			AG_SLIST_HEAD
-#define SLIST_HEAD_			AG_SLIST_HEAD_
-#define SLIST_HEAD_INITIALIZER		AG_SLIST_HEAD_INITIALIZER
-#define SLIST_ENTRY			AG_SLIST_ENTRY
-#define SLIST_FIRST			AG_SLIST_FIRST
-#define SLIST_END			AG_SLIST_END
-#define SLIST_EMPTY			AG_SLIST_EMPTY
-#define SLIST_NEXT			AG_SLIST_NEXT
-#define SLIST_FOREACH			AG_SLIST_FOREACH
-#define SLIST_INIT			AG_SLIST_INIT
-#define SLIST_INSERT_AFTER		AG_SLIST_INSERT_AFTER
-#define SLIST_INSERT_HEAD		AG_SLIST_INSERT_HEAD
-#define SLIST_REMOVE_HEAD		AG_SLIST_REMOVE_HEAD
-#define SLIST_REMOVE			AG_SLIST_REMOVE
+#define SLIST_HEAD(name,t)			AG_SLIST_HEAD(name,t)
+#define SLIST_HEAD_(t)				AG_SLIST_HEAD_(t)
+#define SLIST_HEAD_INITIALIZER(head)		AG_SLIST_HEAD_INITIALIZER(head)
+#define SLIST_ENTRY(t)				AG_SLIST_ENTRY(t)
+#define SLIST_FIRST(head)			AG_SLIST_FIRST(head)
+#define SLIST_END(head)				AG_SLIST_END(head)
+#define SLIST_EMPTY(head)			AG_SLIST_EMPTY(head)
+#define SLIST_NEXT(elm,field)			AG_SLIST_NEXT(elm,field)
+#define SLIST_FOREACH(var,head,field)		AG_SLIST_FOREACH(var,head,field)
+#define SLIST_INIT(head)			AG_SLIST_INIT(head)
+#define SLIST_INSERT_AFTER(slistelm,elm,field)	AG_SLIST_INSERT_AFTER(slistelm,elm,field)
+#define SLIST_INSERT_HEAD(head,elm,field)	AG_SLIST_INSERT_HEAD(head,elm,field)
+#define SLIST_REMOVE_HEAD(head,field)		AG_SLIST_REMOVE_HEAD(head,field)
+#define SLIST_REMOVE(head,elm,t,field)		AG_SLIST_REMOVE(head,elm,t,field)
 
-#define LIST_HEAD			AG_LIST_HEAD
-#define LIST_HEAD_			AG_LIST_HEAD_
-#define LIST_HEAD_INITIALIZER		AG_LIST_HEAD_INITIALIZER
-#define LIST_ENTRY			AG_LIST_ENTRY
-#define LIST_FIRST			AG_LIST_FIRST
-#define LIST_END			AG_LIST_END
-#define LIST_EMPTY			AG_LIST_EMPTY
-#define LIST_NEXT			AG_LIST_NEXT
-#define LIST_FOREACH			AG_LIST_FOREACH
-#define LIST_INIT			AG_LIST_INIT
-#define LIST_INSERT_AFTER		AG_LIST_INSERT_AFTER
-#define LIST_INSERT_BEFORE		AG_LIST_INSERT_BEFORE
-#define LIST_INSERT_HEAD		AG_LIST_INSERT_HEAD
-#define LIST_REMOVE			AG_LIST_REMOVE
-#define LIST_REPLACE			AG_LIST_REPLACE
+#define LIST_HEAD(name,t)			AG_LIST_HEAD(name,t)
+#define LIST_HEAD_(t)				AG_LIST_HEAD_(t)
+#define LIST_HEAD_INITIALIZER(head)		AG_LIST_HEAD_INITIALIZER(head)
+#define LIST_ENTRY(t)				AG_LIST_ENTRY(t)
+#define LIST_FIRST(head)			AG_LIST_FIRST(head)
+#define LIST_END(head)				AG_LIST_END(head)
+#define LIST_EMPTY(head)			AG_LIST_EMPTY(head)
+#define LIST_NEXT(elm,field)			AG_LIST_NEXT(elm,field)
+#define LIST_FOREACH(var,head,field)		AG_LIST_FOREACH(var,head,field)
+#define LIST_INIT(head)				AG_LIST_INIT(head)
+#define LIST_INSERT_AFTER(listelm,elm,field)	AG_LIST_INSERT_AFTER(listelm,elm,field)
+#define LIST_INSERT_BEFORE(listelm,elm,field)	AG_LIST_INSERT_BEFORE(listelm,elm,field)
+#define LIST_INSERT_HEAD(head,elm,field)	AG_LIST_INSERT_HEAD(head,elm,field)
+#define LIST_REMOVE(elm,field)			AG_LIST_REMOVE(elm,field)
+#define LIST_REPLACE(elm,elm2,field)		AG_LIST_REPLACE(elm,elm2,field)
 
-#define SIMPLEQ_HEAD			AG_SIMPLEQ_HEAD
-#define SIMPLEQ_HEAD_			AG_SIMPLEQ_HEAD_
-#define SIMPLEQ_HEAD_INITIALIZER	AG_SIMPLEQ_HEAD_INITIALIZER
-#define SIMPLEQ_ENTRY			AG_SIMPLEQ_ENTRY
-#define SIMPLEQ_FIRST			AG_SIMPLEQ_FIRST
-#define SIMPLEQ_END			AG_SIMPLEQ_END
-#define SIMPLEQ_EMPTY			AG_SIMPLEQ_EMPTY
-#define SIMPLEQ_NEXT			AG_SIMPLEQ_NEXT
-#define SIMPLEQ_FOREACH			AG_SIMPLEQ_FOREACH
-#define SIMPLEQ_INIT			AG_SIMPLEQ_INIT
-#define SIMPLEQ_INSERT_HEAD		AG_SIMPLEQ_INSERT_HEAD
-#define SIMPLEQ_INSERT_TAIL		AG_SIMPLEQ_INSERT_TAIL
-#define SIMPLEQ_INSERT_AFTER		AG_SIMPLEQ_INSERT_AFTER
-#define SIMPLEQ_REMOVE_HEAD		AG_SIMPLEQ_REMOVE_HEAD
+#define SIMPLEQ_HEAD(name,t)				AG_SIMPLEQ_HEAD(name,t)
+#define SIMPLEQ_HEAD_(t)				AG_SIMPLEQ_HEAD_(t)
+#define SIMPLEQ_HEAD_INITIALIZER(head)			AG_SIMPLEQ_HEAD_INITIALIZER(head)
+#define SIMPLEQ_ENTRY(t)				AG_SIMPLEQ_ENTRY(t)
+#define SIMPLEQ_FIRST(head)				AG_SIMPLEQ_FIRST(head)
+#define SIMPLEQ_END(head)				AG_SIMPLEQ_END(head)
+#define SIMPLEQ_EMPTY(head)				AG_SIMPLEQ_EMPTY(head)
+#define SIMPLEQ_NEXT(elm,field)				AG_SIMPLEQ_NEXT(elm,field)
+#define SIMPLEQ_FOREACH(var,head,field)			AG_SIMPLEQ_FOREACH(var,head,field)
+#define SIMPLEQ_INIT(head)				AG_SIMPLEQ_INIT(head)
+#define SIMPLEQ_INSERT_HEAD(head,elm,field)		AG_SIMPLEQ_INSERT_HEAD(head,elm,field)
+#define SIMPLEQ_INSERT_TAIL(head,elm,field)		AG_SIMPLEQ_INSERT_TAIL(head,elm,field)
+#define SIMPLEQ_INSERT_AFTER(head,listelm,elm,field)	AG_SIMPLEQ_INSERT_AFTER(head,listelm,elm,field)
+#define SIMPLEQ_REMOVE_HEAD(head,elm,field)		AG_SIMPLEQ_REMOVE_HEAD(head,elm,field)
 
-#define TAILQ_HEAD			AG_TAILQ_HEAD
-#define TAILQ_HEAD_			AG_TAILQ_HEAD_
-#define TAILQ_HEAD_INITIALIZER		AG_TAILQ_HEAD_INITIALIZER
-#define TAILQ_ENTRY			AG_TAILQ_ENTRY
-#define TAILQ_FIRST			AG_TAILQ_FIRST
-#define TAILQ_END			AG_TAILQ_END
-#define TAILQ_NEXT			AG_TAILQ_NEXT
-#define TAILQ_LAST			AG_TAILQ_LAST
-#define TAILQ_PREV			AG_TAILQ_PREV
-#define TAILQ_EMPTY			AG_TAILQ_EMPTY
-#define TAILQ_FOREACH			AG_TAILQ_FOREACH
-#define TAILQ_FOREACH_REVERSE		AG_TAILQ_FOREACH_REVERSE
-#define TAILQ_INIT			AG_TAILQ_INIT
-#define TAILQ_INSERT_HEAD		AG_TAILQ_INSERT_HEAD
-#define TAILQ_INSERT_TAIL		AG_TAILQ_INSERT_TAIL
-#define TAILQ_INSERT_AFTER		AG_TAILQ_INSERT_AFTER
-#define TAILQ_INSERT_BEFORE		AG_TAILQ_INSERT_BEFORE
-#define TAILQ_REMOVE			AG_TAILQ_REMOVE
-#define TAILQ_REPLACE			AG_TAILQ_REPLACE
+#define TAILQ_HEAD(name,t)				AG_TAILQ_HEAD(name,t)
+#define TAILQ_HEAD_(t)					AG_TAILQ_HEAD_(t)
+#define TAILQ_HEAD_INITIALIZER(head)			AG_TAILQ_HEAD_INITIALIZER(head)
+#define TAILQ_ENTRY(t)					AG_TAILQ_ENTRY(t)
+#define TAILQ_FIRST(head)				AG_TAILQ_FIRST(head)
+#define TAILQ_END(head)					AG_TAILQ_END(head)
+#define TAILQ_NEXT(elm,field)				AG_TAILQ_NEXT(elm,field)
+#define TAILQ_LAST(head,headname)			AG_TAILQ_LAST(head,headname)
+#define TAILQ_PREV(elm,headname,field)			AG_TAILQ_PREV(elm,headname,field)
+#define TAILQ_EMPTY(head)				AG_TAILQ_EMPTY(head)
+#define TAILQ_INIT(head)				AG_TAILQ_INIT(head)
+#define TAILQ_FOREACH(var,head,field)			AG_TAILQ_FOREACH(var,head,field)
+#define TAILQ_FOREACH_REVERSE(var,head,headname,field)	AG_TAILQ_FOREACH_REVERSE(var,head,headname,field)
+#define TAILQ_INSERT_HEAD(head,elm,field)		AG_TAILQ_INSERT_HEAD(head,elm,field)
+#define TAILQ_INSERT_TAIL(head,elm,field)		AG_TAILQ_INSERT_TAIL(head,elm,field)
+#define TAILQ_INSERT_AFTER(head,listelm,elm,field)	AG_TAILQ_INSERT_AFTER(head,listelm,elm,field)
+#define TAILQ_INSERT_BEFORE(listelm,elm,field)		AG_TAILQ_INSERT_BEFORE(listelm,elm,field)
+#define TAILQ_REMOVE(head,elm,field)			AG_TAILQ_REMOVE(head,elm,field)
+#define TAILQ_REPLACE(head,elm,elm2,field)		AG_TAILQ_REPLACE(head,elm,elm2,field)
 
-#define CIRCLEQ_HEAD			AG_CIRCLEQ_HEAD
-#define CIRCLEQ_HEAD_			AG_CIRCLEQ_HEAD_
-#define CIRCLEQ_HEAD_INITIALIZER	AG_CIRCLEQ_HEAD_INITIALIZER
-#define CIRCLEQ_ENTRY			AG_CIRCLEQ_ENTRY
-#define CIRCLEQ_FIRST			AG_CIRCLEQ_FIRST
-#define CIRCLEQ_LAST			AG_CIRCLEQ_LAST
-#define CIRCLEQ_END			AG_CIRCLEQ_END
-#define CIRCLEQ_NEXT			AG_CIRCLEQ_NEXT
-#define CIRCLEQ_PREV			AG_CIRCLEQ_PREV
-#define CIRCLEQ_EMPTY			AG_CIRCLEQ_EMPTY
-#define CIRCLEQ_FOREACH			AG_CIRCLEQ_FOREACH
-#define CIRCLEQ_FOREACH_REVERSE		AG_CIRCLEQ_FOREACH_REVERSE
-#define CIRCLEQ_INIT			AG_CIRCLEQ_INIT
-#define CIRCLEQ_INSERT_AFTER		AG_CIRCLEQ_INSERT_AFTER
-#define CIRCLEQ_INSERT_BEFORE		AG_CIRCLEQ_INSERT_BEFORE
-#define CIRCLEQ_INSERT_HEAD		AG_CIRCLEQ_INSERT_HEAD
-#define CIRCLEQ_INSERT_TAIL		AG_CIRCLEQ_INSERT_TAIL
-#define CIRCLEQ_REMOVE			AG_CIRCLEQ_REMOVE
-#define CIRCLEQ_REPLACE			AG_CIRCLEQ_REPLACE
+#define CIRCLEQ_HEAD(name,t)				AG_CIRCLEQ_HEAD(name,t)
+#define CIRCLEQ_HEAD_(t)				AG_CIRCLEQ_HEAD_(t)
+#define CIRCLEQ_HEAD_INITIALIZER(head)			AG_CIRCLEQ_HEAD_INITIALIZER(head)
+#define CIRCLEQ_ENTRY(t)				AG_CIRCLEQ_ENTRY(t)
+#define CIRCLEQ_FIRST(head)				AG_CIRCLEQ_FIRST(head)
+#define CIRCLEQ_LAST(head)				AG_CIRCLEQ_LAST(head)
+#define CIRCLEQ_END(head)				AG_CIRCLEQ_END(head)
+#define CIRCLEQ_NEXT(elm,field)				AG_CIRCLEQ_NEXT(elm,field)
+#define CIRCLEQ_PREV(elm,field)				AG_CIRCLEQ_PREV(elm,field)
+#define CIRCLEQ_EMPTY(head)				AG_CIRCLEQ_EMPTY(head)
+#define CIRCLEQ_INIT(head)				AG_CIRCLEQ_INIT(head)
+#define CIRCLEQ_FOREACH(var,head,field)			AG_CIRCLEQ_FOREACH(var,head,field)
+#define CIRCLEQ_FOREACH_REVERSE(var,head,field)		AG_CIRCLEQ_FOREACH_REVERSE
+#define CIRCLEQ_INSERT_AFTER(head,listelm,elm,field)	AG_CIRCLEQ_INSERT_AFTER(head,listelm,elm,field)
+#define CIRCLEQ_INSERT_BEFORE(head,listelm,elm,field)	AG_CIRCLEQ_INSERT_BEFORE(head,listelm,elm,field)
+#define CIRCLEQ_INSERT_HEAD(head,elm,field)		AG_CIRCLEQ_INSERT_HEAD(head,elm,field)
+#define CIRCLEQ_INSERT_TAIL(head,elm,field)		AG_CIRCLEQ_INSERT_TAIL(head,elm,field)
+#define CIRCLEQ_REMOVE(head,elm,field)			AG_CIRCLEQ_REMOVE(head,elm,field)
+#define CIRCLEQ_REPLACE(head,elm,elm2,field)		AG_CIRCLEQ_REPLACE(head,elm,elm2,field)
 
 #endif /* _AGAR_INTERNAL or _USE_AGAR_QUEUE */
 
