@@ -24,7 +24,7 @@
 # include <agar/core/begin.h>
 /* Begin generated block */
 __BEGIN_DECLS
-extern DECLSPEC size_t AG_Snprintf(char *, size_t, const char *, ...);
+extern DECLSPEC AG_Size AG_Snprintf(char *_Nonnull, AG_Size, const char *_Nonnull, ...);
 __END_DECLS
 /* Close generated block */
 # include <agar/core/close.h>
