@@ -61,6 +61,7 @@ typedef enum ag_variable_type {
 					   (and hard dependency) */
 	AG_VARIABLE_P_VARIABLE,		/* Serializable reference to specific
 					   Object Variable (by name) */
+	AG_VARIABLE_FUNCTION,		/* A function with optional arguments */
 	AG_VARIABLE_TYPE_LAST
 } AG_VariableType;
 #if AG_MODEL == AG_SMALL
@@ -84,10 +85,8 @@ typedef struct ag_variable_type_info {
 	AG_VariableType typeTgt;   /* Pointer target type (if indirLvl > 0) */
 	int code;                  /* Numerical code (-1 = not serializable) */
 	AG_Size size;              /* Size in bytes */
+	AG_SIZE_PADDING(_pad);
 #endif /* !AG_SMALL */
-#if AG_MODEL == AG_MEDIUM
-	Uint32 _pad;
-#endif
 } AG_VariableTypeInfo;
 
 /* Variable-stored data */
@@ -150,7 +149,7 @@ typedef struct ag_variable {
 #endif
 	union {
 		Uint pFlags;           /* Pointer flags (for [P_]POINTER) */
-#define AG_VARIABLE_P_READONLY 0x01    /* Hard const (need AG_CONST_PTR()) */
+#define AG_VARIABLE_P_READONLY 0x01    /* Hard const (needs AG_cPTR()) */
 #define AG_VARIABLE_P_FREE     0x02    /* Auto free() target on cleanup */
 #define AG_VARIABLE_P_SENDER   0x04    /* Is a pointer to a sender Object */
 		union {                /* Bitmask (for P_FLAG_*) */
@@ -172,6 +171,33 @@ typedef struct ag_variable {
 #define AG_VARIABLE_TYPE(V)      (agVariableTypes[(V)->type].typeTgt)
 #define AG_VARIABLE_TYPE_NAME(V) (agVariableTypes[(V)->type].name)
 
+/*
+ * Description of a bit in a word with writeability information (and
+ * optional user-defined flags).
+ */
+typedef struct ag_flag_descr {
+#ifdef AG_HAVE_64BIT
+	Uint64 bitmask;                 /* Bitmask */
+#else
+	Uint bitmask;                   /* Bitmask */
+#endif
+	const char *_Nonnull descr;     /* Description (UTF-8) */
+	int writeable;                  /* Bit is user-editable? */
+	Uint flags;                     /* User-defined flags */
+} AG_FlagDescr;
+
+/*
+ * Description of a bit in a word without writeability information.
+ */
+typedef struct ag_flag_descr_ro {
+#ifdef AG_HAVE_64BIT
+	Uint64 bitmask;			/* Bitmask */
+#else
+	Uint bitmask;			/* Bitmask */
+#endif
+	const char *_Nonnull descr;	/* Description (UTF-8) */
+} AG_FlagDescrRO;
+
 /* Begin generated block */
 __BEGIN_DECLS
 extern DECLSPEC const AG_VariableTypeInfo agVariableTypes[];
@@ -193,8 +219,10 @@ extern DECLSPEC void AG_InitUint(AG_Variable *_Nonnull, Uint);
 #endif
 extern DECLSPEC AG_Variable *_Nonnull AG_SetUint(void *_Nonnull, const char *_Nonnull, Uint);
 extern DECLSPEC AG_Variable *_Nonnull AG_BindUint(void *_Nonnull, const char *_Nonnull, Uint *_Nonnull);
-#ifdef AG_THREADS
+#if defined(AG_THREADS)
 extern DECLSPEC AG_Variable *_Nonnull AG_BindUintMp(void *_Nonnull, const char *_Nonnull, Uint *_Nonnull, _Nonnull_Mutex AG_Mutex *_Nonnull);
+#else
+# define AG_BindUintMp(o,v,p,m) AG_BindUint((o),(v),(p))
 #endif
 /*
  * INT: Natural integer
@@ -207,6 +235,8 @@ extern DECLSPEC AG_Variable *_Nonnull AG_SetInt(void *_Nonnull, const char *_Non
 extern DECLSPEC AG_Variable *_Nonnull AG_BindInt(void *_Nonnull, const char *_Nonnull, int *_Nonnull);
 #ifdef AG_THREADS
 extern DECLSPEC AG_Variable *_Nonnull AG_BindIntMp(void *_Nonnull, const char *_Nonnull, int *_Nonnull, _Nonnull_Mutex AG_Mutex *_Nonnull);
+#else
+# define AG_BindIntMp(o,v,p,m) AG_BindInt((o),(v),(p))
 #endif
 /*
  * BOOL: Alias for INT
@@ -225,6 +255,8 @@ extern DECLSPEC AG_Variable *_Nonnull AG_SetUlong(void *_Nonnull, const char *_N
 extern DECLSPEC AG_Variable *_Nonnull AG_BindUlong(void *_Nonnull, const char *_Nonnull, Ulong *_Nonnull);
 # ifdef AG_THREADS
 extern DECLSPEC AG_Variable *_Nonnull AG_BindUlongMp(void *_Nonnull, const char *_Nonnull, Ulong *_Nonnull, _Nonnull_Mutex AG_Mutex *_Nonnull);
+# else
+# define AG_BindUlongMp(o,v,p,m) AG_BindUlong((o),(v),(p))
 # endif
 /*
  * LONG: Natural long integer.
@@ -235,6 +267,8 @@ extern DECLSPEC AG_Variable *_Nonnull AG_SetLong(void *_Nonnull, const char *_No
 extern DECLSPEC AG_Variable *_Nonnull AG_BindLong(void *_Nonnull, const char *_Nonnull, long *_Nonnull);
 # ifdef AG_THREADS
 extern DECLSPEC AG_Variable *_Nonnull AG_BindLongMp(void *_Nonnull, const char *_Nonnull, long *_Nonnull, _Nonnull_Mutex AG_Mutex *_Nonnull);
+# else
+# define AG_BindLongMp(o,v,p,m) AG_BindLong((o),(v),(p))
 # endif
 #endif 
 /*
@@ -248,6 +282,8 @@ extern DECLSPEC AG_Variable *_Nonnull AG_SetUint8(void *_Nonnull, const char *_N
 extern DECLSPEC AG_Variable *_Nonnull AG_BindUint8(void *_Nonnull, const char *_Nonnull, Uint8 *_Nonnull);
 #ifdef AG_THREADS
 extern DECLSPEC AG_Variable *_Nonnull AG_BindUint8Mp(void *_Nonnull, const char *_Nonnull, Uint8 *_Nonnull, _Nonnull_Mutex AG_Mutex *_Nonnull);
+#else
+# define AG_BindUint8Mp(o,v,p,m) AG_BindUint8((o),(v),(p))
 #endif
 /*
  * UINT8: Signed 8-bit integer
@@ -260,6 +296,8 @@ extern DECLSPEC AG_Variable *_Nonnull AG_SetSint8(void *_Nonnull, const char *_N
 extern DECLSPEC AG_Variable *_Nonnull AG_BindSint8(void *_Nonnull, const char *_Nonnull, Sint8 *_Nonnull);
 #ifdef AG_THREADS
 extern DECLSPEC AG_Variable *_Nonnull AG_BindSint8Mp(void *_Nonnull, const char *_Nonnull, Sint8 *_Nonnull, _Nonnull_Mutex AG_Mutex *_Nonnull);
+#else
+# define AG_BindSint8Mp(o,v,p,m) AG_BindSint8((o),(v),(p))
 #endif
 #if AG_MODEL != AG_SMALL
 /*
@@ -271,6 +309,8 @@ extern DECLSPEC AG_Variable *_Nonnull AG_SetUint16(void *_Nonnull, const char *_
 extern DECLSPEC AG_Variable *_Nonnull AG_BindUint16(void *_Nonnull, const char *_Nonnull, Uint16 *_Nonnull);
 # ifdef AG_THREADS
 extern DECLSPEC AG_Variable *_Nonnull AG_BindUint16Mp(void *_Nonnull, const char *_Nonnull, Uint16 *_Nonnull, _Nonnull_Mutex AG_Mutex *_Nonnull);
+# else
+# define AG_BindUint16Mp(o,v,p,m) AG_BindUint16((o),(v),(p))
 # endif
 /*
  * SINT16: Signed 16-bit integer
@@ -281,6 +321,8 @@ extern DECLSPEC AG_Variable *_Nonnull AG_SetSint16(void *_Nonnull, const char *_
 extern DECLSPEC AG_Variable *_Nonnull AG_BindSint16(void *_Nonnull, const char *_Nonnull, Sint16 *_Nonnull);
 # ifdef AG_THREADS
 extern DECLSPEC AG_Variable *_Nonnull AG_BindSint16Mp(void *_Nonnull, const char *_Nonnull, Sint16 *_Nonnull, _Nonnull_Mutex AG_Mutex *_Nonnull);
+# else
+# define AG_BindSint16Mp(o,v,p,m) AG_BindSint16((o),(v),(p))
 # endif
 /*
  * UINT32: Unsigned 32-bit integer
@@ -291,6 +333,8 @@ extern DECLSPEC AG_Variable *_Nonnull AG_SetUint32(void *_Nonnull, const char *_
 extern DECLSPEC AG_Variable *_Nonnull AG_BindUint32(void *_Nonnull, const char *_Nonnull, Uint32 *_Nonnull);
 # ifdef AG_THREADS
 extern DECLSPEC AG_Variable *_Nonnull AG_BindUint32Mp(void *_Nonnull, const char *_Nonnull, Uint32 *_Nonnull, _Nonnull_Mutex AG_Mutex *_Nonnull);
+# else
+# define AG_BindUint32Mp(o,v,p,m) AG_BindUint32((o),(v),(p))
 # endif
 /*
  * SINT32: Signed 32-bit integer
@@ -301,7 +345,9 @@ extern DECLSPEC AG_Variable *_Nonnull AG_SetSint32(void *_Nonnull, const char *_
 extern DECLSPEC AG_Variable *_Nonnull AG_BindSint32(void *_Nonnull, const char *_Nonnull, Sint32 *_Nonnull);
 # ifdef AG_THREADS
 extern DECLSPEC AG_Variable *_Nonnull AG_BindSint32Mp(void *_Nonnull, const char *_Nonnull, Sint32 *_Nonnull, _Nonnull_Mutex AG_Mutex *_Nonnull);
-# endif 
+# else
+# define AG_BindSint32Mp(o,v,p,m) AG_BindSint32((o),(v),(p))
+# endif
 #endif 
 #ifdef AG_HAVE_64BIT
 /*
@@ -313,6 +359,8 @@ extern DECLSPEC AG_Variable *_Nonnull AG_SetUint64(void *_Nonnull, const char *_
 extern DECLSPEC AG_Variable *_Nonnull AG_BindUint64(void *_Nonnull, const char *_Nonnull, Uint64 *_Nonnull);
 # ifdef AG_THREADS
 extern DECLSPEC AG_Variable *_Nonnull AG_BindUint64Mp(void *_Nonnull, const char *_Nonnull, Uint64 *_Nonnull, _Nonnull_Mutex AG_Mutex *_Nonnull);
+# else
+# define AG_BindUint64Mp(o,v,p,m) AG_BindUint64((o),(v),(p))
 # endif
 /*
  * SINT64: Signed 64-bit integer
@@ -323,6 +371,8 @@ extern DECLSPEC AG_Variable *_Nonnull AG_SetSint64(void *_Nonnull, const char *_
 extern DECLSPEC AG_Variable *_Nonnull AG_BindSint64(void *_Nonnull, const char *_Nonnull, Sint64 *_Nonnull);
 # ifdef AG_THREADS
 extern DECLSPEC AG_Variable *_Nonnull AG_BindSint64Mp(void *_Nonnull, const char *_Nonnull, Sint64 *_Nonnull, _Nonnull_Mutex AG_Mutex *_Nonnull);
+# else
+# define AG_BindSint64Mp(o,v,p,m) AG_BindSint64((o),(v),(p))
 # endif
 #endif 
 #ifdef AG_HAVE_FLOAT
@@ -335,6 +385,8 @@ extern DECLSPEC AG_Variable *_Nonnull AG_SetFloat(void *_Nonnull, const char *_N
 extern DECLSPEC AG_Variable *_Nonnull AG_BindFloat(void *_Nonnull, const char *_Nonnull, float *_Nonnull);
 # ifdef AG_THREADS
 extern DECLSPEC AG_Variable *_Nonnull AG_BindFloatMp(void *_Nonnull, const char *_Nonnull, float *_Nonnull, _Nonnull_Mutex AG_Mutex *_Nonnull);
+# else
+# define AG_BindFloatMp(o,v,p,m) AG_BindFloat((o),(v),(p))
 # endif
 /*
  * DOUBLE: Double-precision IEEE float.
@@ -345,6 +397,8 @@ extern DECLSPEC AG_Variable *_Nonnull AG_SetDouble(void *_Nonnull, const char *_
 extern DECLSPEC AG_Variable *_Nonnull AG_BindDouble(void *_Nonnull, const char *_Nonnull, double *_Nonnull);
 # ifdef AG_THREADS
 extern DECLSPEC AG_Variable *_Nonnull AG_BindDoubleMp(void *_Nonnull, const char *_Nonnull, double *_Nonnull, _Nonnull_Mutex AG_Mutex *_Nonnull);
+# else
+# define AG_BindDoubleMp(o,v,p,m) AG_BindDouble((o),(v),(p))
 # endif
 #endif 
 /*
@@ -366,6 +420,8 @@ extern DECLSPEC void AG_InitStringNODUP(AG_Variable *_Nonnull, char *_Nonnull);
 extern DECLSPEC AG_Variable *_Nonnull AG_BindString(void *_Nonnull, const char *_Nonnull, char *_Nonnull, AG_Size);
 #ifdef AG_THREADS
 extern DECLSPEC AG_Variable *_Nonnull AG_BindStringMp(void *_Nonnull, const char *_Nonnull, char *_Nonnull, AG_Size, _Nonnull_Mutex AG_Mutex *_Nonnull);
+#else
+# define AG_BindStringMp(o,v,p,s,m) AG_BindString((o),(v),(p),(s))
 #endif
 /*
  * POINTER: A generic pointer or memory address.
@@ -376,6 +432,8 @@ extern DECLSPEC AG_Variable *_Nonnull AG_SetPointer(void *_Nonnull, const char *
 extern DECLSPEC AG_Variable *_Nonnull AG_BindPointer(void *_Nonnull, const char *_Nonnull, void *_Nonnull *_Nullable);
 #ifdef AG_THREADS
 extern DECLSPEC AG_Variable *_Nonnull AG_BindPointerMp(void *_Nonnull, const char *_Nonnull, void *_Nonnull *_Nullable, _Nonnull_Mutex AG_Mutex *_Nonnull);
+#else
+# define AG_BindPointerMp(o,v,p,m) AG_BindPointer((o),(v),(p))
 #endif
 #if AG_MODEL != AG_SMALL
 extern DECLSPEC const void *_Nullable AG_GetConstPointer(void *_Nonnull, const char *_Nonnull) _Pure_Attribute_If_Unthreaded;
@@ -388,6 +446,8 @@ extern DECLSPEC AG_Variable *_Nonnull AG_SetConstPointer(void *_Nonnull, const c
 extern DECLSPEC AG_Variable *_Nonnull AG_BindFlag(void *_Nonnull, const char *_Nonnull, Uint *_Nonnull, Uint);
 #ifdef AG_THREADS
 extern DECLSPEC AG_Variable *_Nonnull AG_BindFlagMp(void *_Nonnull, const char *_Nonnull, Uint *_Nonnull, Uint, _Nonnull_Mutex AG_Mutex *_Nonnull);
+#else
+# define AG_BindFlagMp(o,v,p,b,m) AG_BindFlag((o),(v),(p),(b))
 #endif
 extern DECLSPEC AG_Variable *_Nonnull AG_BindFlag8(void *_Nonnull, const char *_Nonnull, Uint8 *_Nonnull, Uint8);
 #if AG_MODEL != AG_SMALL
@@ -398,6 +458,10 @@ extern DECLSPEC AG_Variable *_Nonnull AG_BindFlag32(void *_Nonnull, const char *
 extern DECLSPEC AG_Variable *_Nonnull AG_BindFlag8Mp(void *_Nonnull, const char *_Nonnull, Uint8 *_Nonnull, Uint8, _Nonnull_Mutex AG_Mutex *_Nonnull);
 extern DECLSPEC AG_Variable *_Nonnull AG_BindFlag16Mp(void *_Nonnull, const char *_Nonnull, Uint16 *_Nonnull, Uint16, _Nonnull_Mutex AG_Mutex *_Nonnull);
 extern DECLSPEC AG_Variable *_Nonnull AG_BindFlag32Mp(void *_Nonnull, const char *_Nonnull, Uint32 *_Nonnull, Uint32, _Nonnull_Mutex AG_Mutex *_Nonnull);
+#else
+# define AG_BindFlag8Mp(o,v,p,b,m) AG_BindFlag8((o),(v),(p),(b))
+# define AG_BindFlag16Mp(o,v,p,b,m) AG_BindFlag16((o),(v),(p),(b))
+# define AG_BindFlag32Mp(o,v,p,b,m) AG_BindFlag32((o),(v),(p),(b))
 #endif
 extern DECLSPEC AG_Variable *_Nonnull AG_BindObject(void *_Nonnull, const char *_Nonnull, void *_Nonnull);
 extern DECLSPEC AG_Variable *_Nonnull AG_BindVariable(void *_Nonnull, const char *_Nonnull, void *_Nonnull, const char *_Nonnull);
