@@ -1,5 +1,7 @@
 /*	Public domain	*/
+
 #ifdef _XBOX
+
 # include <agar/core/queue_close.h>		/* Naming conflicts */
 # include <xtl.h>
 # include <agar/core/queue_close.h>		/* Naming conflicts */
@@ -14,9 +16,9 @@ typedef struct {
 } ANSI_STRING, *PANSI_STRING;
 
 typedef struct {
-    HANDLE RootDirectory;
-    PANSI_STRING ObjectName;
-    ULONG Attributes;
+	HANDLE RootDirectory;
+	PANSI_STRING ObjectName;
+	ULONG Attributes;
 } OBJECT_ATTRIBUTES, *POBJECT_ATTRIBUTES;
 
 typedef struct {
@@ -42,25 +44,31 @@ typedef struct {
 
 /* Undocumented externals */
 extern PANSI_STRING XeImageFileName;
-XBOXAPI LONG WINAPI RtlInitAnsiString(OUT PANSI_STRING DestinationString, IN LPCSTR SourceString);
+XBOXAPI LONG WINAPI RtlInitAnsiString(OUT PANSI_STRING DestinationString,
+                                      IN LPCSTR SourceString);
 XBOXAPI INT WINAPI XWriteTitleInfoAndRebootA(LPVOID,LPVOID,DWORD,DWORD,LPVOID);
-XBOXAPI NTSTATUS WINAPI NtOpenSymbolicLinkObject(OUT PHANDLE LinkHandle, IN POBJECT_ATTRIBUTES ObjectAttributes);
-XBOXAPI NTSTATUS WINAPI NtQuerySymbolicLinkObject(IN HANDLE LinkHandle, IN OUT PANSI_STRING LinkTarget,
-												  OUT PULONG ReturnedLength OPTIONAL);
-XBOXAPI NTSTATUS WINAPI NtCreateFile(OUT PHANDLE FileHandle, IN ACCESS_MASK DesiredAccess,
-									 IN POBJECT_ATTRIBUTES ObjectAttributes,  OUT PIO_STATUS_BLOCK IoStatusBlock,
-									 IN PLARGE_INTEGER AllocationSize OPTIONAL, IN ULONG FileAttributes,
-									 IN ULONG ShareAccess, IN ULONG CreateDisposition, IN ULONG CreateOptions);
+XBOXAPI NTSTATUS WINAPI NtOpenSymbolicLinkObject(OUT PHANDLE LinkHandle,
+                                                 IN POBJECT_ATTRIBUTES ObjectAttributes);
+XBOXAPI NTSTATUS WINAPI NtQuerySymbolicLinkObject(IN HANDLE LinkHandle,
+                                                  IN OUT PANSI_STRING LinkTarget,
+                                                  OUT PULONG ReturnedLength OPTIONAL);
+XBOXAPI NTSTATUS WINAPI NtCreateFile(OUT PHANDLE FileHandle,
+                                     IN ACCESS_MASK DesiredAccess,
+                                     IN POBJECT_ATTRIBUTES ObjectAttributes,
+                                     OUT PIO_STATUS_BLOCK IoStatusBlock,
+                                     IN PLARGE_INTEGER AllocationSize OPTIONAL, IN ULONG FileAttributes,
+                                     IN ULONG ShareAccess, IN ULONG CreateDisposition, IN ULONG CreateOptions);
 
-// Custom LAUNCH_DATA struct for external XBE execution from AG_Execute()
+/* Custom LAUNCH_DATA struct for external XBE execution from AG_Execute(). */
 #define AG_LAUNCH_MAGIC 0x41474152
 
 typedef struct {
-	DWORD magic;               // Test this against AG_LAUNCH_MAGIC to know this special struct was used
-	DWORD dwID;                // The Title ID of the launcher XBE
-	CHAR  szLauncherXBE[256];  // The full path to the launcher XBE
-	CHAR  szLaunchedXBE[256];  // The full path to the launched XBE
-	CHAR  szCmdLine[MAX_LAUNCH_DATA_SIZE - 520]; // The command-line parameters
+	DWORD magic;                 /* Test this against AG_LAUNCH_MAGIC to
+	                                know this special struct was used */
+	DWORD dwID;                  /* Title ID of the launcher XBE */
+	CHAR szLauncherXBE[256];     /* Full path to the launcher XBE */
+	CHAR szLaunchedXBE[256];     /* Full path to the launched XBE */
+	CHAR szCmdLine[MAX_LAUNCH_DATA_SIZE-520]; /* Command-line parameters */
 } AG_LAUNCH_DATA, *PAG_LAUNCH_DATA;
 
 #include <agar/core/begin.h>
@@ -75,4 +83,4 @@ __END_DECLS
 /* Close generated block */
 #include <agar/core/close.h>
 
-#endif
+#endif /* _XBOX */
