@@ -60,14 +60,15 @@ typedef struct ag_box {
 #define AG_VBOX_VFILL      AG_BOX_VFILL
 #define AG_VBOX_EXPAND    (AG_BOX_HFILL | AG_BOX_VFILL)
 
-#define AGBOX(obj)            ((AG_Box *)(obj))
-#define AGCBOX(obj)           ((const AG_Box *)(obj))
-#define AG_BOX_SELF()          AGBOX( AG_OBJECT(0,"AG_Widget:AG_Box:*") )
-#define AG_BOX_PTR(n)          AGBOX( AG_OBJECT((n),"AG_Widget:AG_Box:*") )
-#define AG_BOX_NAMED(n)        AGBOX( AG_OBJECT_NAMED((n),"AG_Widget:AG_Box:*") )
-#define AG_CONST_BOX_SELF()   AGCBOX( AG_CONST_OBJECT(0,"AG_Widget:AG_Box:*") )
-#define AG_CONST_BOX_PTR(n)   AGCBOX( AG_CONST_OBJECT((n),"AG_Widget:AG_Box:*") )
-#define AG_CONST_BOX_NAMED(n) AGCBOX( AG_CONST_OBJECT_NAMED((n),"AG_Widget:AG_Box:*") )
+#define   AGBOX(o)       ((AG_Box *)(o))
+#define  AGcBOX(o)       ((const AG_Box *)(o))
+#define  AG_BOX_ISA(o)   (((AGOBJECT(o)->cid & 0xff000000) >> 24) == 0x09)
+#define  AG_BOX_SELF()    AGBOX(  AG_OBJECT(0,         "AG_Widget:AG_Box:*") )
+#define  AG_BOX_PTR(n)    AGBOX(  AG_OBJECT((n),       "AG_Widget:AG_Box:*") )
+#define  AG_BOX_NAMED(n)  AGBOX(  AG_OBJECT_NAMED((n), "AG_Widget:AG_Box:*") )
+#define AG_cBOX_SELF()   AGcBOX( AG_cOBJECT(0,         "AG_Widget:AG_Box:*") )
+#define AG_cBOX_PTR(n)   AGcBOX( AG_cOBJECT((n),       "AG_Widget:AG_Box:*") )
+#define AG_cBOX_NAMED(n) AGcBOX( AG_cOBJECT_NAMED((n), "AG_Widget:AG_Box:*") )
 
 /* Begin generated block */
 __BEGIN_DECLS
