@@ -50,6 +50,9 @@ typedef struct ag_driver_mw_class {
 	void (*_Nullable tweakAlignment)(struct ag_window *_Nonnull,
 	                                 struct ag_size_alloc *_Nonnull,
 	                                 Uint,Uint);
+	/* Set window size bounds */
+	void (*_Nullable setWindowMinSize)(struct ag_window *_Nonnull, int,int);
+	void (*_Nullable setWindowMaxSize)(struct ag_window *_Nonnull, int,int);
 } AG_DriverMwClass;
 
 typedef struct ag_driver_mw {
@@ -57,11 +60,20 @@ typedef struct ag_driver_mw {
 	struct ag_window *_Nullable win;	/* Back pointer to window */
 	Uint flags;
 #define AG_DRIVER_MW_OPEN	0x01		/* Rendering is operational */
-	Uint32 _pad;
+	Uint32 windowID;			/* Optional window ID */
 } AG_DriverMw;
 
-#define AGDRIVER_MW(obj) ((AG_DriverMw *)(obj))
-#define AGDRIVER_MW_CLASS(obj) ((struct ag_driver_mw_class *)(AGOBJECT(obj)->cls))
+#define  AGDRIVERMW(o)       ((AG_DriverMw *)(o))
+#define AGcDRIVERMW(o)       ((const AG_DriverMw *)(o))
+#define  AG_DRIVER_MW_ISA(o) (((AGOBJECT(o)->cid & 0xffff0000) >> 16) == 0x0502)
+#define  AG_DRIVER_MW_SELF()    AGDRIVERMW(  AG_OBJECT(0,        "AG_Driver:AG_DriverMw:*") )
+#define  AG_DRIVER_MW_PTR(n)    AGDRIVERMW(  AG_OBJECT((n),      "AG_Driver:AG_DriverMw:*") )
+#define  AG_DRIVER_MW_NAMED(n)  AGDRIVERMW(  AG_OBJECT_NAMED((n),"AG_Driver:AG_DriverMw:*") )
+#define AG_cDRIVER_MW_SELF()   AGcDRIVERMW( AG_cOBJECT(0,        "AG_Driver:AG_DriverMw:*") )
+#define AG_cDRIVER_MW_PTR(n)   AGcDRIVERMW( AG_cOBJECT((n),      "AG_Driver:AG_DriverMw:*") )
+#define AG_cDRIVER_MW_NAMED(n) AGcDRIVERMW( AG_cOBJECT_NAMED((n),"AG_Driver:AG_DriverMw:*") )
+
+#define AGDRIVER_MW_CLASS(o) ((struct ag_driver_mw_class *)(AGOBJECT(o)->cls))
 
 /* Flags to openWindow */
 #define AG_DRIVER_MW_ANYPOS		0x01	/* Autoposition window */
