@@ -15,6 +15,7 @@
 
 #include <agar/gui/begin.h>
 
+struct ag_dir;
 struct ag_file_dlg;
 
 enum ag_file_type_option_type {
@@ -46,7 +47,9 @@ typedef struct ag_file_type {
 	char *_Nonnull allExts;            /* Filename extensions/patterns */
 	char *_Nonnull *_Nonnull exts;     /* Extensions/patterns as an array */
 	Uint                    nExts;
-	Uint32 _pad;
+	Uint flags;
+#define AG_FILE_TYPE_SELECTED 0x01         /* Type is selected */
+
 	AG_Event *_Nullable action;                /* Load/save callback */
 	AG_TAILQ_HEAD_(ag_file_type_option) opts;  /* Type-specific options */
 	AG_TAILQ_ENTRY(ag_file_type) types;
@@ -76,6 +79,7 @@ typedef struct ag_file_dlg {
 	char cfile[AG_PATHNAME_MAX];		/* Current file path */
 	int fdDir;				/* Open directory FD */
 
+	AG_Dir *_Nullable openDir;		/* Open AG_Dir(3) */
 	AG_EventSink *_Nullable esFollow;	/* Open directory event sink */
 	AG_Pane *_Nonnull hPane;		/* Horizontal split container */
 	AG_Tlist *_Nonnull tlDirs;		/* List of directories */
@@ -103,14 +107,15 @@ typedef struct ag_file_dlg {
 	AG_TAILQ_HEAD_(ag_file_type) types;	/* File type handlers */
 } AG_FileDlg;
 
-#define AGFILEDLG(obj)            ((AG_FileDlg *)(obj))
+#define  AGFILEDLG(obj)            ((AG_FileDlg *)(obj))
 #define AGCFILEDLG(obj)           ((const AG_FileDlg *)(obj))
+#define AG_FILEDLG_ISA(o) (((AGOBJECT(o)->cid & 0xff000000) >> 24) == 0x11)
 #define AG_FILEDLG_SELF()          AGFILEDLG( AG_OBJECT(0,"AG_Widget:AG_FileDlg:*") )
 #define AG_FILEDLG_PTR(n)          AGFILEDLG( AG_OBJECT((n),"AG_Widget:AG_FileDlg:*") )
 #define AG_FILEDLG_NAMED(n)        AGFILEDLG( AG_OBJECT_NAMED((n),"AG_Widget:AG_FileDlg:*") )
-#define AG_CONST_FILEDLG_SELF()   AGCFILEDLG( AG_CONST_OBJECT(0,"AG_Widget:AG_FileDlg:*") )
-#define AG_CONST_FILEDLG_PTR(n)   AGCFILEDLG( AG_CONST_OBJECT((n),"AG_Widget:AG_FileDlg:*") )
-#define AG_CONST_FILEDLG_NAMED(n) AGCFILEDLG( AG_CONST_OBJECT_NAMED((n),"AG_Widget:AG_FileDlg:*") )
+#define AG_cFILEDLG_SELF()   AGcFILEDLG( AG_cOBJECT(0,"AG_Widget:AG_FileDlg:*") )
+#define AG_cFILEDLG_PTR(n)   AGcFILEDLG( AG_cOBJECT((n),"AG_Widget:AG_FileDlg:*") )
+#define AG_cFILEDLG_NAMED(n) AGcFILEDLG( AG_cOBJECT_NAMED((n),"AG_Widget:AG_FileDlg:*") )
 
 /* Begin generated block */
 __BEGIN_DECLS
@@ -135,6 +140,7 @@ extern DECLSPEC int AG_FileDlgCheckReadAccess(AG_FileDlg *_Nonnull);
 extern DECLSPEC int AG_FileDlgCheckWriteAccess(AG_FileDlg *_Nonnull);
 extern DECLSPEC void AG_FileDlgRefresh(AG_FileDlg *_Nonnull);
 extern DECLSPEC void AG_FileDlgAddImageTypes(AG_FileDlg *, AG_EventFn, const char *, ...);
+extern DECLSPEC void AG_FileDlgSelectType(AG_FileDlg *_Nonnull, AG_FileType *_Nullable);
 extern DECLSPEC void AG_FileDlgCopyTypes(AG_FileDlg *_Nonnull _Restrict, const AG_FileDlg *_Nonnull _Restrict);
 extern DECLSPEC AG_FileType *_Nonnull AG_FileDlgAddType(AG_FileDlg *_Nonnull, const char *_Nonnull, const char *_Nonnull, _Nullable AG_EventFn, const char *_Nullable, ...);
 extern DECLSPEC AG_FileOption *_Nonnull AG_FileOptionNewBool(AG_FileType *_Nonnull, const char *_Nonnull, const char *_Nonnull, int);
