@@ -19,11 +19,8 @@ struct ag_fixed_plotter;
 typedef struct ag_fixed_plotter_item {
 	char name[AG_FIXED_PLOTTER_LABEL_MAX];		/* Description */
 	AG_Color color;					/* Line color */
-
+	AG_COLOR_PADDING(pad);
 	Uint32                        nvals;		/* Values total */
-#if AG_MODEL == AG_MEDIUM
-	Uint32 _pad;
-#endif
 	AG_FixedPlotterValue *_Nonnull vals;		/* Values array */
 	Uint32                      maxvals;		/* Values allocated */
 	Uint32                        limit;		/* Hard limit */
@@ -39,30 +36,30 @@ enum ag_fixed_plotter_type {
 };
 
 typedef struct ag_fixed_plotter {
-	struct ag_widget wid;		/* AG_Widget -> AG_FixedPlotter */
-
+	struct ag_widget wid;                /* AG_Widget -> AG_FixedPlotter */
 	enum ag_fixed_plotter_type type;
 	Uint flags;
-#define AG_FIXED_PLOTTER_SCROLL	0x01	/* Scroll if the end is not visible */
-#define AG_FIXED_PLOTTER_XAXIS	0x02	/* Display X axis */
-#define AG_FIXED_PLOTTER_HFILL	0x04
-#define AG_FIXED_PLOTTER_VFILL	0x08
-#define AG_FIXED_PLOTTER_EXPAND (AG_FIXED_PLOTTER_HFILL|AG_FIXED_PLOTTER_VFILL)
-	AG_FixedPlotterValue yrange;		/* Max. value */
-	AG_FixedPlotterValue xoffs;		/* Display offset */
-	int yOrigin;				/* Origin position (%) */
+#define AG_FIXED_PLOTTER_SCROLL 0x01         /* Scroll if end is not visible */
+#define AG_FIXED_PLOTTER_XAXIS  0x02         /* Display X axis */
+#define AG_FIXED_PLOTTER_HFILL  0x04
+#define AG_FIXED_PLOTTER_VFILL  0x08
+#define AG_FIXED_PLOTTER_EXPAND (AG_FIXED_PLOTTER_HFILL | AG_FIXED_PLOTTER_VFILL)
+	AG_FixedPlotterValue yrange;         /* Max. value */
+	AG_FixedPlotterValue xoffs;          /* Display offset */
+	int yOrigin;                         /* Origin position (%) */
 	Uint32 _pad;
-	struct ag_fixed_plotter_itemq items;	/* Items to plot */
+	struct ag_fixed_plotter_itemq items; /* Sample points */
 } AG_FixedPlotter;
 
-#define AGFIXEDPLOTTER(obj)            ((AG_FixedPlotter *)(obj))
-#define AGCFIXEDPLOTTER(obj)           ((const AG_FixedPlotter *)(obj))
-#define AG_FIXEDPLOTTER_SELF()          AGFIXEDPLOTTER( AG_OBJECT(0,"AG_Widget:AG_FixedPlotter:*") )
-#define AG_FIXEDPLOTTER_PTR(n)          AGFIXEDPLOTTER( AG_OBJECT((n),"AG_Widget:AG_FixedPlotter:*") )
-#define AG_FIXEDPLOTTER_NAMED(n)        AGFIXEDPLOTTER( AG_OBJECT_NAMED((n),"AG_Widget:AG_FixedPlotter:*") )
-#define AG_CONST_FIXEDPLOTTER_SELF()   AGCFIXEDPLOTTER( AG_CONST_OBJECT(0,"AG_Widget:AG_FixedPlotter:*") )
-#define AG_CONST_FIXEDPLOTTER_PTR(n)   AGCFIXEDPLOTTER( AG_CONST_OBJECT((n),"AG_Widget:AG_FixedPlotter:*") )
-#define AG_CONST_FIXEDPLOTTER_NAMED(n) AGCFIXEDPLOTTER( AG_CONST_OBJECT_NAMED((n),"AG_Widget:AG_FixedPlotter:*") )
+#define   AGFIXEDPLOTTER(o)     ((AG_FixedPlotter *)(o))
+#define  AGcFIXEDPLOTTER(o)     ((const AG_FixedPlotter *)(o))
+#define  AG_FIXEDPLOTTER_ISA(o) (((AGOBJECT(o)->cid & 0xff000000) >> 24) == 0x13)
+#define  AG_FIXEDPLOTTER_SELF()    AGFIXEDPLOTTER(  AG_OBJECT(0,        "AG_Widget:AG_FixedPlotter:*") )
+#define  AG_FIXEDPLOTTER_PTR(n)    AGFIXEDPLOTTER(  AG_OBJECT((n),      "AG_Widget:AG_FixedPlotter:*") )
+#define  AG_FIXEDPLOTTER_NAMED(n)  AGFIXEDPLOTTER(  AG_OBJECT_NAMED((n),"AG_Widget:AG_FixedPlotter:*") )
+#define AG_cFIXEDPLOTTER_SELF()   AGcFIXEDPLOTTER( AG_cOBJECT(0,        "AG_Widget:AG_FixedPlotter:*") )
+#define AG_cFIXEDPLOTTER_PTR(n)   AGcFIXEDPLOTTER( AG_cOBJECT((n),      "AG_Widget:AG_FixedPlotter:*") )
+#define AG_cFIXEDPLOTTER_NAMED(n) AGcFIXEDPLOTTER( AG_cOBJECT_NAMED((n),"AG_Widget:AG_FixedPlotter:*") )
 
 /* Begin generated block */
 __BEGIN_DECLS
