@@ -93,14 +93,15 @@ typedef struct ag_graph {
 	AG_Rect r;			 /* Display area */
 } AG_Graph;
 
-#define AGGRAPH(obj)            ((AG_Graph *)(obj))
-#define AGCGRAPH(obj)           ((const AG_Graph *)(obj))
-#define AG_GRAPH_SELF()          AGGRAPH( AG_OBJECT(0,"AG_Widget:AG_Graph:*") )
-#define AG_GRAPH_PTR(n)          AGGRAPH( AG_OBJECT((n),"AG_Widget:AG_Graph:*") )
-#define AG_GRAPH_NAMED(n)        AGGRAPH( AG_OBJECT_NAMED((n),"AG_Widget:AG_Graph:*") )
-#define AG_CONST_GRAPH_SELF()   AGCGRAPH( AG_CONST_OBJECT(0,"AG_Widget:AG_Graph:*") )
-#define AG_CONST_GRAPH_PTR(n)   AGCGRAPH( AG_CONST_OBJECT((n),"AG_Widget:AG_Graph:*") )
-#define AG_CONST_GRAPH_NAMED(n) AGCGRAPH( AG_CONST_OBJECT_NAMED((n),"AG_Widget:AG_Graph:*") )
+#define   AGGRAPH(obj)    ((AG_Graph *)(obj))
+#define  AGcGRAPH(obj)    ((const AG_Graph *)(obj))
+#define  AG_GRAPH_ISA(o)  (((AGOBJECT(o)->cid & 0xff000000) >> 24) == 0x14)
+#define  AG_GRAPH_SELF()    AGGRAPH(  AG_OBJECT(0,         "AG_Widget:AG_Graph:*") )
+#define  AG_GRAPH_PTR(n)    AGGRAPH(  AG_OBJECT((n),       "AG_Widget:AG_Graph:*") )
+#define  AG_GRAPH_NAMED(n)  AGGRAPH(  AG_OBJECT_NAMED((n), "AG_Widget:AG_Graph:*") )
+#define AG_cGRAPH_SELF()   AGcGRAPH( AG_cOBJECT(0,         "AG_Widget:AG_Graph:*") )
+#define AG_cGRAPH_PTR(n)   AGcGRAPH( AG_cOBJECT((n),       "AG_Widget:AG_Graph:*") )
+#define AG_cGRAPH_NAMED(n) AGcGRAPH( AG_cOBJECT_NAMED((n), "AG_Widget:AG_Graph:*") )
 
 /* Begin generated block */
 __BEGIN_DECLS
