@@ -11,14 +11,14 @@ typedef struct ag_static_icon {
 	Uint32 Gmask;			/* Green mask */
 	Uint32 Bmask;			/* Blue mask */
 	Uint32 Amask;			/* Alpha mask */
-	const Uint32 *data;		/* Pixel data */
-	AG_Surface *s;			/* Initialized surface */
+	const Uint32 *_Nonnull data;	/* Pixel data */
+	AG_Surface *_Nullable s;	/* Initialized surface */
 } AG_StaticIcon;
 
 /* Begin generated block */
 __BEGIN_DECLS
-extern DECLSPEC void AG_InitStaticIcon(AG_StaticIcon *);
-extern DECLSPEC void AG_FreeStaticIcon(AG_StaticIcon *);
+extern DECLSPEC void AG_InitStaticIcon(AG_StaticIcon *_Nonnull);
+extern DECLSPEC void AG_FreeStaticIcon(AG_StaticIcon *_Nonnull);
 __END_DECLS
 /* Close generated block */
 
