@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2019 Julien Nadeau Carriere <vedge@csoft.net>
+ * Copyright (c) 2009-2023 Julien Nadeau Carriere <vedge@csoft.net>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,28 +35,33 @@ ag_pixel_format_is_supported(AG_SurfaceMode mode, int BitsPerPixel)
 	switch (mode) {
 	case AG_SURFACE_PACKED:
 		switch (BitsPerPixel) {
-		case 8:
-		case 16:
-		case 24:
 		case 32:
-		case 48:
+		case 24:
+#if AG_MODEL == AG_LARGE
 		case 64:
+		case 48:
+		case 40:
+#endif
+		case 16:
 			return (1);
 		}
 		break;
 	case AG_SURFACE_INDEXED:
 		switch (BitsPerPixel) {
-		case 1:
-		case 2:
-		case 4:
 		case 8:
+		case 4:
+		case 2:
+		case 1:
 			return (1);
 		}
 		break;
 	case AG_SURFACE_GRAYSCALE:
 		switch (BitsPerPixel) {
+		case 16:
 		case 32:
+#if AG_MODEL == AG_LARGE
 		case 64:
+#endif
 			return (1);
 		}
 		break;
@@ -83,7 +88,6 @@ ag_pixel_format_compare(const AG_PixelFormat *a, const AG_PixelFormat *b)
 	}
 	switch (a->mode) {
 	case AG_SURFACE_PACKED:
-	default:
 		return !(a->Rmask == b->Rmask &&
 			 a->Gmask == b->Gmask &&
 			 a->Bmask == b->Bmask &&
@@ -99,8 +103,9 @@ ag_pixel_format_compare(const AG_PixelFormat *a, const AG_PixelFormat *b)
 #endif
 		return memcmp(a->palette->colors, b->palette->colors,
 		              a->palette->nColors * sizeof(AG_Color));
+	default:
 	case AG_SURFACE_GRAYSCALE:
-		return !(a->graymode == b->graymode);
+		return (0);
 	}
 }
 
@@ -139,9 +144,11 @@ ag_map_pixel32(const AG_PixelFormat *pf, const AG_Color *c)
 }
 
 #if AG_MODEL == AG_LARGE
-# define AG_EXTRACT_COMPONENT(rv, mask, shift, loss, bits) tmp = (px & mask) >> shift; (rv) = (tmp << loss) + (tmp >> (bits - (loss << 1)));
+
+# define AG_EXTRACT_COMPONENT16(rv, mask, shift, loss) tmp = (px & mask) >> shift; (rv) = (tmp << loss) | (tmp >> (16 - (loss << 1)))
+
 /*
- * Map 16-bit RGB components to a 64-bit (1- to 64-bpp) pixel.
+ * Map 16bpc RGB components to a 64-bit (1- to 64-bpp) pixel.
  */
 # ifdef AG_INLINE_HEADER
 static __inline__ Uint64 _Pure_Attribute
@@ -167,7 +174,7 @@ ag_map_pixel64_rgb16(const AG_PixelFormat *pf, Uint16 r, Uint16 g, Uint16 b)
 }
 
 /*
- * Map 16-bit RGBA components to a 64-bit (1- to 64-bpp) pixel.
+ * Map 16bpc RGBA components to a 64-bit (1- to 64-bpp) pixel.
  */
 # ifdef AG_INLINE_HEADER
 static __inline__ Uint64 _Pure_Attribute
@@ -182,10 +189,10 @@ ag_map_pixel64_rgba16(const AG_PixelFormat *pf,
 	switch (pf->mode) {
 	case AG_SURFACE_PACKED:
 	default:
-		return (r >> pf->Rloss) << pf->Rshift |
-		       (g >> pf->Gloss) << pf->Gshift |
-		       (b >> pf->Bloss) << pf->Bshift |
-		      ((a >> pf->Aloss) << pf->Ashift & pf->Amask);
+		return ((Uint64)r >> pf->Rloss) << pf->Rshift |
+		       ((Uint64)g >> pf->Gloss) << pf->Gshift |
+		       ((Uint64)b >> pf->Bloss) << pf->Bshift |
+		      (((Uint64)a >> pf->Aloss) << pf->Ashift & pf->Amask);
 	case AG_SURFACE_INDEXED:
 		return AG_MapPixelIndexed(pf, r,g,b,a);
 	case AG_SURFACE_GRAYSCALE:
@@ -220,32 +227,27 @@ ag_get_color64_rgb16(Uint64 px, const AG_PixelFormat *pf,
     Uint16 *r, Uint16 *g, Uint16 *b)
 # endif
 {
-	Uint64 tmp;
-	AG_Color *c;
+	if (pf->mode == AG_SURFACE_PACKED) {
+		Uint64 tmp;
 
-	switch (pf->mode) {
-	case AG_SURFACE_PACKED:
-		AG_EXTRACT_COMPONENT(*r, pf->Rmask, pf->Rshift, pf->Rloss, 16);
-		AG_EXTRACT_COMPONENT(*g, pf->Gmask, pf->Gshift, pf->Gloss, 16);
-		AG_EXTRACT_COMPONENT(*b, pf->Bmask, pf->Bshift, pf->Bloss, 16);
-		break;
-	case AG_SURFACE_INDEXED:
-		c = &pf->palette->colors[px % pf->palette->nColors];
+		AG_EXTRACT_COMPONENT16(*r, pf->Rmask, pf->Rshift, pf->Rloss);
+		AG_EXTRACT_COMPONENT16(*g, pf->Gmask, pf->Gshift, pf->Gloss);
+		AG_EXTRACT_COMPONENT16(*b, pf->Bmask, pf->Bshift, pf->Bloss);
+	} else if (pf->mode == AG_SURFACE_INDEXED) {
+		const AG_Color *c = &pf->palette->colors[px %
+		                                         pf->palette->nColors];
 		*r = c->r;
 		*g = c->g;
 		*b = c->b;
-		break;
-	case AG_SURFACE_GRAYSCALE: {
+	} else if (pf->mode == AG_SURFACE_GRAYSCALE) {
 		Uint16 dummy;
-		AG_GetColor64_Gray16(px, pf->graymode, r,g,b, &dummy);
-		break;
-	}
-	/* pf->mode */
+
+		AG_GetColor64_Gray16(px, r,g,b, &dummy);
 	}
 }
 
 /*
- * Extract 16-bit RGBA components from a 64-bit (1- to 64-bpp) pixel.
+ * Extract 16bpc RGBA components from a 64-bit (1- to 64-bpp) pixel.
  */
 # ifdef AG_INLINE_HEADER
 static __inline__ void
@@ -258,26 +260,23 @@ ag_get_color64_rgba16(Uint64 px, const AG_PixelFormat *pf,
     Uint16 *r, Uint16 *g, Uint16 *b, Uint16 *a)
 # endif
 {
-	Uint64 tmp;
-	AG_Color *c;
+	if (pf->mode == AG_SURFACE_PACKED) {
+		Uint64 tmp;
+
+		AG_EXTRACT_COMPONENT16(*r, pf->Rmask, pf->Rshift, pf->Rloss);
+		AG_EXTRACT_COMPONENT16(*g, pf->Gmask, pf->Gshift, pf->Gloss);
+		AG_EXTRACT_COMPONENT16(*b, pf->Bmask, pf->Bshift, pf->Bloss);
+		AG_EXTRACT_COMPONENT16(*a, pf->Amask, pf->Ashift, pf->Aloss);
+	} else if (pf->mode == AG_SURFACE_INDEXED) {
+		const AG_Color *c = &pf->palette->colors[px %
+		                                         pf->palette->nColors];
 
-	switch (pf->mode) {
-	case AG_SURFACE_PACKED:
-		AG_EXTRACT_COMPONENT(*r, pf->Rmask, pf->Rshift, pf->Rloss, 16);
-		AG_EXTRACT_COMPONENT(*g, pf->Gmask, pf->Gshift, pf->Gloss, 16);
-		AG_EXTRACT_COMPONENT(*b, pf->Bmask, pf->Bshift, pf->Bloss, 16);
-		AG_EXTRACT_COMPONENT(*a, pf->Amask, pf->Ashift, pf->Aloss, 16);
-		break;
-	case AG_SURFACE_INDEXED:
-		c = &pf->palette->colors[px % pf->palette->nColors];
 		*r = c->r;
 		*g = c->g;
 		*b = c->b;
 		*a = c->a;
-		break;
-	case AG_SURFACE_GRAYSCALE:
-		AG_GetColor64_Gray16(px, pf->graymode, r,g,b,a);
-		break;
+	} else if (pf->mode == AG_SURFACE_GRAYSCALE) {
+		AG_GetColor64_Gray16(px, r,g,b,a);
 	}
 }
 
@@ -294,16 +293,16 @@ ag_get_color64_rgb8(Uint64 px, const AG_PixelFormat *pf,
     Uint8 *r, Uint8 *g, Uint8 *b)
 # endif
 {
-	Uint16 R,G,B;
+	Uint16 r16, g16, b16;
 
-	AG_GetColor64_RGB16(px, pf, &R,&G,&B);
-	*r = AG_16to8(R);
-	*g = AG_16to8(G);
-	*b = AG_16to8(B);
+	AG_GetColor64_RGB16(px, pf, &r16, &g16, &b16);
+	*r = AG_16to8(r16);
+	*g = AG_16to8(g16);
+	*b = AG_16to8(b16);
 }
 
 /*
- * Extract compressed 8-bit RGBA components from a 64-bit (1- to 64-bpp) pixel.
+ * Extract compressed 8bpc RGBA components from a 64-bit (1- to 64-bpp) pixel.
  */
 # ifdef AG_INLINE_HEADER
 static __inline__ void
@@ -316,16 +315,16 @@ ag_get_color64_rgba8(Uint64 px, const AG_PixelFormat *pf,
     Uint8 *r, Uint8 *g, Uint8 *b, Uint8 *a)
 # endif
 {
-	Uint16 R,G,B,A;
+	Uint16 r16, g16, b16, a16;
 
-	AG_GetColor64_RGBA16(px, pf, &R,&G,&B,&A);
-	*r = AG_16to8(R);
-	*g = AG_16to8(G);
-	*b = AG_16to8(B);
-	*a = AG_16to8(A);
+	AG_GetColor64_RGBA16(px, pf, &r16, &g16, &b16, &a16);
+	*r = AG_16to8(r16);
+	*g = AG_16to8(g16);
+	*b = AG_16to8(b16);
+	*a = AG_16to8(a16);
 }
 
-# undef AG_EXTRACT_COMPONENT
+# undef AG_EXTRACT_COMPONENT16
 #endif /* AG_LARGE */
 
 /*
@@ -340,7 +339,7 @@ Uint32
 ag_surface_get32(const AG_Surface *S, int x, int y)
 #endif
 {
-#ifdef AG_DEBUG
+#ifdef DEBUG_SURFACE_GET
 	if (x < 0 || y < 0 || x >= S->w || y >= S->h)
 		AG_FatalErrorF("Illegal AG_SurfaceGet32() access (%d,%d in %ux%u)", x,y, S->w, S->h);
 #endif
@@ -348,7 +347,7 @@ ag_surface_get32(const AG_Surface *S, int x, int y)
 		return (Uint32)AG_SurfaceGet8(S, x,y);
 	} else {
 		return AG_SurfaceGet32_At(S, S->pixels +
-		    y*S->pitch +
+		    y*S->pitch + S->Lpadding +
 		    x*S->format.BytesPerPixel);
 	}
 }
@@ -366,28 +365,76 @@ ag_surface_get32_at(const AG_Surface *S, const Uint8 *p)
 #endif
 {
 	switch (S->format.BitsPerPixel) {
-#if AG_MODEL == AG_LARGE
-	case 64:
-	case 48: {
-		AG_Color c;
-		AG_GetColor64(&c, *(Uint64 *)p, &S->format);
-		return AG_MapPixel32(&S->format, &c);
-	}
-#endif
 	case 32:
 		return (*(Uint32 *)p);
 	case 24:
 #if AG_BYTEORDER == AG_BIG_ENDIAN
-		return ((p[0] << 16) +
-		        (p[1] << 8) +
+		return ((p[0] << 16) |
+		        (p[1] << 8) |
 		         p[2]);
 #else
-		return  (p[0] +
-		        (p[1] << 8) +
+		return  (p[0] |
+		        (p[1] << 8) |
 		        (p[2] << 16));
 #endif
+#if AG_MODEL == AG_LARGE
+	case 64:
+		{
+			AG_Color c;
+
+			AG_GetColor64(&c, *(Uint64 *)p, &S->format);
+			return AG_MapPixel32(&S->format, &c);
+		}
+		break;
+	case 48:
+		{
+			Uint64 px;
+			AG_Color c;
+
+# if AG_BYTEORDER == AG_BIG_ENDIAN
+			px = ((Uint64)p[0] << 40) |
+			     ((Uint64)p[1] << 32) |
+			     ((Uint64)p[2] << 24) |
+			     ((Uint64)p[3] << 16) |
+			     ((Uint64)p[4] <<  8) |
+			      (Uint64)p[5];
+# else
+			px =  (Uint64)p[0] |
+			     ((Uint64)p[1] <<  8) |
+			     ((Uint64)p[2] << 16) |
+			     ((Uint64)p[3] << 24) |
+			     ((Uint64)p[4] << 32) |
+			     ((Uint64)p[5] << 40);
+# endif
+			AG_GetColor64(&c, px, &S->format);
+			return AG_MapPixel32(&S->format, &c);
+		}
+	case 40:
+		{
+			Uint64 px;
+			AG_Color c;
+
+# if AG_BYTEORDER == AG_BIG_ENDIAN
+			px = (((Uint64)p[0] << 32) |
+			      ((Uint64)p[1] << 24) |
+			      ((Uint64)p[2] << 16) |
+			      ((Uint64)p[3] <<  8) |
+			       (Uint64)p[4];
+# else
+			px =   (Uint64)p[0] |
+			      ((Uint64)p[1] <<  8) |
+			      ((Uint64)p[2] << 16) |
+			      ((Uint64)p[3] << 24) |
+			      ((Uint64)p[4] << 32);
+# endif
+			AG_GetColor64(&c, px, &S->format);
+			return AG_MapPixel32(&S->format, &c);
+		}
+#endif /* AG_LARGE */
 	case 16:
 		return (*(Uint16 *)p);
+	default:
+		break;
 	}
 	return (*p);
 }
@@ -404,15 +451,16 @@ Uint64
 ag_surface_get64(const AG_Surface *S, int x, int y)
 # endif
 {
-# ifdef AG_DEBUG
+# ifdef DEBUG_SURFACE_GET
 	if (x < 0 || y < 0 || x >= S->w || y >= S->h)
-		AG_FatalErrorF("Illegal AG_SurfaceGet64() access (%d,%d in %ux%u)", x,y, S->w, S->h);
+		AG_FatalErrorF("Illegal AG_SurfaceGet64() access "
+		               "(%d,%d in %ux%u)", x, y, S->w, S->h);
 # endif
 	if (S->format.BitsPerPixel < 8) {
 		return (Uint64)AG_SurfaceGet8(S, x,y);
 	} else {
 		return AG_SurfaceGet64_At(S, S->pixels +
-		    y*S->pitch +
+		    y*S->pitch + S->Lpadding +
 		    x*S->format.BytesPerPixel);
 	}
 }
@@ -429,38 +477,54 @@ ag_surface_get64_at(const AG_Surface *S, const Uint8 *p)
 # endif
 {
 	switch (S->format.BitsPerPixel) {
-	case 64:
-		return (*(Uint64 *)p);
-	case 48:
-# if AG_BYTEORDER == AG_BIG_ENDIAN
-		return (((Uint64)p[0] << 40) +
-		        ((Uint64)p[1] << 32) +
-		        ((Uint64)p[2] << 24) +
-		        ((Uint64)p[3] << 16) +
-		        ((Uint64)p[4] << 8) +
-		         (Uint64)p[5]);
-# else
-		return  ((Uint64)p[0] +
-		        ((Uint64)p[1] << 8) +
-		        ((Uint64)p[2] << 16) +
-			((Uint64)p[3] << 24) +
-			((Uint64)p[4] << 32) +
-			((Uint64)p[5] << 40));
-# endif
 	case 32:
 		return (*(Uint32 *)p);
 	case 24:
 # if AG_BYTEORDER == AG_BIG_ENDIAN
-		return ((p[0] << 16) +
-		        (p[1] << 8) +
+		return ((p[0] << 16) |
+		        (p[1] <<  8) |
 		         p[2]);
 # else
-		return  (p[0] +
-		        (p[1] << 8) +
+		return  (p[0] |
+		        (p[1] <<  8) |
 		        (p[2] << 16));
+# endif
+	case 64:
+		return (*(Uint64 *)p);
+	case 40:
+# if AG_BYTEORDER == AG_BIG_ENDIAN
+		return ((Uint64)p[0] << 32) |
+		       ((Uint64)p[1] << 24) |
+		       ((Uint64)p[2] << 16) |
+		       ((Uint64)p[3] <<  8) |
+		        (Uint64)p[4];
+# else
+		return  (Uint64)p[0] |
+		       ((Uint64)p[1] <<  8) |
+		       ((Uint64)p[2] << 16) |
+		       ((Uint64)p[3] << 24) |
+		       ((Uint64)p[4] << 32);
+# endif
+	case 48:
+# if AG_BYTEORDER == AG_BIG_ENDIAN
+		return ((Uint64)p[0] << 40) |
+		       ((Uint64)p[1] << 32) |
+		       ((Uint64)p[2] << 24) |
+		       ((Uint64)p[3] << 16) |
+		       ((Uint64)p[4] <<  8) |
+		        (Uint64)p[5];
+# else
+		return  (Uint64)p[0] |
+		       ((Uint64)p[1] <<  8) |
+		       ((Uint64)p[2] << 16) |
+		       ((Uint64)p[3] << 24) |
+		       ((Uint64)p[4] << 32) |
+		       ((Uint64)p[5] << 40);
 # endif
 	case 16:
 		return (*(Uint16 *)p);
+	default:
+		break;
 	}
 	return (*p);
 }
@@ -482,13 +546,13 @@ ag_surface_put32(AG_Surface *S, int x, int y, Uint32 px)
 		AG_SurfacePut8(S, x,y, (Uint8)px);
 	} else {
 		AG_SurfacePut32_At(S, S->pixels +
-		    y*S->pitch +
+		    y*S->pitch + S->Lpadding +
 		    x*S->format.BytesPerPixel, px);
 	}
 }
 
 /*
- * Write to the (8- to 32-bit) pixel at address p in surface S.
+ * Write to the (8- to 64-bit) pixel at address p in surface S.
  */
 #ifdef AG_INLINE_HEADER
 static __inline__ void
@@ -499,15 +563,6 @@ ag_surface_put32_at(AG_Surface *S, Uint8 *p, Uint32 px)
 #endif
 {
 	switch (S->format.BitsPerPixel) {
-#if AG_MODEL == AG_LARGE
-	case 64:                                             /* Decompressed */
-	case 48: {
-		AG_Color c;
-		AG_GetColor32(&c, px, &S->format);
-		*(Uint64 *)p = AG_MapPixel64(&S->format, &c);
-		break;
-	}
-#endif
 	case 32:
 		*(Uint32 *)p = px;
 		break;
@@ -522,6 +577,64 @@ ag_surface_put32_at(AG_Surface *S, Uint8 *p, Uint32 px)
 		p[2] = (px >> 16) & 0xff;
 #endif
 		break;
+
+#if AG_MODEL == AG_LARGE
+	case 64:
+		{
+			AG_Color c;
+
+			AG_GetColor32(&c, px, &S->format);
+			*(Uint64 *)p = AG_MapPixel64(&S->format, &c);
+			break;
+		}
+	case 48:
+		{
+			AG_Color c;
+			Uint64 px64;
+
+			AG_GetColor32(&c, px, &S->format);
+			px64 = AG_MapPixel64(&S->format, &c);
+# if AG_BYTEORDER == AG_BIG_ENDIAN
+			p[0] = (px64 >> 40) & 0xff;
+			p[1] = (px64 >> 32) & 0xff;
+			p[2] = (px64 >> 24) & 0xff;
+			p[3] = (px64 >> 16) & 0xff;
+			p[4] = (px64 >> 8)  & 0xff;
+			p[5] = (px64)       & 0xff;
+# else
+			p[0] = (px64)       & 0xff;
+			p[1] = (px64 >> 8)  & 0xff;
+			p[2] = (px64 >> 16) & 0xff;
+			p[3] = (px64 >> 24) & 0xff;
+			p[4] = (px64 >> 32) & 0xff;
+			p[5] = (px64 >> 40) & 0xff;
+# endif
+			break;
+		}
+	case 40:
+		{
+			AG_Color c;
+			Uint64 px64;
+
+			AG_GetColor32(&c, px, &S->format);
+			px64 = AG_MapPixel64(&S->format, &c);
+# if AG_BYTEORDER == AG_BIG_ENDIAN
+			p[0] = (px64 >> 32) & 0xff;
+			p[1] = (px64 >> 24) & 0xff;
+			p[2] = (px64 >> 16) & 0xff;
+			p[3] = (px64 >> 8)  & 0xff;
+			p[4] = (px64)       & 0xff;
+# else
+			p[0] = (px64)       & 0xff;
+			p[1] = (px64 >> 8)  & 0xff;
+			p[2] = (px64 >> 16) & 0xff;
+			p[3] = (px64 >> 24) & 0xff;
+			p[4] = (px64 >> 32) & 0xff;
+# endif
+			break;
+		}
+#endif /* AG_LARGE */
+
 	case 16:
 		*(Uint16 *)p = (Uint16)px;
 		break;
@@ -547,7 +660,7 @@ ag_surface_put64(AG_Surface *S, int x, int y, Uint64 px)
 		AG_SurfacePut8(S, x,y, (Uint8)px);
 	} else {
 		AG_SurfacePut64_At(S, S->pixels +
-		    y*S->pitch +
+		    y*S->pitch + S->Lpadding +
 		    x*S->format.BytesPerPixel, px);
 	}
 }
@@ -564,13 +677,12 @@ ag_surface_put64_at(AG_Surface *S, Uint8 *p, Uint64 px)
 # endif
 {
 	switch (S->format.BitsPerPixel) {
-	case 64:
-	case 48:
-		*(Uint64 *)p = px;
-		break;
 	case 32:
 		*(Uint32 *)p = px;
 		break;
+	case 64:
+		*(Uint64 *)p = px;
+		break;
 	case 24:
 #if AG_BYTEORDER == AG_BIG_ENDIAN
 		p[0] = (px >> 16) & 0xff;
@@ -580,11 +692,44 @@ ag_surface_put64_at(AG_Surface *S, Uint8 *p, Uint64 px)
 		p[0] = (px)       & 0xff;
 		p[1] = (px >> 8)  & 0xff;
 		p[2] = (px >> 16) & 0xff;
+#endif
+		break;
+	case 48:
+#if AG_BYTEORDER == AG_BIG_ENDIAN
+		p[0] = (px >> 40) & 0xff;
+		p[1] = (px >> 32) & 0xff;
+		p[2] = (px >> 24) & 0xff;
+		p[3] = (px >> 16) & 0xff;
+		p[4] = (px >> 8)  & 0xff;
+		p[5] = (px)       & 0xff;
+#else
+		p[0] = (px)       & 0xff;
+		p[1] = (px >> 8)  & 0xff;
+		p[2] = (px >> 16) & 0xff;
+		p[3] = (px >> 24) & 0xff;
+		p[4] = (px >> 32) & 0xff;
+		p[5] = (px >> 40) & 0xff;
+#endif
+		break;
+	case 40:
+#if AG_BYTEORDER == AG_BIG_ENDIAN
+		p[0] = (px >> 32) & 0xff;
+		p[1] = (px >> 24) & 0xff;
+		p[2] = (px >> 16) & 0xff;
+		p[3] = (px >> 8)  & 0xff;
+		p[4] = (px)       & 0xff;
+#else
+		p[0] = (px)       & 0xff;
+		p[1] = (px >> 8)  & 0xff;
+		p[2] = (px >> 16) & 0xff;
+		p[3] = (px >> 24) & 0xff;
+		p[4] = (px >> 32) & 0xff;
 #endif
 		break;
 	case 16:
 		*(Uint16 *)p = (Uint16)px;
 		break;
+	case 8:
 	default:
 		*p = (Uint8)px;
 		break;
@@ -592,108 +737,92 @@ ag_surface_put64_at(AG_Surface *S, Uint8 *p, Uint64 px)
 }
 
 /*
- * Blend 16-bit RGBA components with the pixel at x,y and overwrite it with
- * the result. No clipping. Target pixel alpha is set according to fn.
+ * Blend 16bpc RGBA components with the pixel at x,y and overwrite it with
+ * the result. No clipping is done.
  */
 # ifdef AG_INLINE_HEADER
 static __inline__ void
 AG_SurfaceBlendRGB16(AG_Surface *_Nonnull S, int x, int y,
-    Uint16 r, Uint16 g, Uint16 b, Uint16 a, AG_AlphaFn fn)
+    Uint16 r, Uint16 g, Uint16 b, Uint16 a)
 # else
 void
 ag_surface_blend_rgb16(AG_Surface *S, int x, int y,
-    Uint16 r, Uint16 g, Uint16 b, Uint16 a, AG_AlphaFn fn)
+    Uint16 r, Uint16 g, Uint16 b, Uint16 a)
 # endif
 {
 	AG_Color c = { r,g,b,a };
 
 	AG_SurfaceBlend_At(S, S->pixels +
-	    y*S->pitch +
-	    x*S->format.BytesPerPixel,
-	    &c, fn);
+	    y * S->pitch + S->Lpadding +
+	    x * S->format.BytesPerPixel,
+	    &c);
 }
 
 /*
- * Blend 16-bit RGBA components with the pixel at address p and overwrite it
- * with the result.  No clipping is done. Set target alpha according to fn.
+ * Blend 16bpc RGBA components with the pixel at address p and overwrite it
+ * with the result.  No clipping is done.
  */
 # ifdef AG_INLINE_HEADER
 static __inline__ void
 AG_SurfaceBlendRGB16_At(AG_Surface *_Nonnull S, Uint8 *_Nonnull p,
-    Uint16 r, Uint16 g, Uint16 b, Uint16 a, AG_AlphaFn fn)
+    Uint16 r, Uint16 g, Uint16 b, Uint16 a)
 # else
 void
 ag_surface_blend_rgb16_at(AG_Surface *S, Uint8 *p,
-    Uint16 r, Uint16 g, Uint16 b, Uint16 a, AG_AlphaFn fn)
+    Uint16 r, Uint16 g, Uint16 b, Uint16 a)
 # endif
 {
 	AG_Color c = { r,g,b,a };
 
-	AG_SurfaceBlend_At(S,p, &c, fn);
+	AG_SurfaceBlend_At(S,p, &c);
 }
 #endif /* AG_LARGE */
 
 /*
  * Blend a color c with the pixel at x,y and overwrite it with the result.
- * No clipping is done. Sets the pixel's alpha component according to fn.
+ * No clipping is done.
  */
 #ifdef AG_INLINE_HEADER
 static __inline__ void
-AG_SurfaceBlend(AG_Surface *_Nonnull S, int x, int y, const AG_Color *_Nonnull c,
-    AG_AlphaFn fn)
+AG_SurfaceBlend(AG_Surface *_Nonnull S, int x, int y, const AG_Color *_Nonnull c)
 #else
 void
-ag_surface_blend(AG_Surface *S, int x, int y, const AG_Color *c, AG_AlphaFn fn)
+ag_surface_blend(AG_Surface *S, int x, int y, const AG_Color *c)
 #endif
 {
 	AG_SurfaceBlend_At(S, S->pixels +
-	    y*S->pitch +
-	    x*S->format.BytesPerPixel,
-	    c, fn);
+	    y * S->pitch + S->Lpadding +
+	    x * S->format.BytesPerPixel,
+	    c);
 }
 
 /*
  * Blend a color c with the pixel at address p and overwrite it with the result.
- * No clipping is done. Sets the pixel's alpha component according to fn.
+ * No clipping is done.
  */
 #ifdef AG_INLINE_HEADER
 static __inline__ void
 AG_SurfaceBlend_At(AG_Surface *_Nonnull S, Uint8 *_Nonnull p,
-    const AG_Color *_Nonnull c, AG_AlphaFn fn)
+    const AG_Color *_Nonnull c)
 #else
 void
-ag_surface_blend_at(AG_Surface *S, Uint8 *p, const AG_Color *_Nonnull c,
-    AG_AlphaFn fn)
+ag_surface_blend_at(AG_Surface *S, Uint8 *p, const AG_Color *_Nonnull c)
 #endif
 {
-	AG_Pixel px = AG_SurfaceGet_At(S,p);
 	AG_Color pc;
+	AG_Pixel px;
 	AG_Component ca;
 
-	if ((S->flags & AG_SURFACE_COLORKEY) &&		/* No blending needed */
-	    (px == S->colorkey)) {
-		AG_SurfacePut_At(S, p, AG_MapPixel(&S->format, c));
-		return;
-	}
-	pc.a = AG_COLOR_LAST;				/* compiler happy */
+	px = AG_SurfaceGet_At(S,p);
 	AG_GetColor(&pc, px, &S->format);
 	ca = c->a;
-	switch (fn) {
-	case AG_ALPHA_OVERLAY:
-		pc.a = ((pc.a+ca) > AG_COLOR_LAST) ? AG_COLOR_LAST : (pc.a+ca);
-		break;
-	case AG_ALPHA_DST:        /* pc.a = pc.a */			break;
-	case AG_ALPHA_SRC:           pc.a = ca;				break;
-	case AG_ALPHA_ZERO:          pc.a = 0;				break;
-	case AG_ALPHA_ONE_MINUS_DST: pc.a = AG_COLOR_LAST - pc.a;	break;
-	case AG_ALPHA_ONE_MINUS_SRC: pc.a = AG_COLOR_LAST - ca;		break;
-	case AG_ALPHA_ONE: default:  pc.a = AG_COLOR_LAST;		break;
-	}
+	pc.a = ((pc.a + ca) > AG_COLOR_LAST) ? AG_COLOR_LAST : (pc.a + ca);
+
 	AG_SurfacePut_At(S, p,
 	    AG_MapPixel_RGBA(&S->format,
-	        pc.r + (((c->r - pc.r)*ca) >> AG_COMPONENT_BITS),
-	        pc.g + (((c->g - pc.g)*ca) >> AG_COMPONENT_BITS),
-	        pc.b + (((c->b - pc.b)*ca) >> AG_COMPONENT_BITS),
+	        pc.r + (((c->r - pc.r) * ca) >> AG_COMPONENT_BITS),
+	        pc.g + (((c->g - pc.g) * ca) >> AG_COMPONENT_BITS),
+	        pc.b + (((c->b - pc.b) * ca) >> AG_COMPONENT_BITS),
 	        pc.a));
 }
 
@@ -712,7 +841,7 @@ ag_pixel_format_free(AG_PixelFormat *pf)
 	}
 }
 
-/* Set the source alpha flag and per-surface alpha. */
+/* Set the per-surface alpha value. */
 #ifdef AG_INLINE_HEADER
 static __inline__ void
 AG_SurfaceSetAlpha(AG_Surface *_Nonnull S, Uint flags, AG_Component alpha)
@@ -721,11 +850,6 @@ void
 ag_surface_set_alpha(AG_Surface *S, Uint flags, AG_Component alpha)
 #endif
 {
-	if (flags & AG_SURFACE_ALPHA) {
-		S->flags |= AG_SURFACE_ALPHA;
-	} else {
-		S->flags &= ~(AG_SURFACE_ALPHA);
-	}
 	S->alpha = alpha;
 }
 
