@@ -11,17 +11,18 @@ typedef enum ag_keyboard_action {
 } AG_KeyboardAction;
 
 typedef enum ag_key_category {
-	AG_KCAT_NONE,			/* No category */
-	AG_KCAT_CONTROL,		/* Control character */
-	AG_KCAT_SPACING,		/* Whitespace */
-	AG_KCAT_RETURN,			/* Return / Line feed */
-	AG_KCAT_PRINT,			/* Printable character (not alphanumerici) */
-	AG_KCAT_ALPHA,			/* Alphabetic character */
-	AG_KCAT_NUMBER,			/* Numerical character */
-	AG_KCAT_DIR,			/* Directional keys */
-	AG_KCAT_FUNCTION,		/* Function keys */
-	AG_KCAT_LOCK,			/* Num/caps/scroll lock */
-	AG_KCAT_MODIFIER,		/* Shift/ctrl/alt/meta/super/compose */
+	AG_KCAT_NONE,      /* No category */
+	AG_KCAT_CONTROL,   /* Control character */
+	AG_KCAT_SPACING,   /* Whitespace */
+	AG_KCAT_RETURN,    /* Return / Line feed */
+	AG_KCAT_PRINT,     /* Printable character (not alphanumeric) */
+	AG_KCAT_ALPHA,     /* Alphabetic character */
+	AG_KCAT_NUMBER,    /* Numerical character */
+	AG_KCAT_DIR,       /* Directional key */
+	AG_KCAT_FUNCTION,  /* Function key */
+	AG_KCAT_LOCK,      /* (Num|Caps|Scroll) lock key */
+	AG_KCAT_MODIFIER,  /* (Shift|Ctrl|Alt|Meta|Super|Compose) key */
+	AG_KCAT_LAST
 } AG_KeyCategory;
 
 typedef enum ag_key_sym {
@@ -194,7 +195,7 @@ typedef enum ag_key_sym {
 	AG_KEY_F13		= 0x0126,
 	AG_KEY_F14		= 0x0127,
 	AG_KEY_F15		= 0x0128,
-	AG_KEY_NUMLOCK		= 0x012c,
+	AG_KEY_NUMLOCK		= 0x012c, /* Num Lock on PC, Clear on Mac */
 	AG_KEY_CAPSLOCK		= 0x012d,
 	AG_KEY_SCROLLOCK	= 0x012e,
 	AG_KEY_RSHIFT		= 0x012f,
@@ -263,34 +264,141 @@ typedef enum ag_key_sym {
 	AG_KEY_REDO		= 0x016e,
 	AG_KEY_FIND		= 0x016f,
 	AG_KEY_MODE_SWITCH	= 0x0170,
-	AG_KEY_LAST		= 0x0171,
-	AG_KEY_ANY		= 0xffff	/* For matching */
+	AG_KEY_NON_US_BACKSLASH = 0x0171,
+	AG_KEY_APPLICATION	= 0x0172,	/* Contextual menu, compose */
+	AG_KEY_AGAIN		= 0x0173,	/* Redo */
+	AG_KEY_CUT		= 0x0174,
+	AG_KEY_PASTE		= 0x0175,
+	AG_KEY_KP_COMMA		= 0x0176,
+	AG_KEY_KP_EQUALS_AS_400	= 0x0177,
+	AG_KEY_INTERNATIONAL_1	= 0x0178,	/* Asian keyboards */
+	AG_KEY_INTERNATIONAL_2	= 0x0179,
+	AG_KEY_INTERNATIONAL_3	= 0x017a,	/* Yen */
+	AG_KEY_INTERNATIONAL_4	= 0x017b,
+	AG_KEY_INTERNATIONAL_5	= 0x017c,
+	AG_KEY_INTERNATIONAL_6	= 0x017d,
+	AG_KEY_INTERNATIONAL_7	= 0x017e,
+	AG_KEY_INTERNATIONAL_8	= 0x017f,
+	AG_KEY_INTERNATIONAL_9	= 0x0180,
+	AG_KEY_LANGUAGE_1	= 0x0181,	/* Hangul/English */
+	AG_KEY_LANGUAGE_2	= 0x0182,	/* Hanja */
+	AG_KEY_LANGUAGE_3	= 0x0183,	/* Katakana */
+	AG_KEY_LANGUAGE_4	= 0x0184,	/* Hiragana */
+	AG_KEY_LANGUAGE_5	= 0x0185,	/* Zenkaku / Hankaku */
+	AG_KEY_LANGUAGE_6	= 0x0186,
+	AG_KEY_LANGUAGE_7	= 0x0187,
+	AG_KEY_LANGUAGE_8	= 0x0188,
+	AG_KEY_LANGUAGE_9	= 0x0189,
+
+	AG_KEY_ALT_ERASE		= 0x0190, /* Erase-Eaze */
+	AG_KEY_CANCEL			= 0x0191,
+	AG_KEY_PRIOR			= 0x0192,
+	AG_KEY_RETURN2			= 0x0193,
+	AG_KEY_SEPARATOR		= 0x0194,
+	AG_KEY_OUT			= 0x0195,
+	AG_KEY_OPER			= 0x0196,
+	AG_KEY_CLEAR_AGAIN		= 0x0197,
+	AG_KEY_CRSEL			= 0x0198,
+	AG_KEY_EXSEL			= 0x0199,
+	AG_KEY_KP_00			= 0x019a,
+	AG_KEY_KP_000			= 0x019b,
+	AG_KEY_THOUSANDS_SEPARATOR	= 0x019c,
+	AG_KEY_DECIMALS_SEPARATOR	= 0x019d,
+	AG_KEY_CURRENCY_UNIT		= 0x019e,
+	AG_KEY_CURRENCY_SUBUNIT		= 0x019f,
+	AG_KEY_KP_LEFT_PAREN		= 0x01a0,
+	AG_KEY_KP_RIGHT_PAREN		= 0x01a1,
+	AG_KEY_KP_LEFT_BRACE		= 0x01a2,
+	AG_KEY_KP_RIGHT_BRACE		= 0x01a3,
+	AG_KEY_KP_TAB			= 0x01a4,
+	AG_KEY_KP_BACKSPACE		= 0x01a5,
+	AG_KEY_KP_A			= 0x01a6,
+	AG_KEY_KP_B			= 0x01a7,
+	AG_KEY_KP_C			= 0x01a8,
+	AG_KEY_KP_D			= 0x01a9,
+	AG_KEY_KP_E			= 0x01aa,
+	AG_KEY_KP_F			= 0x01ab,
+	AG_KEY_KP_XOR			= 0x01ac,
+	AG_KEY_KP_POWER			= 0x01ad,
+	AG_KEY_KP_PERCENT		= 0x01ae,
+	AG_KEY_KP_LESS			= 0x01af,
+	AG_KEY_KP_GREATER		= 0x01b1,
+	AG_KEY_KP_AMPERSAND		= 0x01b2,
+	AG_KEY_KP_DBL_AMPERSAND		= 0x01b3,
+	AG_KEY_KP_VERTICAL_BAR		= 0x01b4,
+	AG_KEY_KP_DBL_VERTICAL_BAR	= 0x01b5,
+	AG_KEY_KP_COLON			= 0x01b6,
+	AG_KEY_KP_HASH			= 0x01b7,
+	AG_KEY_KP_SPACE			= 0x01b8,
+	AG_KEY_KP_AT			= 0x01b9,
+	AG_KEY_KP_EXCLAM		= 0x01ba,
+	AG_KEY_KP_MEM_STORE		= 0x01bb,
+	AG_KEY_KP_MEM_RECALL		= 0x01bc,
+	AG_KEY_KP_MEM_CLEAR		= 0x01bd,
+	AG_KEY_KP_MEM_ADD		= 0x01be,
+	AG_KEY_KP_MEM_SUBTRACT		= 0x01bf,
+	AG_KEY_KP_MEM_MULTIPLY		= 0x01c0,
+	AG_KEY_KP_MEM_DIVIDE		= 0x01c1,
+	AG_KEY_KP_PLUS_MINUS		= 0x01c2,
+
+	AG_KEY_KP_CLEAR_ENTRY		= 0x01c3,
+	AG_KEY_KP_BINARY		= 0x01c4,
+	AG_KEY_KP_OCTAL			= 0x01c5,
+	AG_KEY_KP_DECIMAL		= 0x01c6,
+	AG_KEY_KP_HEXADECIMAL		= 0x01c7,
+
+	AG_KEY_LGUI			= 0x01c8, /* Windows/Command/Meta key */
+	AG_KEY_RGUI			= 0x01c9,
+
+	AG_KEY_AUDIO_NEXT		= 0x01ca,
+	AG_KEY_AUDIO_PREV		= 0x01cb,
+	AG_KEY_AUDIO_STOP		= 0x01cc,
+	AG_KEY_AUDIO_PLAY		= 0x01cd,
+	AG_KEY_AUDIO_MUTE		= 0x01ce,
+	AG_KEY_MEDIA_SELECT		= 0x01cf,
+	AG_KEY_WWW			= 0x01d0,
+	AG_KEY_MAIL			= 0x01d1,
+	AG_KEY_CALCULATOR		= 0x01d2,
+	AG_KEY_COMPUTER			= 0x01d3,
+	AG_KEY_AC_SEARCH		= 0x01d4,
+	AG_KEY_AC_HOME			= 0x01d5,
+	AG_KEY_AC_BACK			= 0x01d6,
+	AG_KEY_AC_FORWARD		= 0x01d7,
+	AG_KEY_AC_STOP			= 0x01d8,
+	AG_KEY_AC_REFRESH		= 0x01d9,
+	AG_KEY_AC_BOOKMARKS		= 0x01da,
+	AG_KEY_AUDIO_REWIND		= 0x01db,
+	AG_KEY_AUDIO_FASTFORWARD	= 0x01dc,
+
+	AG_KEY_LAST			= 0x01de,	/* For array bounds */
+	AG_KEY_ANY			= 0xffff	/* Mask for matching */
 } AG_KeySym;
 
 typedef unsigned int AG_KeyMod;
 
-#define AG_KEYMOD_NONE		0x0000
-#define AG_KEYMOD_LSHIFT	0x0001		/* Left Shift */
-#define AG_KEYMOD_RSHIFT	0x0002		/* Right Shift */
-#define AG_KEYMOD_CTRL_SHIFT	0x0004		/* Ctrl+Shift combined */
-#define AG_KEYMOD_CTRL_ALT	0x0008		/* Ctrl+Alt combined */
-			/*      0x0010 Reserved */
-			/*      0x0020 Reserved */
-#define AG_KEYMOD_LCTRL		0x0040		/* Left Ctrl */
-#define AG_KEYMOD_RCTRL		0x0080		/* Right Ctrl */
-#define AG_KEYMOD_LALT		0x0100		/* Left Alt */
-#define AG_KEYMOD_RALT		0x0200		/* Right Alt */
-#define AG_KEYMOD_LMETA		0x0400		/* Left Meta */
-#define AG_KEYMOD_RMETA		0x0800		/* Right Meta */
-#define AG_KEYMOD_NUMLOCK	0x1000		/* Num lock */
-#define AG_KEYMOD_CAPSLOCK	0x2000		/* Caps lock */
-#define AG_KEYMOD_MODE		0x4000		/* Mode key */
-			/*      0x8000 Reserved */
-#define AG_KEYMOD_CTRL		(AG_KEYMOD_LCTRL  | AG_KEYMOD_RCTRL)
-#define AG_KEYMOD_SHIFT		(AG_KEYMOD_LSHIFT | AG_KEYMOD_RSHIFT)
-#define AG_KEYMOD_ALT		(AG_KEYMOD_LALT   | AG_KEYMOD_RALT)
-#define AG_KEYMOD_META		(AG_KEYMOD_LMETA  | AG_KEYMOD_RMETA)
-#define AG_KEYMOD_ANY		0xffff		/* Any modifier */
+#define AG_KEYMOD_NONE       0x0000
+#define AG_KEYMOD_LSHIFT     0x0001             /* Left Shift */
+#define AG_KEYMOD_RSHIFT     0x0002             /* Right Shift */
+#define AG_KEYMOD_CTRL_SHIFT 0x0004             /* Ctrl+Shift combined */
+#define AG_KEYMOD_CTRL_ALT   0x0008             /* Ctrl+Alt combined */
+                        /*   0x0010 Reserved */
+                        /*   0x0020 Reserved */
+#define AG_KEYMOD_LCTRL      0x0040             /* Left Ctrl */
+#define AG_KEYMOD_RCTRL      0x0080             /* Right Ctrl */
+#define AG_KEYMOD_LALT       0x0100             /* Left Alt */
+#define AG_KEYMOD_RALT       0x0200             /* Right Alt */
+#define AG_KEYMOD_LMETA      0x0400             /* Left Meta */
+#define AG_KEYMOD_RMETA      0x0800             /* Right Meta */
+#define AG_KEYMOD_NUMLOCK    0x1000             /* Num lock */
+#define AG_KEYMOD_CAPSLOCK   0x2000             /* Caps lock */
+#define AG_KEYMOD_MODE       0x4000             /* Mode key */
+                        /*   0x8000 Reserved */
+#define AG_KEYMOD_CTRL      (AG_KEYMOD_LCTRL | AG_KEYMOD_RCTRL)
+#define AG_KEYMOD_SHIFT     (AG_KEYMOD_LSHIFT | AG_KEYMOD_RSHIFT)
+#define AG_KEYMOD_ALT       (AG_KEYMOD_LALT | AG_KEYMOD_RALT)
+#define AG_KEYMOD_META      (AG_KEYMOD_LMETA | AG_KEYMOD_RMETA)
+#define AG_KEYMOD_ALT_SHIFT (AG_KEYMOD_ALT | AG_KEYMOD_SHIFT)
+#define AG_KEYMOD_ANY   0xffff		/* Any modifier */
 
 struct ag_window;
 
@@ -304,18 +412,29 @@ typedef struct ag_keyboard {
 	struct ag_input_device _inherit;
 	int *_Nonnull keyState;		/* Key state */
 	Uint keyCount;
-	Uint modState;			/* Modifiers state */
+	AG_KeyMod modState;		/* Modifiers state */
 } AG_Keyboard;
 
+#define   AGKEYBOARD(obj)   ((AG_Keyboard *)(obj))
+#define  AGcKEYBOARD(obj)   ((const AG_Keyboard *)(obj))
+#define  AG_KEYBOARD_ISA(o) (((AGOBJECT(o)->cid & 0xffff0000) >> 16) == 0x0602)
+#define  AG_KEYBOARD_SELF()    AGKEYBOARD(  AG_OBJECT(0,         "AG_InputDevice:AG_Keyboard:*") )
+#define  AG_KEYBOARD_PTR(n)    AGKEYBOARD(  AG_OBJECT((n),       "AG_InputDevice:AG_Keyboard:*") )
+#define  AG_KEYBOARD_NAMED(n)  AGKEYBOARD(  AG_OBJECT_NAMED((n), "AG_InputDevice:AG_Keyboard:*") )
+#define AG_cKEYBOARD_SELF()   AGcKEYBOARD( AG_cOBJECT(0,         "AG_InputDevice:AG_Keyboard:*") )
+#define AG_cKEYBOARD_PTR(n)   AGcKEYBOARD( AG_cOBJECT((n),       "AG_InputDevice:AG_Keyboard:*") )
+#define AG_cKEYBOARD_NAMED(n) AGcKEYBOARD( AG_cOBJECT_NAMED((n), "AG_InputDevice:AG_Keyboard:*") )
+
 /* Begin generated block */
 __BEGIN_DECLS
 extern DECLSPEC AG_ObjectClass agKeyboardClass;
 extern DECLSPEC const char *agKeySyms[]; 
 extern DECLSPEC const int agKeySymCount;
 extern DECLSPEC AG_Keyboard *_Nullable AG_KeyboardNew(void *_Nonnull, const char *_Nonnull);
-extern DECLSPEC int AG_KeyboardUpdate(AG_Keyboard *_Nonnull, AG_KeyboardAction, AG_KeySym);
+extern DECLSPEC void AG_KeyboardUpdate(AG_Keyboard *_Nonnull, AG_KeyboardAction, AG_KeySym);
 extern DECLSPEC int AG_ProcessKey(AG_Keyboard *_Nonnull, struct ag_window *_Nonnull, AG_KeyboardAction, AG_KeySym, AG_Char);
 extern DECLSPEC const char *_Nullable AG_LookupKeyName(AG_KeySym) _Pure_Attribute;
+extern DECLSPEC char *_Nonnull AG_LookupKeyMod(AG_KeyMod) _Pure_Attribute;
 extern DECLSPEC AG_KeySym AG_LookupKeySym(const char *_Nonnull) _Pure_Attribute;
 extern DECLSPEC void AG_InitGlobalKeys(void);
 extern DECLSPEC void AG_DestroyGlobalKeys(void);
