@@ -30,12 +30,12 @@ struct ag_key_mapping {
 
 /* Begin generated block */
 __BEGIN_DECLS
-extern DECLSPEC const struct ag_keycode agKeymap[];
+extern DECLSPEC const struct ag_keycode agKeymap[]; 
 #ifdef AG_UNICODE
-extern DECLSPEC const struct ag_key_mapping agKeymapLATIN1[];
-extern DECLSPEC const struct ag_key_composition agCompositionMap[];
-extern DECLSPEC const int agCompositionMapSize;
-extern DECLSPEC int AG_KeyInputCompose(struct ag_editable *_Nonnull, AG_Char, AG_Char *_Nonnull);
+extern DECLSPEC const struct ag_key_mapping agKeymapLatin1[];
+extern DECLSPEC const struct ag_key_composition agKeyComposeMapLatin[];
+extern DECLSPEC const int agKeyComposeMapLatinSize;
+extern DECLSPEC int AG_KeyComposeLatin(struct ag_editable *_Nonnull, AG_Char, AG_Char *_Nonnull);
 #endif 
 __END_DECLS
 /* Close generated block */
