@@ -9,37 +9,32 @@
 
 #include <agar/gui/begin.h>
 
-#define AG_MENU_ITEM_TAG "AgMnItm"
-#define AG_MENU_ITEM_TAG_LEN 8
-
 struct ag_menu;
 struct ag_menu_view;
 struct ag_button;
 
 typedef struct ag_menu_item {
-#ifdef AG_TYPE_SAFETY
-	char tag[AG_MENU_ITEM_TAG_LEN];	/* Tagged Non-object */
-#endif
+	Uint32 tag;			/* Tagged Non-Object */
 	char *_Nonnull text;		/* Label text */
-	int lblMenu[2];			/* Cached surfaces (for AG_Menu) */
-	int lblView[2];			/* Cached surfaces (for AG_MenuView) */
-	AG_Surface *_Nullable iconSrc;	/* Icon surface source */
-	int                   icon;	/* Icon surface mapping */
+	int lblMenu[2];			/* Cached surfaces of AG_Menu(3) */
+					/* 0=Disabled, 1=Enabled */
+	int lblView[3];			/* Cached surfaces of AG_MenuView(3) */
+					/* 0=Disabled, 1=Enabled, 2=Selected */
 	int value;			/* Default bool value binding */
+	AG_Surface *_Nullable icon;	/* Icon surface */
 	AG_Function *_Nullable stateFn;	/* State function (overrides flag) */
 	int                    state;	/* State flag */
 	AG_KeySym key_equiv;		/* Key shortcut */
 	AG_KeyMod key_mod;
 	int x, y;			/* Position in parent view */
 	Uint flags;
-#define AG_MENU_ITEM_ICONS     0x01 /* At least one item has an icon */
-#define AG_MENU_ITEM_NOSELECT  0x02 /* Non-selectable regardless of state */
-#define AG_MENU_ITEM_SEPARATOR 0x04 /* Item is a cosmetic separator */
-#define AG_MENU_ITEM_INVERTED  0x08 /* Invert the binding state */
+#define AG_MENU_ITEM_ICONS     0x01     /* At least one item has an icon */
+#define AG_MENU_ITEM_NOSELECT  0x02     /* Non-selectable regardless of state */
+#define AG_MENU_ITEM_SEPARATOR 0x04     /* Item is a cosmetic separator */
+#define AG_MENU_ITEM_INVERTED  0x08     /* Invert the binding state */
 
 	AG_Event *_Nullable clickFn;	/* Raised on click */
 	AG_Event *_Nullable poll;	/* Raised before the item is drawn */
-
 	enum ag_menu_binding {		/* Boolean binding */
 		AG_MENU_NO_BINDING,
 		AG_MENU_INT_BOOL,
@@ -59,28 +54,26 @@ typedef struct ag_menu_item {
 	struct ag_menu_item *_Nullable sel_subitem; /* Selected subitem */
 	struct ag_button *_Nullable    tbButton;    /* Related toolbar button */
 	struct ag_menu_item *_Nullable parent;      /* Parent MenuItem if any */
-
 	AG_TAILQ_HEAD_(ag_menu_item) subItems;      /* Child items */
 	Uint                        nSubItems;
 	Uint32 _pad;
 	AG_TAILQ_ENTRY(ag_menu_item) items;         /* In parent */
 } AG_MenuItem;
 
-#define AGMENUITEM(p)             ((AG_MenuItem *)(p))
-#define AGCMENUITEM(p)            ((const AG_MenuItem *)(p))
-#define AG_MENU_ITEM_SELF()       AGMENUITEM( AG_MENU_ITEM_PTR(0) )
-#define AG_CONST_MENU_ITEM_SELF() AGCMENUITEM( AG_CONST_MENU_ITEM(0) )
+/* Tagged Non-Object */
+#define    AGMENUITEM(p)      ((AG_MenuItem *)(p))
+#define   AGcMENUITEM(p)      ((const AG_MenuItem *)(p))
 
 #ifdef AG_TYPE_SAFETY
-# define AG_MENU_ITEM_VALID(p)     (strncmp(AGMENUITEM(p)->tag, AG_MENU_ITEM_TAG, AG_MENU_ITEM_TAG_LEN) == 0)
-# define AG_MENU_ITEM_IS_VALID(p)  if (!AG_MENU_ITEM_VALID(p)) { AG_FatalError("Illegal AG_MenuItem access"); }
-# define AG_MENU_ITEM_PTR(v)       AG_MenuGetItemPtr(event,(v),0)
-# define AG_CONST_MENU_ITEM_PTR(v) AG_MenuGetItemPtr(event,(v),1)
+# define  AG_MENUITEM_VALID(p)     (AGMENUITEM(p)->tag == agNonObjectSignature)
+# define  AG_MENUITEM_IS_VALID(p)  if (!AG_MENUITEM_VALID(p)) { AG_FatalError("Illegal AG_MenuItem access"); }
+# define  AG_MENUITEM_PTR(v) AG_MenuGetItemPtr(event,(v),0)
+# define AG_cMENUITEM_PTR(v) AG_MenuGetItemPtr(event,(v),1)
 #else
-# define AG_MENU_ITEM_VALID(p)     1
-# define AG_MENU_ITEM_IS_VALID(p)
-# define AG_MENU_ITEM_PTR(v)       event->argv[v].data.p
-# define AG_CONST_MENU_ITEM_PTR(v) event->argv[v].data.p
+# define  AG_MENUITEM_VALID(p)     1
+# define  AG_MENUITEM_IS_VALID(p)
+# define  AG_MENUITEM_PTR(v) event->argv[v].data.p
+# define AG_cMENUITEM_PTR(v) event->argv[v].data.p
 #endif
 
 enum ag_menu_style {
@@ -90,32 +83,41 @@ enum ag_menu_style {
 };
 
 typedef struct ag_menu {
-	struct ag_widget wid;             /* AG_Widget -> AG_Menu */
+	struct ag_widget wid;       /* AG_Widget -> AG_Menu */
+
 	Uint flags;
-#define AG_MENU_HFILL  0x01
-#define AG_MENU_VFILL  0x02
-#define AG_MENU_EXPAND (AG_MENU_HFILL | AG_MENU_VFILL)
-	enum ag_menu_style style;         /* Menu style */
-	AG_MenuItem *_Nonnull root;       /* Root menu item */
-	int selecting;                    /* Selection in progress */
-	int lPadLbl, rPadLbl;             /* Item label padding in pixels */
+#define AG_MENU_HFILL         0x01
+#define AG_MENU_VFILL         0x02
+#define AG_MENU_NO_COLOR_BG   0x04  /* Don't use parent MenuView's "color" as the
+                                       "background-color" of menu-expansion windows. */
+#define AG_MENU_NO_BOOL_MSG   0x08  /* Don't notify the user when a boolean has
+                                       been toggled by a keyboard shortcut. */
+#define AG_MENU_FAST_BOOL_MSG 0x10  /* Display the notification message only
+                                       for 400ms (default = 1s). */
+#define AG_MENU_EXPAND             (AG_MENU_HFILL | AG_MENU_VFILL)
+
+	enum ag_menu_style style;   /* Menu style */
+	AG_MenuItem *_Nonnull root; /* Root menu item */
+	int selecting;              /* Selection in progress */
+	int lPadLbl, rPadLbl;       /* Item label padding in pixels */
 	int tPadLbl, bPadLbl;
-	int itemh;                        /* Item height (optimization) */
-	int curState;                     /* For MenuState() */
+	int itemh;                  /* Item height (optimization) */
+	int curState;               /* For MenuState() */
 	Uint32 _pad;
 	AG_MenuItem *_Nullable itemSel;   /* Selected top-level item */
 	AG_Toolbar *_Nullable curToolbar; /* For MenuToolbar() */
 	AG_Rect r;                        /* View area */
 } AG_Menu;
 
-#define AGMENU(obj)            ((AG_Menu *)(obj))
-#define AGCMENU(obj)           ((const AG_Menu *)(obj))
-#define AG_MENU_SELF()          AGMENU( AG_OBJECT(0,"AG_Widget:AG_Menu:*") )
-#define AG_MENU_PTR(n)          AGMENU( AG_OBJECT((n),"AG_Widget:AG_Menu:*") )
-#define AG_MENU_NAMED(n)        AGMENU( AG_OBJECT_NAMED((n),"AG_Widget:AG_Menu:*") )
-#define AG_CONST_MENU_SELF()   AGCMENU( AG_CONST_OBJECT(0,"AG_Widget:AG_Menu:*") )
-#define AG_CONST_MENU_PTR(n)   AGCMENU( AG_CONST_OBJECT((n),"AG_Widget:AG_Menu:*") )
-#define AG_CONST_MENU_NAMED(n) AGCMENU( AG_CONST_OBJECT_NAMED((n),"AG_Widget:AG_Menu:*") )
+#define   AGMENU(o)        ((AG_Menu *)(o))
+#define  AGcMENU(o)        ((const AG_Menu *)(o))
+#define  AG_MENU_ISA(o)    (((AGOBJECT(o)->cid & 0xff000000) >> 24) == 0x18)
+#define  AG_MENU_SELF()    AGMENU(  AG_OBJECT(0,        "AG_Widget:AG_Menu:*") )
+#define  AG_MENU_PTR(n)    AGMENU(  AG_OBJECT((n),      "AG_Widget:AG_Menu:*") )
+#define  AG_MENU_NAMED(n)  AGMENU(  AG_OBJECT_NAMED((n),"AG_Widget:AG_Menu:*") )
+#define AG_cMENU_SELF()   AGcMENU( AG_cOBJECT(0,        "AG_Widget:AG_Menu:*") )
+#define AG_cMENU_PTR(n)   AGcMENU( AG_cOBJECT((n),      "AG_Widget:AG_Menu:*") )
+#define AG_cMENU_NAMED(n) AGcMENU( AG_cOBJECT_NAMED((n),"AG_Widget:AG_Menu:*") )
 
 typedef struct ag_popup_menu {
 	AG_Widget *_Nonnull widget;     /* Parent widget */
@@ -134,14 +136,15 @@ typedef struct ag_menu_view {
 	int arrowRight;                 /* Right arrow surface handle */
 } AG_MenuView;
 
-#define AGMENUVIEW(obj)            ((AG_MenuView *)(obj))
-#define AGCMENUVIEW(obj)           ((const AG_MenuView *)(obj))
-#define AG_MENUVIEW_SELF()         AGMENUVIEW( AG_OBJECT(0,"AG_Widget:AG_MenuView:*") )
-#define AG_MENUVIEW_PTR(n)         AGMENUVIEW( AG_OBJECT((n),"AG_Widget:AG_MenuView:*") )
-#define AG_MENUVIEW_NAMED(n)       AGMENUVIEW( AG_OBJECT_NAMED((n),"AG_Widget:AG_MenuView:*") )
-#define AG_CONST_MENUVIEW_SELF()   AGCMENUVIEW( AG_CONST_OBJECT(0,"AG_Widget:AG_MenuView:*") )
-#define AG_CONST_MENUVIEW_PTR(n)   AGCMENUVIEW( AG_CONST_OBJECT((n),"AG_Widget:AG_MenuView:*") )
-#define AG_CONST_MENUVIEW_NAMED(n) AGCMENUVIEW( AG_CONST_OBJECT_NAMED((n),"AG_Widget:AG_MenuView:*") )
+#define   AGMENUVIEW(o)        ((AG_MenuView *)(o))
+#define  AGcMENUVIEW(o)        ((const AG_MenuView *)(o))
+#define  AG_MENUVIEW_ISA(o)    (((AGOBJECT(o)->cid & 0xff000000) >> 24) == 0x19)
+#define  AG_MENUVIEW_SELF()    AGMENUVIEW(  AG_OBJECT(0,         "AG_Widget:AG_MenuView:*") )
+#define  AG_MENUVIEW_PTR(n)    AGMENUVIEW(  AG_OBJECT((n),       "AG_Widget:AG_MenuView:*") )
+#define  AG_MENUVIEW_NAMED(n)  AGMENUVIEW(  AG_OBJECT_NAMED((n), "AG_Widget:AG_MenuView:*") )
+#define AG_cMENUVIEW_SELF()   AGcMENUVIEW( AG_cOBJECT(0,         "AG_Widget:AG_MenuView:*") )
+#define AG_cMENUVIEW_PTR(n)   AGcMENUVIEW( AG_cOBJECT((n),       "AG_Widget:AG_MenuView:*") )
+#define AG_cMENUVIEW_NAMED(n) AGcMENUVIEW( AG_cOBJECT_NAMED((n), "AG_Widget:AG_MenuView:*") )
 
 /* Begin generated block */
 __BEGIN_DECLS
@@ -171,12 +174,13 @@ extern DECLSPEC void AG_MenuSetIcon(AG_MenuItem *_Nonnull, const AG_Surface *_Nu
 extern DECLSPEC void AG_MenuSetLabel(AG_MenuItem *_Nonnull, const char *_Nonnull, ...) FORMAT_ATTRIBUTE(printf,2,3);
 extern DECLSPEC void AG_MenuSetLabelS(AG_MenuItem *_Nonnull, const char *_Nonnull);
 extern DECLSPEC void AG_MenuSetPollFn(AG_MenuItem *_Nonnull, _Nonnull AG_EventFn, const char *_Nullable, ...);
-extern DECLSPEC void AG_MenuUpdateItem(AG_MenuItem *_Nonnull);
 extern DECLSPEC void AG_MenuInvalidateLabels(AG_MenuItem *_Nonnull);
 extern DECLSPEC void AG_MenuFreeSubitems(AG_MenuItem *_Nonnull);
 extern DECLSPEC void AG_MenuState(AG_MenuItem *_Nonnull, int);
 #define AG_MenuDisable(m) AG_MenuState((m),0)
 #define AG_MenuEnable(m) AG_MenuState((m),1)
+extern DECLSPEC int AG_MenuBoolGet(AG_MenuItem *_Nonnull);
+extern DECLSPEC void AG_MenuBoolToggle(AG_MenuItem *_Nonnull);
 extern DECLSPEC void AG_MenuToolbar(AG_MenuItem *_Nonnull, AG_Toolbar *_Nullable);
 extern DECLSPEC AG_MenuItem *_Nonnull AG_MenuNode(AG_MenuItem *_Nonnull, const char *_Nullable, const AG_Surface *_Nullable);
 extern DECLSPEC AG_MenuItem *_Nonnull AG_MenuSeparator(AG_MenuItem *_Nonnull);
@@ -212,6 +216,8 @@ extern DECLSPEC void AG_MenuSetIntFlagsMp(AG_MenuItem *_Nonnull, int *_Nonnull,
 extern DECLSPEC AG_MenuItem *_Nullable AG_MenuGetItemPtr(const AG_Event *_Nonnull, int, int);
 #endif
 #ifdef AG_LEGACY
+# define AG_MENU_ITEM_PTR(v) AG_MENUITEM_PTR(v)
+# define AG_CONST_MENU_ITEM_PTR(v) AG_cMENUITEM_PTR(v)
 # define AG_MenuAddItem(m,lbl) AG_MenuNode((m)->root,(lbl),NULL)
 # define AG_MenuSetPaddingLeft(m,v) AG_MenuSetPadding((m),(v),-1,-1,-1)
 # define AG_MenuSetPaddingRight(m,v) AG_MenuSetPadding((m),-1,(v),-1,-1)
