@@ -31,21 +31,20 @@ struct ag_window;
 
 typedef struct ag_mouse {
 	struct ag_input_device _inherit;
-	Uint nButtons;		/* Button count (0 = unknown) */
-	Uint btnState;		/* Last button state (AG_MouseButton) */
-	int x, y;		/* Last cursor position */
-	int xRel, yRel;		/* Last relative motion */
+	Uint nButtons;                    /* Button count (0 = unknown) */
+	AG_MouseButton btnState;          /* Last button state */
+	int x, y;                         /* Last cursor position */
+	int xRel, yRel;                   /* Last relative motion */
 } AG_Mouse;
 
+#define AG_MOUSE_ISA(o) (((AGOBJECT(o)->cid & 0xffff0000) >> 16) == 0x0601)
+
 /* Begin generated block */
 __BEGIN_DECLS
 extern DECLSPEC AG_ObjectClass agMouseClass;
 extern DECLSPEC AG_Mouse *_Nullable AG_MouseNew(void *_Nonnull, const char *_Nonnull);
-extern DECLSPEC Uint8 AG_MouseGetState(AG_Mouse *_Nonnull, int *_Nullable, int *_Nullable);
-extern DECLSPEC void AG_MouseMotionUpdate(AG_Mouse *_Nonnull, int,int);
 extern DECLSPEC void AG_MouseCursorUpdate(struct ag_window *_Nonnull, int,int);
-extern DECLSPEC void AG_MouseButtonUpdate(AG_Mouse *_Nonnull, AG_MouseButtonAction, int);
-extern DECLSPEC void AG_ProcessMouseMotion(struct ag_window *_Nonnull, int,int, int,int, Uint);
+extern DECLSPEC void AG_ProcessMouseMotion(struct ag_window *_Nonnull, int,int, int,int);
 extern DECLSPEC void AG_ProcessMouseButtonUp(struct ag_window *_Nonnull, int,int, AG_MouseButton);
 extern DECLSPEC void AG_ProcessMouseButtonDown(struct ag_window *_Nonnull, int,int, AG_MouseButton);
 __END_DECLS
