@@ -12,23 +12,27 @@
 #include <agar/gui/begin.h>
 
 #ifndef AG_NUMERICAL_INPUT_MAX
-#define AG_NUMERICAL_INPUT_MAX 56
+#define AG_NUMERICAL_INPUT_MAX 128
 #endif
 
 typedef struct ag_numerical {
-	struct ag_widget wid;          /* AG_Widget -> AG_Numerical */
+	struct ag_widget wid;        /* AG_Widget -> AG_Numerical */
 	Uint flags;
-#define AG_NUMERICAL_HFILL     0x01
-#define AG_NUMERICAL_VFILL     0x02
-#define AG_NUMERICAL_INT       0x04  /* Make the default binding an int */
-                                     /* (default = float) */
-#define AG_NUMERICAL_EXCL      0x08  /* Assume exclusive access to variable */
-#define AG_NUMERICAL_READONLY  0x10  /* Read-only mode */
-#define AG_NUMERICAL_SLOW      0x20  /* Update every 2s (default = 250ms) */
-#define AG_NUMERICAL_EXPAND   (AG_NUMERICAL_HFILL | AG_NUMERICAL_VFILL)
+#define AG_NUMERICAL_HFILL      0x01
+#define AG_NUMERICAL_VFILL      0x02
+#define AG_NUMERICAL_INT        0x04   /* Make the default binding an int */
+                                       /* (default = float) */
+#define AG_NUMERICAL_EXCL       0x08   /* Assume exclusive access to variable */
+#define AG_NUMERICAL_READONLY   0x10   /* Read-only mode */
+#define AG_NUMERICAL_SLOW       0x20   /* Update every 2s (default = 250ms) */
+#define AG_NUMERICAL_NO_POS_INF 0x40   /* Disallow "inf" or infinity */
+#define AG_NUMERICAL_NO_NEG_INF 0x80   /* Disallow "-inf" or negative infinity */
+#define AG_NUMERICAL_NO_INF (AG_NUMERICAL_NO_POS_INF | AG_NUMERICAL_NO_NEG_INF)
+#define AG_NUMERICAL_EXPAND     (AG_NUMERICAL_HFILL | AG_NUMERICAL_VFILL)
 
 	char format[28];                     /* Print format (for reals) */
-	const AG_Unit *_Nonnull unit;        /* Conversion unit in use */
+	const AG_Unit *_Nonnull unit;        /* Selected conversion unit */
+	const AG_Unit *_Nonnull unitGroup;   /* Group of conversion unit */
 	AG_UCombo *_Nullable units;          /* Unit selector */
 	char inTxt[AG_NUMERICAL_INPUT_MAX];  /* Input text buffer */
 	AG_Textbox *_Nonnull input;          /* Input textbox */
@@ -41,14 +45,15 @@ typedef struct ag_numerical {
 	AG_Timer toInc, toDec;               /* For keyboard increment/decrement */
 } AG_Numerical;
 
-#define AGNUMERICAL(obj)            ((AG_Numerical *)(obj))
-#define AGCNUMERICAL(obj)           ((const AG_Numerical *)(obj))
-#define AG_NUMERICAL_SELF()          AGNUMERICAL( AG_OBJECT(0,"AG_Widget:AG_Numerical:*") )
-#define AG_NUMERICAL_PTR(n)          AGNUMERICAL( AG_OBJECT((n),"AG_Widget:AG_Numerical:*") )
-#define AG_NUMERICAL_NAMED(n)        AGNUMERICAL( AG_OBJECT_NAMED((n),"AG_Widget:AG_Numerical:*") )
-#define AG_CONST_NUMERICAL_SELF()   AGCNUMERICAL( AG_CONST_OBJECT(0,"AG_Widget:AG_Numerical:*") )
-#define AG_CONST_NUMERICAL_PTR(n)   AGCNUMERICAL( AG_CONST_OBJECT((n),"AG_Widget:AG_Numerical:*") )
-#define AG_CONST_NUMERICAL_NAMED(n) AGCNUMERICAL( AG_CONST_OBJECT_NAMED((n),"AG_Widget:AG_Numerical:*") )
+#define   AGNUMERICAL(obj)      ((AG_Numerical *)(obj))
+#define  AGcNUMERICAL(obj)      ((const AG_Numerical *)(obj))
+#define  AG_NUMERICAL_ISA(o)    (((AGOBJECT(o)->cid & 0xff000000) >> 24) == 0x1B)
+#define  AG_NUMERICAL_SELF()    AGNUMERICAL(  AG_OBJECT(0,         "AG_Widget:AG_Numerical:*") )
+#define  AG_NUMERICAL_PTR(n)    AGNUMERICAL(  AG_OBJECT((n),       "AG_Widget:AG_Numerical:*") )
+#define  AG_NUMERICAL_NAMED(n)  AGNUMERICAL(  AG_OBJECT_NAMED((n), "AG_Widget:AG_Numerical:*") )
+#define AG_cNUMERICAL_SELF()   AGcNUMERICAL( AG_cOBJECT(0,         "AG_Widget:AG_Numerical:*") )
+#define AG_cNUMERICAL_PTR(n)   AGcNUMERICAL( AG_cOBJECT((n),       "AG_Widget:AG_Numerical:*") )
+#define AG_cNUMERICAL_NAMED(n) AGcNUMERICAL( AG_cOBJECT_NAMED((n), "AG_Widget:AG_Numerical:*") )
 
 /* Begin generated block */
 __BEGIN_DECLS
