@@ -27,9 +27,10 @@ typedef struct ag_radio {
 	struct ag_widget wid;           /* AG_Widget -> AG_Radio */
 	AG_RadioType type;              /* Disposition */
 	Uint flags;
-#define AG_RADIO_HFILL	0x01
-#define AG_RADIO_VFILL	0x02
-#define AG_RADIO_EXPAND (AG_RADIO_HFILL | AG_RADIO_VFILL)
+#define AG_RADIO_HFILL      0x01
+#define AG_RADIO_VFILL      0x02
+#define AG_RADIO_HOMOGENOUS 0x04	/* Divide space equally between items */
+#define AG_RADIO_EXPAND     (AG_RADIO_HFILL | AG_RADIO_VFILL)
 
 	AG_RadioItem *_Nullable items;  /* Array of radio items */
 	int                    nItems;
@@ -44,14 +45,15 @@ typedef struct ag_radio {
 	AG_Timer moveTo;      /* Timer for keyboard motion */
 } AG_Radio;
 
-#define AGRADIO(obj)            ((AG_Radio *)(obj))
-#define AGCRADIO(obj)           ((const AG_Radio *)(obj))
-#define AG_RADIO_SELF()          AGRADIO( AG_OBJECT(0,"AG_Widget:AG_Radio:*") )
-#define AG_RADIO_PTR(n)          AGRADIO( AG_OBJECT((n),"AG_Widget:AG_Radio:*") )
-#define AG_RADIO_NAMED(n)        AGRADIO( AG_OBJECT_NAMED((n),"AG_Widget:AG_Radio:*") )
-#define AG_CONST_RADIO_SELF()   AGCRADIO( AG_CONST_OBJECT(0,"AG_Widget:AG_Radio:*") )
-#define AG_CONST_RADIO_PTR(n)   AGCRADIO( AG_CONST_OBJECT((n),"AG_Widget:AG_Radio:*") )
-#define AG_CONST_RADIO_NAMED(n) AGCRADIO( AG_CONST_OBJECT_NAMED((n),"AG_Widget:AG_Radio:*") )
+#define   AGRADIO(obj)      ((AG_Radio *)(obj))
+#define  AGcRADIO(obj)      ((const AG_Radio *)(obj))
+#define  AG_RADIO_ISA(o)    (((AGOBJECT(o)->cid & 0xff000000) >> 24) == 0x1F)
+#define  AG_RADIO_SELF()    AGRADIO(  AG_OBJECT(0,        "AG_Widget:AG_Radio:*") )
+#define  AG_RADIO_PTR(n)    AGRADIO(  AG_OBJECT((n),      "AG_Widget:AG_Radio:*") )
+#define  AG_RADIO_NAMED(n)  AGRADIO(  AG_OBJECT_NAMED((n),"AG_Widget:AG_Radio:*") )
+#define AG_cRADIO_SELF()   AGcRADIO( AG_cOBJECT(0,        "AG_Widget:AG_Radio:*") )
+#define AG_cRADIO_PTR(n)   AGcRADIO( AG_cOBJECT((n),      "AG_Widget:AG_Radio:*") )
+#define AG_cRADIO_NAMED(n) AGcRADIO( AG_cOBJECT_NAMED((n),"AG_Widget:AG_Radio:*") )
 
 /* Begin generated block */
 __BEGIN_DECLS
