@@ -139,7 +139,7 @@ typedef struct ag_table {
 #define AG_TABLE_POLL           0x008	/* Table is polled */
 #define AG_TABLE_HFILL          0x010
 #define AG_TABLE_VFILL          0x020
-#define AG_TABLE_EXPAND        (AG_TABLE_HFILL|AG_TABLE_VFILL)
+#define AG_TABLE_EXPAND        (AG_TABLE_HFILL | AG_TABLE_VFILL)
 #define AG_TABLE_HIGHLIGHT_COLS 0x040	/* Highlight column selection */
 #define AG_TABLE_WIDGETS        0x080	/* Embedded widgets are in use */
 #define AG_TABLE_NOAUTOSORT     0x100	/* Disable automatic sorting */
@@ -196,14 +196,15 @@ typedef struct ag_table {
 	AG_Timer dblClickTo;		/* For double click */
 } AG_Table;
 
-#define AGTABLE(p)              ((AG_Table *)(p))
-#define AGCTABLE(p)             ((const AG_Table *)(p))
-#define AG_TABLE_SELF()          AGTABLE( AG_OBJECT(0,"AG_Widget:AG_Table:*") )
-#define AG_TABLE_PTR(n)          AGTABLE( AG_OBJECT((n),"AG_Widget:AG_Table:*") )
-#define AG_TABLE_NAMED(n)        AGTABLE( AG_OBJECT_NAMED((n),"AG_Widget:AG_Table:*") )
-#define AG_CONST_TABLE_SELF()   AGCTABLE( AG_CONST_OBJECT(0,"AG_Widget:AG_Table:*") )
-#define AG_CONST_TABLE_PTR(n)   AGCTABLE( AG_CONST_OBJECT((n),"AG_Widget:AG_Table:*") )
-#define AG_CONST_TABLE_NAMED(n) AGCTABLE( AG_CONST_OBJECT_NAMED((n),"AG_Widget:AG_Table:*") )
+#define   AGTABLE(p)        ((AG_Table *)(p))
+#define  AGcTABLE(p)        ((const AG_Table *)(p))
+#define  AG_TABLE_ISA(o)    (((AGOBJECT(o)->cid & 0xff000000) >> 24) == 0x25)
+#define  AG_TABLE_SELF()    AGTABLE(  AG_OBJECT(0,        "AG_Widget:AG_Table:*") )
+#define  AG_TABLE_PTR(n)    AGTABLE(  AG_OBJECT((n),      "AG_Widget:AG_Table:*") )
+#define  AG_TABLE_NAMED(n)  AGTABLE(  AG_OBJECT_NAMED((n),"AG_Widget:AG_Table:*") )
+#define AG_cTABLE_SELF()   AGcTABLE( AG_cOBJECT(0,        "AG_Widget:AG_Table:*") )
+#define AG_cTABLE_PTR(n)   AGcTABLE( AG_cOBJECT((n),      "AG_Widget:AG_Table:*") )
+#define AG_cTABLE_NAMED(n) AGcTABLE( AG_cOBJECT_NAMED((n),"AG_Widget:AG_Table:*") )
 
 /* Begin generated block */
 __BEGIN_DECLS
@@ -249,8 +250,7 @@ extern DECLSPEC void AG_TableDeselectCol(AG_Table *_Nonnull, int);
 extern DECLSPEC int AG_TableSaveASCII(AG_Table *_Nonnull, void *_Nonnull, char);
 #ifdef AG_LEGACY
 #define AG_TablePrescale(c,r) AG_TableSizeHint((c),(r))
-#define AG_TableSetSelectionColor(t,r,g,b,a) { AG_WCOLOR((t),AG_SELECTION_COLOR).r = r; AG_WCOLOR((t),AG_SELECTION_COLOR).g = g; AG_WCOLOR((t),AG_SELECTION_COLOR).b = b; AG_WCOLOR((t),AG_SELECTION_COLOR).a = a; AG_Verbose("AG_TableSetSelectionColor() is deprecated\n"); }
-#endif 
+#endif
 __END_DECLS
 /* Close generated block */
 
