@@ -3,8 +3,9 @@
 #ifndef _AGAR_GUI_TEXT_H_
 #define _AGAR_GUI_TEXT_H_
 
-#include <agar/gui/surface.h>
 #include <agar/gui/drv.h>
+#include <agar/gui/font.h>
+
 #include <agar/gui/begin.h>
 
 /* Maximum height of AG_{Push,Pop}TextState() stack. */
@@ -17,11 +18,6 @@
 #define AG_TEXT_FONTSPEC_MAX AG_MODEL
 #endif
 
-/* Bucket size of glyph cache table used by AG_TextRenderGlyph(). */
-#ifndef AG_GLYPH_NBUCKETS
-#define AG_GLYPH_NBUCKETS (AG_MODEL * 8)
-#endif
-
 /*
  * Maximum length of ANSI control sequences. Longer sequences are ignored.
  * Note: Increasing this impacts the performance of backward ansi-scanning
@@ -36,10 +32,6 @@
 #define AG_TEXT_ANSI_PARAM_MAX 32
 #endif
 
-struct ag_window;
-struct ag_button;
-struct ag_font;
-
 /* Way to justify text horizontally in a space. */
 enum ag_text_justify {
 	AG_TEXT_LEFT,
@@ -61,25 +53,7 @@ enum ag_text_msg_title {
 	AG_MSG_INFO
 };
 
-/* Font engine classes. */
-enum ag_font_type {
-	AG_FONT_VECTOR,				/* Vectorial font */
-	AG_FONT_BITMAP,				/* Raw glyph pixmaps */
-	AG_FONT_DUMMY,				/* No font engine */
-	AG_FONT_TYPE_LAST
-};
-
-/* Data source from which to load a font. */
-enum ag_font_spec_source {
-	AG_FONT_SOURCE_FILE,			/* Load from file */
-	AG_FONT_SOURCE_MEMORY			/* Read from memory */
-};
-
-#ifndef AG_FONT_PTS_EPSILON
-#define AG_FONT_PTS_EPSILON 0.01
-#endif
-
-/* ANSI 3- and 4-bit colors */
+/* ANSI 3-bit and 4-bit colors. */
 enum ag_ansi_color {
 	AG_ANSI_BLACK,		/* fg=30 bg=40 */
 	AG_ANSI_RED,
@@ -100,7 +74,7 @@ enum ag_ansi_color {
 	AG_ANSI_COLOR_LAST
 };
 
-/* ANSI escape codes */
+/* Raw ANSI escape codes. */
 enum ag_text_ansi_control {
 	AG_ANSI_SS2,			/* G2 character set (xterm) */
 	AG_ANSI_SS3,			/* G3 character set (xterm) */
@@ -187,13 +161,24 @@ enum ag_text_sgr_parameter {
 	AG_SGR_NOT_OVERLINED   = 55,	/* Not overlined */
 					/* (56-59 unused) */
 	AG_SGR_NO_FG_NO_BG     = 56,	/* Both NO_FG and NO_BG (internal) */
-	AG_SGR_IDEOGRAM_1      = 60,
-	AG_SGR_IDEOGRAM_2      = 61,
-	AG_SGR_IDEOGRAM_3      = 62,
-	AG_SGR_IDEOGRAM_4      = 63,
-	AG_SGR_IDEOGRAM_5      = 64,
-	AG_SGR_IDEOGRAM_6      = 65,
-					/* (66-89 unused) */
+	AG_SGR_IDEOGRAM_1      = 60,    /* Ideogram underline or right side line */
+	AG_SGR_IDEOGRAM_2      = 61,    /* Ideogram double underline, or
+	                                   double line on the right side */
+	AG_SGR_IDEOGRAM_3      = 62,    /* Ideogram overline or left side line */
+	AG_SGR_IDEOGRAM_4      = 63,    /* Ideogram double overline, or
+	                                   double line on the left side */
+	AG_SGR_IDEOGRAM_5      = 64,    /* Ideogram stress marking */
+	AG_SGR_IDEOGRAM_6      = 65,    /* No Ideogram attributes */
+	AG_SGR_ALT_FONT_11     = 66,    /* Switch to alt font #11 (Agar ext) */
+	AG_SGR_ALT_FONT_12     = 67,    /* Switch to alt font #12 (Agar ext) */
+	AG_SGR_ALT_FONT_13     = 68,    /* Switch to alt font #13 (Agar ext) */
+	AG_SGR_ALT_FONT_14     = 69,    /* Switch to alt font #14 (Agar ext) */
+	AG_SGR_ALT_FONT_15     = 70,    /* Switch to alt font #15 (Agar ext) */
+	AG_SGR_ALT_FONT_16     = 71,    /* Switch to alt font #16 (Agar ext) */
+	AG_SGR_ALT_FONT_17     = 72,    /* Switch to alt font #17 (Agar ext) */
+	AG_SGR_SUPERSCRIPT     = 73,    /* Superscript */
+	AG_SGR_SUBSCRIPT       = 74,    /* Subscript */
+	AG_SGR_NO_SUPSUBSCRIPT = 75,    /* No superscript or subscript */
 	AG_SGR_BRIGHT_FG_1     = 90,
 	AG_SGR_BRIGHT_FG_2     = 91,
 	AG_SGR_BRIGHT_FG_3     = 92,
@@ -223,98 +208,9 @@ typedef struct ag_text_ansi {
 	AG_Color color;	                  /* For BG_COLOR / FG_COLOR */
 } AG_TextANSI;
 
-/* Agar font specification. */
-typedef struct ag_font_spec {
-	float size;				/* Font size in points */
-	int index;				/* Font index (FC_INDEX) */
-	enum ag_font_type type;			/* Font engine type */
-	enum ag_font_spec_source sourceType;	/* Source type */
-	struct {				/* Transformation matrix */
-		double xx, xy;
-		double yx, yy;
-	} matrix;
-	union {
-		/* char *file; */
-		struct {
-			const Uint8 *_Nonnull data;  /* Source memory region */
-			AG_Size size;                /* Size in bytes */
-#if AG_MODEL == AG_MEDIUM
-			Uint32 _pad;
-#endif
-		} mem;
-	} source;
-} AG_FontSpec;
-
-typedef struct ag_font_adjustment {
-	const char *face;               /* Font family */
-	Uint flags;			/* AG_FONT_BOLD = Regular is bold */
-	float size_factor;              /* Scaling tweak */
-	int ascent_offset[6];           /* Ascent tweak (size range specific) */
-} AG_FontAdjustment;
-
-/* A cached rendering of a glyph of a given font and color. */
-typedef struct ag_glyph {
-	struct ag_font *_Nonnull font;   /* Font face */
-	AG_Color colorBG;                /* Background color */
-	AG_Color color;                  /* Foreground color */
-	AG_Surface *_Nonnull su;         /* Rendered surface */
-	AG_Char ch;                      /* Native character */
-	int advance;                     /* Advance (px) */
-	Uint texture;                    /* Mapped texture (driver-specific) */
-	AG_TexCoord texcoords;           /* Mapped texture coordinates */
-	Uint32 _pad1;
-	AG_SLIST_ENTRY(ag_glyph) glyphs; /* Entry in glyph cache */
-} AG_Glyph;
-
-/* Loaded font */
-typedef struct ag_font {
-	struct ag_object obj;           /* AG_Object -> AG_Font */
-	AG_FontSpec spec;               /* Input specification */
-	Uint flags;
-#define AG_FONT_BOLD           0x001    /* Bold style */
-#define AG_FONT_ITALIC         0x002    /* Italic style */
-#define AG_FONT_UNDERLINE      0x004    /* Generate underline */
-#define AG_FONT_UPPERCASE      0x008    /* Force uppercase */
-#define AG_FONT_OBLIQUE        0x010    /* Oblique style */
-#define AG_FONT_UPRIGHT_ITALIC 0x020    /* Upright italic style */
-#define AG_FONT_SEMICONDENSED  0x040    /* Semi-condensed */
-#define AG_FONT_CONDENSED      0x080    /* Condensed */
-#define AG_FONT_SW_BOLD        0x100    /* Software-generated bold */
-#define AG_FONT_SW_OBLIQUE     0x200    /* Software-generated oblique */
-#define AG_FONT_MONOSPACE      0x400	/* Monospace variant */
-#define AG_FONT_SW_ITALIC      AG_FONT_SW_OBLIQUE
-
-#define AG_FONT_WEIGHTS     (AG_FONT_BOLD | AG_FONT_SW_BOLD)
-#define AG_FONT_STYLES (AG_FONT_ITALIC | AG_FONT_SW_ITALIC | AG_FONT_OBLIQUE | AG_FONT_UPRIGHT_ITALIC) 
-#define AG_FONT_WD_VARIANTS (AG_FONT_SEMICONDENSED | AG_FONT_CONDENSED)
-
-	int height;                        /* Body size in pixels */
-	int ascent;                        /* Ascent (relative to baseline) */
-	int descent;                       /* Descent (relative to baseline) */
-	int lineskip;                      /* Multiline y-increment */
-	Uint nRefs;                        /* Global reference count */
-	AG_TAILQ_ENTRY(ag_font) fonts;     /* Entry in global fonts list */
-	union {
-		struct {
-			void *_Nonnull ttf;      /* TTF object (AG_TTFFont) */
-		} vec;
-		struct {
-			char *spec;                               /* "MAP:x-y" label */
-			AG_Surface *_Nullable *_Nullable glyphs;  /* Glyph surfaces */
-			Uint                            nGlyphs;  /* Glyph count */
-			AG_Char c0, c1;                           /* Character mapping (range) */
-			Uint32 _pad;
-		} bmp;
-	} data;
-} AG_Font;
-
-typedef AG_TAILQ_HEAD(ag_fontq, ag_font) AG_FontQ;
-
-#define AG_FONT_BITMAP_SPEC_MAX 28
-
 /*
  * An element of the rendering attribute stack.
- * SYNC with CompareTextStates() in gui/text_cache.c.
+ * Sync with CompareTextStates() in gui/text_cache.c.
  */
 typedef struct ag_text_state {
 	AG_Font *_Nonnull font;		/* Font face */
@@ -324,21 +220,9 @@ typedef struct ag_text_state {
 	enum ag_text_justify justify;	/* Justification mode */
 	enum ag_text_valign valign;	/* Vertical alignment mode */
 	int tabWd;			/* Width of \t in pixels */
-#ifdef AG_DEBUG
-	char name[8];			/* Tag for debugging */
-#endif
 	Uint32 _pad;
 } AG_TextState;
 
-/* Description of font stored in data segment. */
-typedef struct ag_static_font {
-	const char *_Nonnull name;	/* Identifier */
-	enum ag_font_type type;		/* Type of font */
-	Uint32 size;			/* Size in bytes */
-	const Uint8 *_Nonnull data;	/* Font data */
-	AG_Font *_Nullable font;	/* Initialized font */
-} AG_StaticFont;
-
 /* Measures of rendered text. */
 typedef struct ag_text_metrics {
 	int w, h;			/* Dimensions in pixels */
@@ -347,11 +231,6 @@ typedef struct ag_text_metrics {
 	Uint32 _pad;
 } AG_TextMetrics;
 
-/* Cache of pre-rendered glyphs */
-typedef struct ag_glyph_cache {
-	AG_SLIST_HEAD_(ag_glyph) glyphs;
-} AG_GlyphCache;
-
 #ifdef AG_DEBUG
 # define AG_TEXT_STATE_CUR() (((agTextStateCur >= 0 && agTextStateCur < AG_TEXT_STATES_MAX)) ? &agTextStateStack[agTextStateCur] : (AG_TextState *)AG_GenericMismatch("AG_TEXT_STATE"))
 #else
@@ -360,22 +239,17 @@ typedef struct ag_glyph_cache {
 
 /* Begin generated block */
 __BEGIN_DECLS
-extern DECLSPEC AG_ObjectClass agFontClass;
-extern DECLSPEC AG_StaticFont *_Nonnull agBuiltinFonts[];
-extern DECLSPEC int agTextFontHeight;
-extern DECLSPEC int agTextFontAscent;
-extern DECLSPEC int agTextFontDescent;
-extern DECLSPEC int agTextFontLineSkip;
-extern DECLSPEC int agFreetypeInited;
+extern DECLSPEC int agTextFontHeight; 
+extern DECLSPEC int agTextFontAscent; 
+extern DECLSPEC int agTextFontLineSkip; 
+extern DECLSPEC int agFontconfigInited; 
 extern DECLSPEC _Nonnull_Mutex AG_Mutex agTextLock;
 extern DECLSPEC AG_FontQ agFontCache;
 extern DECLSPEC AG_TextState agTextStateStack[AG_TEXT_STATES_MAX];
-extern DECLSPEC Uint agTextStateCur;
+extern DECLSPEC int agTextStateCur;
 extern DECLSPEC AG_Font *_Nullable agDefaultFont;
 extern DECLSPEC const AG_FontAdjustment agFontAdjustments[];
 extern DECLSPEC const char *agCoreFonts[];
-extern DECLSPEC const char *agFontFileExts[];
-extern DECLSPEC const char *agFontTypeNames[];
 extern DECLSPEC const char *agTextMsgTitles[];
 extern DECLSPEC const char *agTextJustifyNames[];
 extern DECLSPEC const char *agTextValignNames[];
@@ -386,8 +260,6 @@ extern DECLSPEC AG_Font *_Nullable AG_TextFontLookup(const char *_Nullable, floa
 extern DECLSPEC AG_Font *_Nullable AG_TextFontPts(float);
 extern DECLSPEC AG_Font *_Nullable AG_TextFontPct(int);
 extern DECLSPEC AG_Font *_Nullable AG_TextFontPctFlags(int, Uint);
-extern DECLSPEC AG_Font *_Nullable AG_FetchFont(const char *_Nullable, float, Uint) _Warn_Unused_Result;
-extern DECLSPEC void AG_UnusedFont(AG_Font *_Nonnull);
 extern DECLSPEC void AG_PopTextState(void);
 extern DECLSPEC void AG_TextClearGlyphCache(AG_Driver *_Nonnull);
 extern DECLSPEC void AG_TextSize(const char *_Nullable, int *_Nullable, int *_Nullable);
@@ -410,16 +282,17 @@ extern DECLSPEC void AG_TextWarning(const char *_Nonnull, const char *_Nonnull,
 extern DECLSPEC void AG_TextErrorS(const char *_Nonnull);
 extern DECLSPEC void AG_TextError(const char *_Nonnull, ...) FORMAT_ATTRIBUTE(printf,1,2);
 extern DECLSPEC void AG_TextAlign(int *_Nonnull, int *_Nonnull, int,int, int,int, int,int,int, int, enum ag_text_justify, enum ag_text_valign);
-extern DECLSPEC int AG_TextJustifyOffset(int, int) _Pure_Attribute;
-extern DECLSPEC int AG_TextValignOffset(int, int) _Pure_Attribute;
 extern DECLSPEC AG_Surface *_Nonnull AG_TextRenderF(const char *_Nonnull, ...) _Warn_Unused_Result;
 extern DECLSPEC AG_Surface *_Nonnull AG_TextRender(const char *_Nonnull) _Warn_Unused_Result;
+extern DECLSPEC AG_Surface *_Nonnull AG_TextRenderRTL(const char *_Nonnull) _Warn_Unused_Result;
+extern DECLSPEC AG_Surface *_Nonnull AG_TextRenderCropped(const char *_Nonnull) _Warn_Unused_Result;
 extern DECLSPEC AG_Surface *_Nonnull AG_TextRenderInternal(const AG_Char *_Nonnull, AG_Font *_Nonnull, const AG_Color *_Nonnull, const AG_Color *_Nonnull) _Warn_Unused_Result;
 #ifdef AG_UNICODE
 # define AG_TextRenderUCS4(s) AG_TextRenderInternal((s), AG_TEXT_STATE_CUR()->font, &AG_TEXT_STATE_CUR()->colorBG, &AG_TEXT_STATE_CUR()->color)
 #endif
 extern DECLSPEC AG_Glyph *_Nonnull AG_TextRenderGlyph(AG_Driver *_Nonnull, AG_Font *_Nonnull, const AG_Color *_Nonnull, const AG_Color *_Nonnull, AG_Char) _Warn_Unused_Result;
 extern DECLSPEC int AG_TextParseANSI(const AG_TextState *_Nonnull, AG_TextANSI *_Nonnull, const AG_Char *_Nonnull);
+extern DECLSPEC int AG_TextExportUnicode_StripANSI(const char *, char *, const AG_Char *, AG_Size);
 extern DECLSPEC void AG_TextColor(const AG_Color *_Nonnull);
 extern DECLSPEC void AG_TextColorRGB(Uint8, Uint8, Uint8);
 extern DECLSPEC void AG_TextColorRGBA(Uint8, Uint8, Uint8, Uint8);
@@ -432,9 +305,8 @@ extern DECLSPEC void AG_TextFont(AG_Font *_Nonnull);
 extern DECLSPEC void AG_TextJustify(enum ag_text_justify);
 extern DECLSPEC void AG_TextValign(enum ag_text_valign);
 extern DECLSPEC void AG_TextTabWidth(int);
-extern DECLSPEC AG_Font *AG_SetDefaultFont(AG_Font *_Nonnull);
+extern DECLSPEC AG_Font *AG_SetDefaultFont(AG_Font *_Nullable);
 extern DECLSPEC void AG_TextParseFontSpec(const char *_Nullable);
-extern DECLSPEC struct ag_window *_Nonnull AG_TextPromptOptions(struct ag_button *_Nonnull *_Nonnull , Uint, const char *_Nonnull, ...);
 extern DECLSPEC int AG_InitTextSubsystem(void);
 extern DECLSPEC void AG_DestroyTextSubsystem(void);
 /*
@@ -539,9 +411,6 @@ AG_CharIsPunct(AG_Char c)
 }
 #define AG_CharIsSpaceOrPunct(ch) (AG_CharIsSpace(ch) || AG_CharIsPunct(ch))
 #define AG_CharIsSpaceOrLF(ch) (AG_CharIsSpace(ch) || (ch == '\n'))
-#ifdef AG_LEGACY
-# define AG_TextRenderf AG_TextRenderF
-#endif
 __END_DECLS
 /* Close generated block */
 
