@@ -67,28 +67,26 @@ typedef struct ag_treetbl_row {
 typedef struct ag_treetbl {
 	struct ag_widget _inherit;	/* AG_Widget -> AG_Treetbl */
 	Uint flags;
-#define AG_TREETBL_MULTI	0x001	/* Allow multiple selections */
-#define AG_TREETBL_MULTITOGGLE  0x002
-#define AG_TREETBL_REORDERCOLS	0x004	/* Allow column reordering */
-#define AG_TREETBL_NODUPCHECKS	0x008	/* Skip checks for duplicate IDs */
-#define AG_TREETBL_SORT		0x010	/* Enable sorting */
-#define AG_TREETBL_POLLED	0x020	/* Polling mode */
-#define AG_TREETBL_HFILL	0x040
-#define AG_TREETBL_VFILL	0x080
-#define AG_TREETBL_EXPAND	(AG_TREETBL_HFILL|AG_TREETBL_VFILL)
+#define AG_TREETBL_MULTI       0x001	/* Allow multiple selections */
+#define AG_TREETBL_MULTITOGGLE 0x002
+#define AG_TREETBL_REORDERCOLS 0x004	/* Allow column reordering */
+#define AG_TREETBL_NODUPCHECKS 0x008	/* Skip checks for duplicate IDs */
+#define AG_TREETBL_SORT        0x010	/* Enable sorting */
+#define AG_TREETBL_POLLED      0x020	/* Polling mode */
+#define AG_TREETBL_HFILL       0x040
+#define AG_TREETBL_VFILL       0x080
+#define AG_TREETBL_EXPAND      (AG_TREETBL_HFILL | AG_TREETBL_VFILL)
 
 	AG_Rect r;			/* View area */
 	int xOffs;			/* Horizontal display offset */
 	int xMax;
 	int yOffs;
 	int yMax;
-
 	int hCol;			/* Header height */
 	int hRow;			/* Per-row height */
 	int dblClicked;			/* Used by double click */
 	int wHint, hHint;		/* Size hint */
 	AG_Timer toDblClick;
-
 	Uint n;					/* Column count */
 	enum ag_treetbl_sort_mode sortMode;	/* Sorting mode */
 	AG_TreetblCol *_Nullable column;	/* Column array */
@@ -117,14 +115,15 @@ typedef struct ag_treetbl {
 	} visible;
 } AG_Treetbl;
 
-#define AGTREETBL(obj)            ((AG_Treetbl *)(obj))
-#define AGCTREETBL(obj)           ((const AG_Treetbl *)(obj))
-#define AG_TREETBL_SELF()          AGTREETBL( AG_OBJECT(0,"AG_Widget:AG_Treetbl:*") )
-#define AG_TREETBL_PTR(n)          AGTREETBL( AG_OBJECT((n),"AG_Widget:AG_Treetbl:*") )
-#define AG_TREETBL_NAMED(n)        AGTREETBL( AG_OBJECT_NAMED((n),"AG_Widget:AG_Treetbl:*") )
-#define AG_CONST_TREETBL_SELF()   AGCTREETBL( AG_CONST_OBJECT(0,"AG_Widget:AG_Treetbl:*") )
-#define AG_CONST_TREETBL_PTR(n)   AGCTREETBL( AG_CONST_OBJECT((n),"AG_Widget:AG_Treetbl:*") )
-#define AG_CONST_TREETBL_NAMED(n) AGCTREETBL( AG_CONST_OBJECT_NAMED((n),"AG_Widget:AG_Treetbl:*") )
+#define   AGTREETBL(o)        ((AG_Treetbl *)(o))
+#define  AGcTREETBL(o)        ((const AG_Treetbl *)(o))
+#define  AG_TREETBL_ISA(o)    (((AGOBJECT(o)->cid & 0xff000000) >> 24) == 0x26)
+#define  AG_TREETBL_SELF()    AGTREETBL(  AG_OBJECT(0,        "AG_Widget:AG_Treetbl:*") )
+#define  AG_TREETBL_PTR(n)    AGTREETBL(  AG_OBJECT((n),      "AG_Widget:AG_Treetbl:*") )
+#define  AG_TREETBL_NAMED(n)  AGTREETBL(  AG_OBJECT_NAMED((n),"AG_Widget:AG_Treetbl:*") )
+#define AG_cTREETBL_SELF()   AGcTREETBL( AG_cOBJECT(0,        "AG_Widget:AG_Treetbl:*") )
+#define AG_cTREETBL_PTR(n)   AGcTREETBL( AG_cOBJECT((n),      "AG_Widget:AG_Treetbl:*") )
+#define AG_cTREETBL_NAMED(n) AGcTREETBL( AG_cOBJECT_NAMED((n),"AG_Widget:AG_Treetbl:*") )
 
 /* Begin generated block */
 __BEGIN_DECLS
