@@ -6,26 +6,19 @@
 #include <agar/gui/begin.h>
 
 typedef struct ag_unit {
-	char *_Nullable key;		/* Key (or list terminator) */
-	char *_Nonnull abbr;		/* Abbreviated symbol */
-	char *_Nonnull name;		/* Long name */
-	double divider;			/* Conversion factor (linear) */
+	char *_Nullable key;                 /* Key (or list terminator) */
+	char *_Nonnull abbr;                 /* Abbreviated symbol */
+	char *_Nonnull name;                 /* Long name */
+	double divider;                      /* Conversion factor (linear units) */
+	double (*_Nullable fn)(double, int); /* Function (non-linear units) */
 } AG_Unit;
 
-typedef struct ag_unit_nl {
-	AG_Unit unit;						/* Inherit */
-	double (*_Nonnull func)(double, int);
-} AG_UnitNL;
-
 #define AG_UNIT(unit)    ((AG_Unit *)(unit))
-#define AG_UNIT_NL(unit) ((AG_UnitNL *)(unit))
 
 /* Begin generated block */
 __BEGIN_DECLS
-extern DECLSPEC double AG_UnitFahrenheit(double, int);
-extern DECLSPEC double AG_UnitCelsius(double, int);
-extern DECLSPEC const AG_Unit *_Nonnull agUnitGroups[];
 extern DECLSPEC const char *_Nullable agUnitGroupNames[];
+extern DECLSPEC const AG_Unit *_Nonnull agUnitGroups[];
 extern DECLSPEC const int agnUnitGroups;
 extern DECLSPEC const AG_Unit agIdentityUnit[];
 extern DECLSPEC const AG_Unit agLengthUnits[];
@@ -38,7 +31,7 @@ extern DECLSPEC const AG_Unit agSpeedUnits[];
 extern DECLSPEC const AG_Unit agMassUnits[];
 extern DECLSPEC const AG_Unit agTimeUnits[];
 extern DECLSPEC const AG_Unit agCurrentUnits[];
-extern DECLSPEC const AG_UnitNL agTemperatureUnits[];
+extern DECLSPEC const AG_Unit agTemperatureUnits[];
 extern DECLSPEC const AG_Unit agSubstanceAmountUnits[];
 extern DECLSPEC const AG_Unit agEnergyPerSubstanceAmountUnits[];
 extern DECLSPEC const AG_Unit agMolarHeatCapacityUnits[];
@@ -57,7 +50,8 @@ extern DECLSPEC const AG_Unit agResistivityUnits[];
 extern DECLSPEC const AG_Unit agThermalConductivityUnits[];
 extern DECLSPEC const AG_Unit agThermalExpansionUnits[];
 extern DECLSPEC const AG_Unit agDensityUnits[];
-extern DECLSPEC int AG_UnitIsNonlinear(const char *_Nonnull) _Const_Attribute;
+extern DECLSPEC double AG_UnitFahrenheit(double, int) _Const_Attribute;
+extern DECLSPEC double AG_UnitCelsius(double, int) _Const_Attribute;
 extern DECLSPEC double AG_Unit2Base(double, const AG_Unit *_Nonnull) _Pure_Attribute;
 extern DECLSPEC double AG_Base2Unit(double, const AG_Unit *_Nonnull) _Pure_Attribute;
 extern DECLSPEC double AG_Unit2Unit(double, const AG_Unit *_Nonnull, const AG_Unit *_Nonnull) _Pure_Attribute;
