@@ -29,7 +29,7 @@ struct ag_font;
  * container widgets for a preferred initial geometry in pixels.
  */
 typedef struct ag_size_req {
-	int w, h;			/* Requested geometry in pixels */
+	int w, h;                            /* Requested size (px) */
 } AG_SizeReq;
 
 /*
@@ -37,23 +37,26 @@ typedef struct ag_size_req {
  * to their child widgets their final allocated position and size in pixels.
  */
 typedef struct ag_size_alloc {
-	int w, h;			/* Allocated geometry in pixels */
-	int x, y;			/* Allocated position in pixels */
+	int w, h;                            /* Widget size (px) */
+	int x, y;                            /* Position in parent (px) */
 } AG_SizeAlloc;
 
 /* Widget class description */
 typedef struct ag_widget_class {
 	struct ag_object_class _inherit;     /* [AG_Object] -> [AG_Widget] */
 
-	/* Rendering routine (rendering context) */
 	void (*_Nullable draw)(void *_Nonnull);
-	
-	/* Size requisition (indicate preferred initial size to parent) */
 	void (*_Nullable size_request)(void *_Nonnull, AG_SizeReq *_Nonnull);
-
-	/* Size allocation (handle final size returned by parent) */
 	int  (*_Nullable size_allocate)(void *_Nonnull,
 	                                const AG_SizeAlloc *_Nonnull);
+	void (*_Nullable mouse_button_down)(void *_Nonnull, AG_MouseButton, int,int);
+	void (*_Nullable mouse_button_up)(void *_Nonnull, AG_MouseButton, int,int);
+	void (*_Nullable mouse_motion)(void *_Nonnull, int,int, int,int);
+	void (*_Nullable key_down)(void *_Nonnull, AG_KeySym, AG_KeyMod, AG_Char);
+	void (*_Nullable key_up)(void *_Nonnull, AG_KeySym, AG_KeyMod, AG_Char);
+	void (*_Nullable touch)(void *_Nonnull, void *_Nonnull, const AG_DriverEvent *_Nonnull);
+	void (*_Nullable ctrl)(void *_Nonnull, void *_Nonnull, const AG_DriverEvent *_Nonnull);
+	void (*_Nullable joy)(void *_Nonnull, void *_Nonnull, const AG_DriverEvent *_Nonnull);
 } AG_WidgetClass;
 
 /* A constant size for some visual element (possibly a relative to a parent) */
@@ -65,18 +68,6 @@ typedef enum ag_widget_sizespec {
 	AG_WIDGET_FILL         /* Expand to fill remaining space ("-") */
 } AG_SizeSpec;
 
-/* Bit flag description (used by AG_Checkbox(3) for example). */
-typedef struct ag_flag_descr {
-#ifdef AG_HAVE_64BIT
-	Uint64 bitmask;			/* Bitmask */
-#else
-	Uint bitmask;			/* Bitmask */
-#endif
-	const char *_Nonnull descr;	/* Description (UTF-8) */
-	int writeable;			/* User-editable */
-	Uint32 _pad;
-} AG_FlagDescr;
-
 /* 
  * High-level widget action.
  */
@@ -233,13 +224,13 @@ typedef struct ag_widget {
 #define AG_WIDGET_HIDE                  0x00000200 /* Don't display */
 #define AG_WIDGET_DISABLED              0x00000400 /* Inactive state */
 #define AG_WIDGET_MOUSEOVER             0x00000800 /* Cursor is hovering (RO) */
-#define AG_WIDGET_CATCH_TAB             0x00001000 /* Receive focus-cycling key events */
+#define AG_WIDGET_CATCH_TAB             0x00001000 /* Inhibit TAB key focus-cycling */
 #define AG_WIDGET_GL_RESHAPE            0x00002000 /* Pending GL view reshape */
 #define AG_WIDGET_UNDERSIZE             0x00004000 /* Allocation could not be met (RO) */
-                                     /* 0x00008000 */
+#define AG_WIDGET_DISABLE_ON_ATTACH     0x00008000 /* Set attached widgets DISABLED */
 #define AG_WIDGET_UNFOCUSED_KEYDOWN     0x00010000 /* Receive keydowns w/o focus */
 #define AG_WIDGET_UNFOCUSED_KEYUP       0x00020000 /* Receive keyups w/o focus */
-                                     /* 0x00040000 */
+#define AG_WIDGET_CATCH_SHOULDER        0x00040000 /* Inhibit controller-driven focus-cycling */
                                      /* 0x00080000 */
 #define AG_WIDGET_UPDATE_WINDOW         0x00100000 /* Request WindowUpdate() ASAP */
 #define AG_WIDGET_QUEUE_SURFACE_BACKUP  0x00200000 /* Software-backup surfaces now */
@@ -273,10 +264,10 @@ typedef struct ag_widget {
 	                                    /* (TODO use a Variable) */
 
 	enum ag_widget_state state;         /* Style-effecting state */
-	Uint8 marginTop, marginRight;       /* Margin (px outside of border) */
-	Uint8 marginBottom, marginLeft;
-	Uint paddingTop, paddingRight;      /* Padding (px around contents) */
-	Uint paddingBottom, paddingLeft;
+	int marginTop, marginRight;         /* Margin (px outside of border) */
+	int marginBottom, marginLeft;
+	int paddingTop, paddingRight;       /* Padding (px around contents) */
+	int paddingBottom, paddingLeft;
 	Uint spacingHoriz, spacingVert;     /* Spacing (px between items) */
 
 	struct ag_font *_Nullable font;    /* Active font (style-generated) */
@@ -290,17 +281,18 @@ typedef struct ag_widget {
 
 typedef AG_VEC_HEAD(AG_Widget *) AG_WidgetVec;
 
-#define AGWIDGET(p)             ((AG_Widget *)(p))
-#define AGCWIDGET(p)            ((const AG_Widget *)(p))
-#define AG_WIDGET_SELF()          AGWIDGET( AG_OBJECT(0,"AG_Widget:*") )
-#define AG_WIDGET_PTR(n)          AGWIDGET( AG_OBJECT((n),"AG_Widget:*") )
-#define AG_WIDGET_NAMED(n)        AGWIDGET( AG_OBJECT_NAMED((n),"AG_Widget:*") )
-#define AG_CONST_WIDGET_SELF()   AGCWIDGET( AG_CONST_OBJECT(0,"AG_Widget:*") )
-#define AG_CONST_WIDGET_PTR(n)   AGCWIDGET( AG_CONST_OBJECT((n),"AG_Widget:*") )
-#define AG_CONST_WIDGET_NAMED(n) AGCWIDGET( AG_CONST_OBJECT((n),"AG_Widget:*") )
+#define   AGWIDGET(o)     ((AG_Widget *)(o))
+#define  AGcWIDGET(o)     ((const AG_Widget *)(o))
+#define AG_WIDGET_ISA(o) (((AGOBJECT(o)->cid & 0xff000000) >> 24) >= 0x08 && ((AGOBJECT(o)->cid & 0xff000000) >> 24) <= 0x35)
+#define  AG_WIDGET_SELF()    AGWIDGET(  AG_OBJECT(0,         "AG_Widget:*") )
+#define  AG_WIDGET_PTR(n)    AGWIDGET(  AG_OBJECT((n),       "AG_Widget:*") )
+#define  AG_WIDGET_NAMED(n)  AGWIDGET(  AG_OBJECT_NAMED((n), "AG_Widget:*") )
+#define AG_cWIDGET_SELF()   AGcWIDGET( AG_cOBJECT(0,         "AG_Widget:*") )
+#define AG_cWIDGET_PTR(n)   AGcWIDGET( AG_cOBJECT((n),       "AG_Widget:*") )
+#define AG_cWIDGET_NAMED(n) AGcWIDGET( AG_cOBJECT((n),       "AG_Widget:*") )
 
-#define AGWIDGET_OPS(wi)          ((AG_WidgetClass *)AGOBJECT(wi)->cls)
-#define AGWIDGET_SUPER_OPS(wi)    ((AG_WidgetClass *)AGOBJECT(wi)->cls->super)
+#define AGWIDGET_OPS(wi)       ((AG_WidgetClass *)AGOBJECT(wi)->cls)
+#define AGWIDGET_SUPER_OPS(wi) ((AG_WidgetClass *)AGOBJECT(wi)->cls->super)
 
 #define AGWIDGET_SURFACE(wi, ind) AGWIDGET(wi)->surfaces[ind]
 #define AGWIDGET_TEXTURE(wi, ind) AGWIDGET(wi)->textures[ind]
@@ -382,7 +374,7 @@ extern DECLSPEC void AG_WidgetShowAll(void *_Nonnull);
 extern DECLSPEC void AG_WidgetHideAll(void *_Nonnull);
 extern DECLSPEC AG_Surface *_Nullable AG_WidgetSurface(void *_Nonnull);
 extern DECLSPEC void AG_RedrawOnChange(void *_Nonnull, int, const char *_Nonnull);
-extern DECLSPEC void AG_RedrawOnTick(void *_Nonnull, int);
+extern DECLSPEC int AG_RedrawOnTick(void *_Nonnull, int);
 extern DECLSPEC void AG_WidgetStdKeyDown(AG_Event *_Nonnull);
 extern DECLSPEC void AG_WidgetStdKeyUp(AG_Event *_Nonnull);
 extern DECLSPEC void AG_WidgetStdMouseButtonDown(AG_Event *_Nonnull);
@@ -403,12 +395,53 @@ extern DECLSPEC int AG_ExecKeyAction(void *_Nonnull, AG_ActionEventType, AG_KeyS
 extern DECLSPEC int AG_ExecAction(void *_Nonnull, const AG_Action *_Nonnull);
 extern DECLSPEC void AG_WidgetEnable(void *_Nonnull);
 extern DECLSPEC void AG_WidgetDisable(void *_Nonnull);
+extern DECLSPEC void AG_PushDisabledState(void *_Nonnull);
+extern DECLSPEC void AG_PopDisabledState(void *_Nonnull);
 extern DECLSPEC void AG_WidgetCompileStyle(void *_Nonnull);
 extern DECLSPEC void AG_WidgetCopyStyle(void *_Nonnull, void *_Nonnull);
 extern DECLSPEC void AG_WidgetFreeStyle(void *_Nonnull);
 extern DECLSPEC void AG_SetFont(void *_Nonnull, const struct ag_font *_Nonnull);
 extern DECLSPEC void AG_SetStyle(void *_Nonnull, const char *_Nonnull, const char *_Nullable);
 extern DECLSPEC void AG_SetStyleF(void *_Nonnull, const char *_Nonnull, const char *_Nullable, ...) FORMAT_ATTRIBUTE(printf,3,4);
+/*
+ * Shorthands for standard style attributes.
+ */
+#define AG_SetFontFamily(wid,s) AG_SetStyle((wid), "font-family", (s))
+#define AG_SetFontSize(wid,s) AG_SetStyle((wid), "font-size", (s))
+#define AG_SetFontWeight(wid,s) AG_SetStyle((wid), "font-weight", (s))
+#define AG_SetFontStyle(wid,s) AG_SetStyle((wid), "font-style", (s))
+#define AG_SetFontStretch(wid,s) AG_SetStyle((wid), "font-stretch", (s))
+#define AG_SetMargin(wid,s) AG_SetStyle((wid), "margin", (s))
+#define AG_SetPadding(wid,s) AG_SetStyle((wid), "padding", (s))
+#define AG_SetSpacing(wid,s) AG_SetStyle((wid), "spacing", (s))
+#define AG_SetColor(wid,s) AG_SetStyle((wid), "color", (s))
+#define AG_SetColorDisabled(wid,s) AG_SetStyle((wid), "color#disabled", (s))
+#define AG_SetColorFocused(wid,s) AG_SetStyle((wid), "color#focused", (s))
+#define AG_SetColorHover(wid,s) AG_SetStyle((wid), "color#hover", (s))
+#define AG_SetBgColor(wid,s) AG_SetStyle((wid), "background-color", (s))
+#define AG_SetBgColorDisabled(wid,s) AG_SetStyle((wid), "background-color#disabled", (s))
+#define AG_SetBgColorFocused(wid,s) AG_SetStyle((wid), "background-color#focused", (s))
+#define AG_SetBgColorHover(wid,s) AG_SetStyle((wid), "background-color#hover", (s))
+#define AG_SetTextColor(wid,s) AG_SetStyle((wid), "text-color", (s))
+#define AG_SetTextColorDisabled(wid,s) AG_SetStyle((wid), "text-color#disabled", (s))
+#define AG_SetTextColorFocused(wid,s) AG_SetStyle((wid), "text-color#focused", (s))
+#define AG_SetTextColorHover(wid,s) AG_SetStyle((wid), "text-color#hover", (s))
+#define AG_SetLineColor(wid,s) AG_SetStyle((wid), "line-color", (s))
+#define AG_SetLineColorDisabled(wid,s) AG_SetStyle((wid), "line-color#disabled", (s))
+#define AG_SetLineColorFocused(wid,s) AG_SetStyle((wid), "line-color#focused", (s))
+#define AG_SetLineColorHover(wid,s) AG_SetStyle((wid), "line-color#hover", (s))
+#define AG_SetHighColor(wid,s) AG_SetStyle((wid), "high-color", (s))
+#define AG_SetHighColorDisabled(wid,s) AG_SetStyle((wid), "high-color#disabled", (s))
+#define AG_SetHighColorFocused(wid,s) AG_SetStyle((wid), "high-color#focused", (s))
+#define AG_SetHighColorHover(wid,s) AG_SetStyle((wid), "high-color#hover", (s))
+#define AG_SetLowColor(wid,s) AG_SetStyle((wid), "low-color", (s))
+#define AG_SetLowColorDisabled(wid,s) AG_SetStyle((wid), "low-color#disabled", (s))
+#define AG_SetLowColorFocused(wid,s) AG_SetStyle((wid), "low-color#focused", (s))
+#define AG_SetLowColorHover(wid,s) AG_SetStyle((wid), "low-color#hover", (s))
+#define AG_SetSelColor(wid,s) AG_SetStyle((wid), "selection-color", (s))
+#define AG_SetSelColorDisabled(wid,s) AG_SetStyle((wid), "selection-color#disabled", (s))
+#define AG_SetSelColorFocused(wid,s) AG_SetStyle((wid), "selection-color#focused", (s))
+#define AG_SetSelColorHover(wid,s) AG_SetStyle((wid), "selection-color#hover", (s))
 /*
  * Inlinables
  */
@@ -428,6 +461,7 @@ extern DECLSPEC void ag_expand_horiz(void *_Nonnull);
 extern DECLSPEC void ag_expand_vert(void *_Nonnull);
 extern DECLSPEC void ag_widget_update(void *_Nonnull);
 extern DECLSPEC void ag_push_clip_rect(void *_Nonnull, const AG_Rect *_Nonnull);
+extern DECLSPEC void ag_push_clip_rect_inner(void *_Nonnull, const AG_Rect *_Nonnull);
 extern DECLSPEC void ag_pop_clip_rect(void *_Nonnull);
 extern DECLSPEC void ag_push_blending_mode(void *_Nonnull, AG_AlphaFn, AG_AlphaFn);
 extern DECLSPEC void ag_pop_blending_mode(void *_Nonnull);
@@ -452,6 +486,7 @@ extern DECLSPEC void ag_set_mod_state(void *_Nonnull, Uint);
 # define AG_ExpandVert(o) ag_expand_vert(o)
 # define AG_WidgetUpdate(o) ag_widget_update(o)
 # define AG_PushClipRect(o,r) ag_push_clip_rect((o),(r))
+# define AG_PushClipRectInner(o,r) ag_push_clip_rect_inner((o),(r))
 # define AG_PopClipRect(o) ag_pop_clip_rect(o)
 # define AG_PushBlendingMode(o,fs,fd) ag_push_blending_mode((o),(fs),(fd))
 # define AG_PopBlendingMode(o) ag_pop_blending_mode(o)
