@@ -5,7 +5,7 @@
 struct map_view;
 struct map_tool;
 
-struct map_insert_tool {
+typedef struct map_insert_tool {
 	struct map_tool tool;
 	enum rg_snap_mode snap_mode;
 	int replace_mode;
@@ -13,6 +13,6 @@ struct map_insert_tool {
 	Uint32 _pad;
 	struct map mTmp;
 	struct map_view *_Nullable mvTmp;
-};
+} MAP_InsertTool;
 
 #include <agar/map/close.h>
