@@ -4,10 +4,9 @@
 /* Begin generated block */
 __BEGIN_DECLS
 extern DECLSPEC const MAP_ToolOps mapNodeselOps;
-extern DECLSPEC const MAP_ToolOps mapRefselOps;
 extern DECLSPEC const MAP_ToolOps mapFillOps;
 extern DECLSPEC const MAP_ToolOps mapInsertOps;
-extern DECLSPEC const MAP_ToolOps mapGInsertOps;
+extern DECLSPEC const MAP_ToolOps mapInsertObjOps;
 extern DECLSPEC const MAP_ToolOps mapFlipOps;
 extern DECLSPEC const MAP_ToolOps mapInvertOps;
 extern DECLSPEC const MAP_ToolOps mapEraserOps;
