@@ -572,7 +572,7 @@ __END_DECLS
  * Used when computing Markowitz products.
  */
 
-#define spcMarkoProd(product, op1, op2) if (( (op1) > LARGEST_SHORT_INTEGER AND (op2) != 0) OR ( (op2) > LARGEST_SHORT_INTEGER AND (op1) != 0)) { double fProduct = (double)(op1) * (double)(op2); if (fProduct >= LARGEST_LONG_INTEGER) (product) = LARGEST_LONG_INTEGER; else (product) = (long)fProduct; } else (product) = (op1)*(op2);
+#define spcMarkoProd(product, op1, op2) if (( (op1) > LARGEST_SHORT_INTEGER AND (op2) != 0) OR ( (op2) > LARGEST_SHORT_INTEGER AND (op1) != 0)) { double fProduct = (double)(op1) * (double)(op2); if (fProduct >= (double)LARGEST_LONG_INTEGER) (product) = LARGEST_LONG_INTEGER; else (product) = (long)fProduct; } else (product) = (op1)*(op2);
 
 /* Real and complex numbers */
 typedef spREAL RealNumber, *RealVector;
