@@ -2,8 +2,8 @@
 
 /* Begin generated block */
 __BEGIN_DECLS
-extern DECLSPEC M_Sphere M_SphereRead(AG_DataSource *);
-extern DECLSPEC void M_SphereWrite(AG_DataSource *, M_Sphere *);
+extern DECLSPEC M_Sphere M_SphereRead(AG_DataSource *_Nonnull);
+extern DECLSPEC void M_SphereWrite(AG_DataSource *_Nonnull, M_Sphere *_Nonnull);
 extern DECLSPEC M_Sphere M_SphereFromPt(M_Vector3, M_Real);
 extern DECLSPEC M_Real M_SpherePointDistance(M_Sphere, M_Vector3);
 extern DECLSPEC M_Real M_SphereSurfaceArea(M_Sphere);
