@@ -17,9 +17,7 @@ struct ag_menu_item;
 
 /* Begin generated block */
 __BEGIN_DECLS
-extern DECLSPEC void VG_SnapPoint(struct vg_view *, VG_Vector *);
-extern DECLSPEC void VG_DrawGrid(struct vg_view *);
-extern DECLSPEC struct ag_toolbar *VG_SnapToolbar(void *, struct vg_view *, int);
-extern DECLSPEC void VG_SnapMenu(struct ag_menu_item *, struct vg_view *);
+extern DECLSPEC struct ag_toolbar *_Nonnull VG_SnapToolbar(void *_Nullable, struct vg_view *_Nonnull, int);
+extern DECLSPEC void VG_SnapMenu(struct ag_menu_item *_Nonnull, struct vg_view *_Nonnull);
 __END_DECLS
 /* Close generated block */
